#!/bin/bash
#
# Copyright 2016 Cohesity Inc.
#
# Author: Sachin Gaikwad (sachin@cohesity.com)
#
# Description:
# This shell script module contains common utility functions.
# This file assume that following variable is defined
# COHESITY_USER, COHESITY_SKIP_RESTORE_CHECK and COHESITY_SKIP_ORACLE_CHECK

# ----------------------------------------------------------------------------
# We do not use Associate Array (Map) here so that we can support
# bash versions < 4.

# We simulate Hashmap below by creating bash variables with "Key" name and
# assign "Value" to that variable. Prefix of these keys will be common.
# Example:
#  Common prefix: CommandsPathMap
#  Then Key1=Value1 and Key2=Value2 variables will be as follows when f_hput()
#  and f_get() APIs are used.
#
#  CommandsPathMapKey1=Value1
#  CommandsPathMapKey1=Value2

#
# Adds entry to hashmap.
#
# Arguments:
#  arg1: Key
#  arg2: Value
#
f_hput() {
  eval hash"$1"='$2'
}

# Gets entry from hashmap.
#
# Arguments:
#  arg1: Key
# Output: Prints Value corresponding to given Key.
#
f_hget () {
  #eval echo '${hash'"$1"'#hash}'
  eval echo '${hash'"$1"}
}

# This array contains all commands for which we need to find full path and these
# gets passed as gflag later on to linux_agent_exec in f_start().
RequiredCommandsArray=(
  "mount"
  "umount"
  "chmod"
  "chown"
  "hostname"
)

not_found=()

# 'sudo' command path is required if target user is non-root.
if [ $COHESITY_USER != "root" ]; then
  RequiredCommandsArray+=("sudo")
fi

if [ $COHESITY_SKIP_ORACLE_CHECK -eq 0 ]; then
  RequiredCommandsArray+=("mkdir")
  RequiredCommandsArray+=("rm")
  RequiredCommandsArray+=("tee")
  RequiredCommandsArray+=("stat")
  RequiredCommandsArray+=("cp")
  RequiredCommandsArray+=("sed")
  RequiredCommandsArray+=("touch")
fi

if [ $COHESITY_SKIP_RESTORE_CHECK -eq 0 ]; then
  RequiredCommandsArray+=("blkid")
  RequiredCommandsArray+=("lsof")
  RequiredCommandsArray+=("ls")
  RequiredCommandsArray+=("losetup")
  RequiredCommandsArray+=("dmsetup")
  RequiredCommandsArray+=("rsync")
  RequiredCommandsArray+=("timeout")
fi

if [ $COHESITY_LVM_INSTALLED -eq 1 ]; then
  RequiredCommandsArray+=("lvs")
  RequiredCommandsArray+=("vgs")
  RequiredCommandsArray+=("lvcreate")
  RequiredCommandsArray+=("lvremove")
  RequiredCommandsArray+=("lvchange")
fi

# This is the array of optional commands. We try to find full path of these
# commands, but do not exit if we cannot find it for some reason (Ex: rsync is
# not installed on the system). linux_agent_exec is suppose to handle "empty"
# command gflag for these commands gracefully.
OptionalCommandsArray=(
  "wget"
  "nproc"
  "host"
  "findmnt"
  "xfs_admin"
  "cat"
  "grep"
  "getfacl"
  "setfacl"
  "readlink"
)

OptionalOracleCommandsArray=(
  "lltconfig"
  "hagrp"
  "hares"
  "haclus"
)

if [ $COHESITY_SKIP_ORACLE_CHECK -ne 0 ]; then
  OptionalCommandsArray+=("mkdir")
  OptionalCommandsArray+=("rm")
  OptionalCommandsArray+=("tee")
  OptionalCommandsArray+=("stat")
  OptionalCommandsArray+=("cp")
  OptionalCommandsArray+=("sed")
  OptionalCommandsArray+=("touch")
fi

if [ $COHESITY_SKIP_RESTORE_CHECK -ne 0 ]; then
  OptionalCommandsArray+=("blkid")
  OptionalCommandsArray+=("lsof")
  OptionalCommandsArray+=("ls")
  OptionalCommandsArray+=("losetup")
  OptionalCommandsArray+=("dmsetup")
  OptionalCommandsArray+=("rsync")
  OptionalCommandsArray+=("timeout")
  OptionalCommandsArray+=("ubax")
fi

if [ $COHESITY_LVM_INSTALLED -ne 1 ]; then
  OptionalCommandsArray+=("lvs")
  OptionalCommandsArray+=("vgs")
  OptionalCommandsArray+=("lvcreate")
  OptionalCommandsArray+=("lvremove")
  OptionalCommandsArray+=("lvchange")
fi


# Go through each command in RequiredCommandsArray and find out its full path.
# If we failed to find full path of any command, we exit right away.
# Full paths found are populated in global hashmap with f_hput() API which is
# explained above.
f_find_cmd_full_path_checked() {
  for key in "${RequiredCommandsArray[@]}"
  do
    local my_cmd=$key

    # Find out full path of command.
    local cmd_path=$(whereis $my_cmd | awk '{ print $2 }')
    if [[ "x$cmd_path" == "x" ]] ; then
      f_log "Error: Failed to find full path of ${my_cmd}."
      exit 1
    fi
    f_hput "$key" "$cmd_path"
  done
}

# Go through each command in OptionalCommandsArray and find out its full path.
# If we failed to find full path of any command, it is ignored.
# Full paths found are populated in global hashmap with f_hput() API which is
# explained above.
f_find_cmd_full_path() {
  for key in "${OptionalCommandsArray[@]}"
  do
    local my_cmd=$key

    # Find out full path of command.
    local cmd_path=$(whereis $my_cmd | awk '{ print $2 }')
    if [[ "x$cmd_path" == "x" ]] ; then
      f_log "Warning: Failed to find full path of ${my_cmd}.", 1
      # We ignore error here and allow these command paths to be set empty.
      # linux agent process should such flags being set empty gracefully.
    fi
    f_hput "$key" "$cmd_path"
  done

  local active_passive=false
  for key in "${OptionalOracleCommandsArray[@]}"
  do
    local my_cmd=$key
    # Find out full path of command.
    local cmd_path=$(whereis $my_cmd | awk '{ print $2 }')
    if [[ "x$cmd_path" == "x"  ]] ; then
      not_found+=("$key")
    else
      active_passive=true
    fi
    f_hput "$key" "$cmd_path"
  done
  if [[ "$active_passive" = true  && ${#not_found[@]} -ne 0  ]] ; then
    for lost in "${not_found[@]}"
    do
      f_log "Warning: Failed to find full path of ${lost}.", 1
    done
  fi

}
