#!/bin/bash
#
# Copyright 2016 Cohesity Inc.
#
# Author: Sachin Gaikwad (sachin@cohesity.com)
#
# Description:
# This shell script module contains common utility functions.
#

# ----------------------------------------------------------------------------

f_run_cmd() {
  local CMD="$@"
  eval "$CMD"
  if [ $? -ne 0 ]; then
    echo "Error: $CMD failed"
    exit 1
  fi
}

# ----------------------------------------------------------------------------

f_run_cmd_nocheck() {
  local CMD="$@"
  eval "$CMD"
  if [ $? -ne 0 ]; then
    echo "Error: $CMD failed"
    EXIT_STATUS=1
  fi
}

# ----------------------------------------------------------------------------

f_get_linux_distro() {
  if [ -f /etc/os-release ]; then
    LINUX_DISTRO=`grep "^PRETTY_NAME=" /etc/os-release | awk -F'=' '{ print $2 }'`
    return 0
  fi
  if [ -f /etc/system-release ]; then
    LINUX_DISTRO=`cat /etc/system-release`
    return 0
  fi
  if [ -f /etc/redhat-release ]; then
    LINUX_DISTRO=`cat /etc/redhat-release`
    return 0
  fi
  if [ -f /etc/centos-release ]; then
    LINUX_DISTRO=`cat /etc/centos-release`
    return 0
  fi
  which lsb_release &> /dev/null
  if [ $? -eq 0 ]; then
    LINUX_DISTRO=`lsb_release -a |grep Description | awk -F':' '{ print $2}' | tr -s "\t" " "`
    return 0
  fi
}

# ----------------------------------------------------------------------------

# This functions finds init system in use on system and sets global variable
# INIT_SYSTEM_TYPE.
# Possible init system detected: "upstart", "systemd" and "SysV".
f_get_init_system() {
  init_type=""
  INIT_SYSTEM_TYPE=""

  # command : cat -e /sbin/init | grep -waoe 'sysvinit' | head -n1
  # output  : sysvinit
  init_type=$(cat -e /sbin/init | grep -waoe 'sysvinit' | head -n1)
  if [[ $init_type == "sysvinit" ]]; then
    INIT_SYSTEM_TYPE="SysV"
    return 0
  fi

  # command : cat -e /sbin/init | grep -waoe 'upstart' | head -n1
  # output  : upstart
  init_type=$(cat -e /sbin/init | grep -waoe 'upstart' | head -n1)
  if [[ $init_type == "upstart" ]]; then
    # Most Ubuntu systems.
    INIT_SYSTEM_TYPE="upstart"
    return 0
  fi

  # command : cat -e /sbin/init | grep -waoe 'systemd' | head -n1
  # output  : systemd
  init_type=$(cat -e /sbin/init | grep -waoe 'systemd' | head -n1)
  if [[ $init_type == "systemd" ]]; then
    # Most CentOS/RedHat systems.
    INIT_SYSTEM_TYPE="systemd"
    return 0
  fi

  f_log "Error: Unable to determine INIT system"
  return 1
}

# ----------------------------------------------------------------------------

# This function finds version of GNU coreutils using timeout command
# and populates it in GNU_COREUTILS_VERSION global variable.
f_get_gnu_coreutils_version() {
  local timeout_cmd=$1
  local timeout_cmd_version=`timeout --version | head -1 | awk '{ print $4 }'`
  GNU_COREUTILS_VERSION=$timeout_cmd_version
}

# ----------------------------------------------------------------------------

f_get_timeout_command_has_kill_after() {
  local timeout_cmd=$1
  local kill_after=`$timeout_cmd --help  | awk '$2=="--kill-after=DURATION"{print $1}'`
  if [ "x$kill_after" == "x" ]; then
    return 1
  fi
  return 0
}

# ----------------------------------------------------------------------------

# Finds software version from given binary with help of gflag '--version'.
# If binary is from master/developer branch, it prints 0.0.0-<branch-name>
#  Example: 0.0.0-master if binary is from git master branch.
# If binary is from 2.9 git branch, it prints software version as 2.9
# If binary does not exist, it prints "NotFound".
f_find_software_version() {
  local exec_full_path=$1
  if [ ! -f $exec_full_path ]; then
    echo "NotFound"
    return 0
  fi
  local software_version=$($exec_full_path --version | head -1 | awk '{ print $3 }' | awk -F'-' '{ print $2 }')

  # If this build is from master branch or any other developer branches, then
  # SOFTWARE_VERSION will be set to git branch name by above command. Detect
  # this below with regex and if true, set SOFTWARE_VERSION to
  # 0.0.0-<branchname>.
  echo $software_version | grep "^[0-9]\.[0-9]" &> /dev/null
  if [ $? -ne 0 ]; then
    software_version="0.0.0-$software_version"
  fi
  echo $software_version
  return 0
}

# ----------------------------------------------------------------------------

# This functions find build type from binary's --version option.
f_get_build_type() {
  local exec_full_path=$1
  local build_type_string=$($exec_full_path --version | head -1 | awk '{ print $3 }' | awk -F'-' '{ print $1 }')
  echo $build_type_string
}

# ----------------------------------------------------------------------------

# Finds all non-zombie process IDs of the given executable and prints those IDs
# on the standard output. The return values are similar to the 'pidof' command:
# EXIT STATUS
#   0      At least one program was found with the requested name.
#   1      No program was found with the requested name.
non_zombie_pidof() {
  local exec_name=$1
  pids=$(pidof "$exec_name")
  [ "$pids" ] || return 1;
  # Get the process state for each PID and filter out zombies (Z).
  # Example: ps -o s= -o pid= $pids
  #   S 11152
  #   S 11154
  #   Z 11111
  non_zombie_pids=$(ps -o s= -o pid= $pids | grep -v ^Z | \
    sed -n 's/^[A-Z][[:space:]]//p');
  # Print the PIDs to stdout.
  echo $non_zombie_pids
  [ "$non_zombie_pids" ] || return 1;
  return 0
}

# ----------------------------------------------------------------------------

# This function finds mountpoint options for a given mountpoint as argument 1.
# Returns mountpoint options in global var $MOUNTPOINT_OPTIONS.
#
f_get_mountpoint_options() {
  local mountpoint=$1
  local options=`mount | grep "on $mountpoint type" | awk '{ print $6 }'`
  MOUNTPOINT_OPTIONS=$options
  return 0
}

# ----------------------------------------------------------------------------

#
# This function checks if upgrade from given 'old_software_version' to
# 'my_software_version' is allowed or not. Returns 0 if upgrade is allowed,
# otherwise returns 1.
#
# NOTE: Allow upgrade to any master build. Master build/developer builds coming
# from any master branch are of the form: 0.0.0-<branchname>.
#
f_is_upgrade_allowed() {
  local old_software_version=$1
  local my_software_version=$2

  # Software version can be in this format:
  #  w.x          ; Example: 3.5
  #  w.x.y        ; Example: 3.5.2
  #  w.x.y-custom ; Example: 3.5.2-cohesity
  #  w.x.yz-custom ; Example: 3.5.2b-cohesity
  #  Note: z can only be alphabet from [a-z]

  local old_w=`echo $old_software_version | awk -F'.' '{ print $1 }'`
  local old_x=`echo $old_software_version | awk -F'.' '{ print $2 }'`
  local old_y=`echo $old_software_version | awk -F'.' '{ print $3 }' | awk -F'-' '{ print $1 }'`
  local old_z_char=0
  if [ "x$old_y" == "x" ]; then
    old_y=0
  else
    # Find first non-numeric character in 'old_y'.
    index=0
    old_y_len=${#old_y}
    while [ $index -lt $old_y_len ]
    do
      old_z_char=${old_y:$index:1}
      if [[ $old_z_char != [0-9] ]]; then
        old_y=${old_y:0:$index}
        if [[ $old_z_char != [a-z] ]]; then
          # Not a alphabet in range [a-z], upgrade not allowed.
          return 1
        fi
        break
      fi
      index=$(($index + 1))
    done
  fi

  local my_w=`echo $my_software_version | awk -F'.' '{ print $1 }'`
  local my_x=`echo $my_software_version | awk -F'.' '{ print $2 }'`
  local my_y=`echo $my_software_version | awk -F'.' '{ print $3 }' | awk -F'-' '{ print $1 }'`
  local my_z_char=0
  if [ "x$my_y" == "x" ]; then
    my_y=0
  else
    # Find first non-numeric character in 'my_y'.
    index=0
    my_y_len=${#my_y}
    while [ $index -lt $my_y_len ]
    do
      my_z_char=${my_y:$index:1}
      if [[ $my_z_char != [0-9] ]]; then
        my_y=${my_y:0:$index}
        if [[ $my_z_char != [a-z] ]]; then
          # Not a alphabet in range [a-z], upgrade not allowed.
          return 1
        fi
        break
      fi
      index=$(($index + 1))
    done
  fi

  if [ $my_w -eq 0 -a $my_x -eq 0 -a $my_y -eq 0 ]; then
    # If my software version is all zeros, allow such upgrades.
    # Example: old=3.5.0 new=0.0.0-quickfix
    return 0
  fi

  if [ $old_w -gt $my_w ]; then
    # Not allowed.
    # Example: old=3.5.0 new=2.9.0
    return 1
  elif [ $old_w -lt $my_w ]; then
    # Allowed.
    # Example: old=2.5.0 new=3.9.0
    return 0
  fi
  # $old_w == $my_w

  if [ $old_x -gt $my_x ]; then
    # Not allowed
    # Example: old=3.5.0 new=3.1.0
    return 1
  elif [ $old_x -lt $my_x ]; then
    # Allowed.
    # Example: old=3.5.0 new=3.9.0
    return 0
  fi
  # $old_x == $my_x

  if [ $old_y -gt $my_y ]; then
    # Not allowed
    # Example: old=3.5.5 new=3.5.2
    return 1
  elif [ $old_y -lt $my_y ]; then
    # Allowed.
    # Example: old=3.5.4 new=3.5.9
    return 0
  fi
  # $old_y == $my_y

  if [[ $old_z_char > $my_z_char ]]; then
    # Not allowed
    # Example: old=3.5.1b new=3.5.1a
    return 1
  elif [[ $old_z_char < $my_z_char ]]; then
    # Allowed.
    # Example: old=3.5.4a new=3.5.4b
    return 0
  fi
  # $old_z_char == $my_z_char

  # Both versions are equal. Allow such upgrades.
  return 0
}

# ----------------------------------------------------------------------------

f_marker() {
  echo "------------------------------------------------------------------------------"
}

# ----------------------------------------------------------------------------

f_log() {
  CUR_LOG_LEVEL=0
  if [ ! -z "${COHESITY_SCRIPT_LOG:-}" ]; then
    CUR_LOG_LEVEL=$COHESITY_SCRIPT_LOG
  fi
  if [ $# -eq 1 ] || [ $2 -le $CUR_LOG_LEVEL ]; then
    echo "==> $@"
  fi
}

# ----------------------------------------------------------------------------

# This function finds the version of systemd.
# Output for command "systemctl --version":
# systemd 229
# +PAM +AUDIT +SELINUX +IMA +APPARMOR +SMACK +SYSVINIT +UTMP +LIBCRYPTSETUP +GCRYPT +GNUTLS +ACL +XZ -LZ4 +SECCOMP +BLKID +ELFUTILS +KMOD -IDN
#
# Returns version number.
#
f_get_systemd_version() {
  return "$(systemctl --version | awk '/systemd/ {print $2}')"
}

# ----------------------------------------------------------------------------
