#!/bin/bash
#
# Copyright 2016 Cohesity Inc.
#
# Author: Sachin Gaikwad (sachin@cohesity.com)
#
# Description:
# This script provides operations like start, stop, status to manage Cohesity
# Linux Agent process on a Linux machine.
#
# Process launch:
# We start linux_agent_exec process in background with '&'. linux_agent_exec
# itself has self-monitoring enabled to handle process crashes. At any point,
# there should be 2 linux_agent_exec processes running: parent and child. If
# parent process itself is killed (for some reason), it will not be launched on
# its own. Launching of parent process should be handled by adding
# 'cohesity-agent' as service in init system which supports 'respawn'
# functionality.
#
# TODO(sachin): For systems that have 'systemd', use systemd and systemctl
# commands to launch process in slices.
#

#set -x

current_dir=$(dirname `readlink -f $0`)

# Below variables are set to default values, this will be overwrite by
# set_env.sh file. It may be possible that old set_env.sh file may not have
# this variables defined. But command_path.sh file requires this variables to
# be present so defined here.
COHESITY_SKIP_RESTORE_CHECK=0
COHESITY_SKIP_ORACLE_CHECK=0
COHESITY_LVM_INSTALLED=0

# Source helper scripts.
if [ "x$1" == "xsingle-mode" ]; then
  echo "Running in single-mode, sourcing $current_dir/set_env.sh"
  . $current_dir/set_env.sh
else
  . /etc/cohesity-agent/set_env.sh
fi

. $current_dir/common.sh
. $current_dir/command_path.sh

f_find_cmd_full_path_checked
f_find_cmd_full_path


# ----------------------------------------------------------------------------

f_start() {
  # Any argument pass to this function are passed to linux agent in single mode
  # operation.
  AGENT_INPUT_FLAGS="$@"

  local exec_name=$AGENT_EXEC_NAME
  whoami
  id

  # Check if process is already running. If yes, bail out. Otherwise start new
  # process.
  local pids=$(pidof $exec_name)
  if [ "$pids" ]; then
    echo "$AGENT_SERVICE_NAME is already running: PID: $pids"
    exit 1
  fi

  echo "Starting ${exec_name}..."

  local exec_path="$AGENT_EXEC_PATH"

  # Find UID for user COHESITY_USER
  local service_user_id=`id -u $COHESITY_USER`
  if [ $? -ne 0 ]; then
    echo "Error: Failed to find UID for user $COHESITY_USER"
    exit 1
  fi

  # We may run linux_agent.sh as 'root' user. In such a case, we should not run
  # any command with 'sudo', otherwise we get error saying 'tty is required to
  # run command with sudo' (especially on RHEL). If we are not running as
  # 'root', then we must be running as 'service_user_id' which is expected to
  # have '!requiredtty' setting in sudoers list.
  local script_run_user_id=`id -u`
  local script_sudo_command=""
  if [ $script_run_user_id -ne 0 ]; then
    script_sudo_command="sudo"
  fi

  if [ $service_user_id -ne 0 ]; then
    # Set the capability bits on linux_agent_exec executable for running all
    # LVM, Restore and oracle commands and to increase max open files.
    $script_sudo_command setcap CAP_SYS_ADMIN,CAP_CHOWN,CAP_FOWNER,CAP_DAC_OVERRIDE,CAP_SYS_RESOURCE,CAP_FSETID,CAP_KILL+epi ${exec_path}
    if [[ $? -ne 0 && $ARG_SINGLE_MODE -ne 1 ]]; then
      echo "Error: Failed to setcap on ${exec_path}"
      exit 1
    fi
  else
    f_hput "sudo" ""
  fi

  # 16 MB.
  local tcmalloc_thread_cache_size=$((1024 * 1024 * 16))

  # if LVM is installed on the machine we would have set
  # "COHESITY_LVM_INSTALLED" to true from setup.sh
  # LVM related flags.
  local LVM_FLAGS=""
  if [ $COHESITY_LVM_INSTALLED -eq 1 ]; then
    LVM_FLAGS="\
      --linux_agent_no_lvm=false \
      --linux_lvs_command=`f_hget lvs` \
      --linux_vgs_command=`f_hget vgs` \
      --linux_lvcreate_command=`f_hget lvcreate` \
      --linux_lvremove_command=`f_hget lvremove` \
    "
  else
    LVM_FLAGS="--linux_agent_no_lvm=true"
  fi

  # linux_agent_exec specific flags.
  AGENT_FLAGS="\
    --linux_agent_tmp_dir=$AGENT_TMP_DIR \
    --linux_agent_nfs_mountpoint_base_dir=$AGENT_NFS_MOUNTS_DIR \
    --linux_agent_oracle_nfs_mountpoint_base_dir=$AGENT_NFS_ORACLE_MOUNTS_DIR \
    --linux_agent_user_script_dir_path=$USER_SCRIPT_DIR \
    --linux_agent_user_script_log_dir_path=$USER_SCRIPT_LOG_DIR \
    --linux_agent_volume_mountpoint_base_dir=$AGENT_VOL_MOUNTS_DIR \
    --linux_agent_config_file_path=$AGENT_CONFIG_FILE \
    --linux_agent_cert_file_path=$AGENT_CERT_FILE \
    --linux_agent_oracle_sql_script_dir_path=$ORACLE_SCRIPT_DIR \
    --linux_agent_oracle_view_user_owner=$COHESITY_USER \
    --linux_agent_oracle_view_group_owner=$COHESITY_GROUP \
    --linux_agent_oracle_host_command=`f_hget host` \
    --linux_cat_location=`f_hget cat`\
    --linux_grep_location=`f_hget grep`\
    --linux_agent_oracle_active_passive_lltconfig_command=`f_hget lltconfig` \
    --linux_agent_oracle_active_passive_hagrp_command=`f_hget hagrp` \
    --linux_agent_oracle_active_passive_hares_command=`f_hget hares` \
    --linux_agent_ubax_cmd=`f_hget ubax` \
    --linux_agent_oracle_active_passive_haclus_command=`f_hget haclus` \
    --util_misc_mount_command=`f_hget mount` \
    --util_misc_umount_command=`f_hget umount` \
    --util_misc_use_findmnt_command=false \
    --util_misc_findmnt_command=`f_hget findmnt` \
    --util_misc_readlink_command=`f_hget readlink` \
    --util_misc_sudo_command=`f_hget sudo` \
    --util_misc_chmod_command=`f_hget chmod` \
    --util_misc_chown_command=`f_hget chown` \
    --util_misc_nproc_command=`f_hget nproc` \
    --util_misc_lvchange_command=`f_hget lvchange` \
    --util_misc_blkid_command=`f_hget blkid` \
    --util_misc_xfs_admin_command=`f_hget xfs_admin` \
    --util_misc_timeout_command=`f_hget timeout` \
    --yoda_util_vmware_mount_command=${COHESITY_BIN_DIR}/vm_mounter.sh \
    --yoda_mount_command=`f_hget mount` \
    --yoda_umount_command=`f_hget umount` \
    --yoda_findmnt_command=`f_hget findmnt` \
    --yoda_sudo_command=`f_hget sudo` \
    --timeout_command=`f_hget timeout` \
    $LVM_FLAGS \
    --yoda_blkid_command=`f_hget blkid` \
    --yoda_lsof_command=`f_hget lsof` \
    --yoda_ls_command=`f_hget ls` \
    --linux_sudo_command=`f_hget sudo` \
    --linux_chmod_command=`f_hget chmod` \
    --linux_timeout_command=`f_hget timeout` \
    --linux_agent_rsync_cmd=`f_hget rsync` \
    --linux_agent_wget_command=`f_hget wget` \
    --yoda_losetup_command=`f_hget losetup` \
    --yoda_dmsetup_command=`f_hget dmsetup` \
    --linux_dump_memory_usage=false \
    --yoda_bindfs_mount=false \
    --libssl_non_fips_path="" \
    --libssl_path="" \
    -v=0 \
    --linux_agent_pkg_type=$AGENT_PKG_TYPE \
    --linux_agent_hostname_command=`f_hget hostname` \
    --linux_setfacl_command=`f_hget setfacl` \
    --linux_getfacl_command=`f_hget getfacl` \
    --linux_agent_which_cmd=`f_hget which` \
    --linux_agent_licmgr_cmd=`f_hget licmgr` \
    --linux_should_stat_check=true \
    "

  f_get_gnu_coreutils_version "`f_hget timeout`"
  if [ "x$GNU_COREUTILS_VERSION" == "x" ]; then
    echo "Error: Failed to find GNU coreutils version."
    exit 1
  fi
  echo "GNU coreutils version = $GNU_COREUTILS_VERSION"

  # Some extra gflags that need to be passed to linux_agent_exec.
  AGENT_EXTRA_FLAGS=""

  #
  # Examples of GNU Coreutils version:
  #
  # bash$ timeout --version | head -1
  # timeout (GNU coreutils) 8.4
  #
  # bash$ timeout --version | head -1
  # timeout (GNU coreutils) 8.21
  #
  # x = coreutils_version_first_digit
  # y = coreutils_version_second_digit
  #
  local x=`echo $GNU_COREUTILS_VERSION | awk -F'.' '{ print $1 }'`
  local y=`echo $GNU_COREUTILS_VERSION | awk -F'.' '{ print $2 }'`
  if [ $x -lt 8 ] || [ $x -eq 8 -a $y -lt 5 ] ; then
    AGENT_EXTRA_FLAGS="\
      --linux_timeout_command_option_kill_after="" \
      --yoda_timeout_command_option_kill_after="" \
      "
  fi

  f_get_timeout_command_has_kill_after "`f_hget timeout`"
  if [ $? -ne 0 ]; then
    echo "Timeout command: `f_hget timeout` does not support --kill-after option"
    AGENT_EXTRA_FLAGS="\
      --linux_timeout_command_option_kill_after="" \
      --yoda_timeout_command_option_kill_after="" \
      "
  else
     echo "Timeout command: `f_hget timeout` does support --kill-after option"
  fi

  # Get build type of linux_agent_exec.
  AGENT_BUILD_TYPE=`f_get_build_type $AGENT_EXEC_PATH`
  if [ "x$AGENT_BUILD_TYPE" == "x" ]; then
    echo "ERROR: Failed to get build type of $exec_path"
  fi

  if [ "x$AGENT_BUILD_TYPE" != "xrelease" ]; then
    # Enable full coredumps for 'lean' or 'qa' builds.
    echo "Setting coredump limit to unlimited for $AGENT_BUILD_TYPE build"
    ulimit -c unlimited
  fi

  # check whether selinux is enabled on system and it is in enforcing mode.
  # Also whether runcon is present on system.
  runcon_present=0
  if [ $ARG_SINGLE_MODE -eq 1 ]; then
    which sestatus
    if [ $? -eq 0 ]; then
      sestatus | grep enabled
      if [ $? -eq 0 ]; then
        sestatus | grep Current | grep enforcing
        if [ $? -eq 0 ]; then
          which runcon
          if [ $? -eq 0 ]; then
            runcon_present=1
          else
            echo "System has selinux enabled with status enforcing but runcon is not present, restore may not work."
          fi
        fi
      fi
    fi
  fi

  # Do not runcon for Rhel 7.
  if [ $runcon_present -eq 1 ]; then
    if [ -f /etc/redhat-release ]; then
      version=`cat /etc/redhat-release | awk '{print $(NF-1)}'`
      version_first=`echo $version | awk -F'.' '{ print $1 }'`
      if [ $version_first -eq 7 ]; then
        runcon_present=0
      fi
    fi
  fi

  OUTFILE=$COHESITY_LOG_DIR/${exec_name}.log.STDOUTERR
  ENV="TCMALLOC_MAX_TOTAL_THREAD_CACHE_BYTES=${tcmalloc_thread_cache_size}"
  local run_process_in_background="&"
  echo "COHESITY USER : $COHESITY_USER"
  local cmd3="$script_sudo_command chown $COHESITY_USER:$COHESITY_GROUP $COHESITY_LOG_DIR"
  eval $cmd3
  if [[ $ARG_SINGLE_MODE -ne 1 ]]; then
    FILE=$COHESITY_BIN_DIR/dlls_dir.tar.gz
    if [ -f "$FILE" ]; then
      $script_sudo_command tar -xzf $COHESITY_BIN_DIR/dlls_dir.tar.gz -C $COHESITY_BIN_DIR/
      chown -R $COHESITY_USER:$COHESITY_GROUP $COHESITY_BIN_DIR/dlls_for_linking
    fi
    case "$ARG_INIT_SYSTEM_TYPE" in
      upstart)
        run_process_in_background=""
        echo "Init system is upstart, will run linux_agent_exec in foreground"
        ;;
      systemd)
        echo "Init system is systemd, will run linux_agent_exec in background"
        ;;
      *)
        echo "Unsupported init system, will run linux_agent_exec in background"
        ;;
    esac
    shell_type=`$script_sudo_command cat /etc/passwd | grep $COHESITY_USER | awk -F ":" '{print $NF}'`
    shell_type=`basename $shell_type`
      if [ $shell_type == 'csh' ]; then
        SHELL_ENV="setenv $ENV;"
      else
        SHELL_ENV=$ENV
      fi
    local cmd_to_run=""
    if [ $COHESITY_USER == root ]; then
      cmd_to_run="$SHELL_ENV exec $exec_path $COHESITY_LOG_FLAGS $AGENT_FLAGS $AGENT_EXTRA_FLAGS>> $OUTFILE 2>&1 $run_process_in_background"
    else
      cmd_to_run="$script_sudo_command su - $COHESITY_USER -c '$SHELL_ENV exec $exec_path $COHESITY_LOG_FLAGS $AGENT_FLAGS $AGENT_EXTRA_FLAGS>> $OUTFILE 2>&1 $run_process_in_background'"
    fi
  else
    cd $COHESITY_BIN_DIR
    if [ $runcon_present -eq 1 ]; then
      echo "Running with runcon"
      local cmd_to_run="$ENV runcon unconfined_u:unconfined_r:unconfined_t:s0-s0:c0.c1023 $exec_path $COHESITY_LOG_FLAGS $AGENT_FLAGS $AGENT_EXTRA_FLAGS $AGENT_INPUT_FLAGS>> $OUTFILE 2>&1 $run_process_in_background"
    else
      echo "Running without runcon"
      local cmd_to_run="$ENV $exec_path $COHESITY_LOG_FLAGS $AGENT_FLAGS $AGENT_EXTRA_FLAGS $AGENT_INPUT_FLAGS>> $OUTFILE 2>&1 $run_process_in_background"
    fi
  fi
  eval $cmd_to_run

  # Wait for process to start.
  sleep 2

  #
  # Running a command in the background always returns 0 even if the executable
  # is not found. To really check if we were successful starting the command, we
  # check that parent process is launched or not with below ps command.
  #
  # Also, we note parent process's pid in $AGENT_SERVICE_PIDFILE. This file is
  # used by systemd init system to monitor parent process.
  #
  if [[ $ARG_SINGLE_MODE -ne 1 ]]; then
    output=$(ps afx)
    parent_pid=$(echo "$output" | sed -e "s/\/$exec_name/\/$exec_name/;tx;d;:x" | \
      grep -v gdb | awk 'NR==1{print $1;}')
    if [ "x$parent_pid" == "x" ]; then
      echo "Could not start: $exec_path $@"
      exit 1
    fi

    # If systemd version is greater than SYSTEMD_PRIVILEGED_VERSION(237),
    # then Explicitly setting root:root as user:group for AGENT_SERVICE_DIR
    # due to updates in systemd.
    # Starting from RHEL 7_7, the systemd version is 219 which only reads
    # PID file only if the user privileges set to root:root
    # ref :- https://access.redhat.com/solutions/4397951
    if [ "x$ARG_INIT_SYSTEM_TYPE" == "xsystemd" ]; then
      f_get_systemd_version
      local systemd_version=$?
      if [ $systemd_version -ge $SYSTEMD_PRIVILEGED_VERSION ] || [ $systemd_version -ge $SYSTEMD_ROOT_VERSION  ]; then
        chown root:root $AGENT_SERVICE_DIR
      fi
    fi

    echo "Parent process pid=$parent_pid"
    $script_sudo_command echo $parent_pid > $AGENT_SERVICE_PIDFILE
    if [ $? -ne 0 ]; then
      echo "Could not write parent pid of cohesity process to $AGENT_SERVICE_PIDFILE , please check if the file exists"
    fi
  fi

  # Just dump PIDs of parent and child process for debugging purpose.
  pidof $exec_name
}

# ----------------------------------------------------------------------------

f_stop() {
  local exec_name=$AGENT_EXEC_NAME
  echo "Stopping ${exec_name}..."

  # Kill the parent process of the given service, which should kill it's
  # children. Ignore any gdb processes.
  output=$(ps afx)
  pids=$(echo "$output" | sed -e "s/\/$exec_name/\/$exec_name/;tx;d;:x" | \
    grep -v gdb | awk 'NR==1{print $1;}')
  echo "linux_agent_exec: parent PID: $pids"
  if [ "$pids" ]; then
    kill $pids
  else
    echo "$exec_name is not running"
    return 0
  fi

  # Wait for process to exit.
  sleep 2

  if non_zombie_pidof $exec_name;
  then
    echo "Could not kill $exec_name"
    return 1
  fi
  return 0
}

# ----------------------------------------------------------------------------

f_status() {
  local exec_name=$AGENT_EXEC_NAME
  local pids=$(pidof $exec_name)
  if [ "$pids" ]; then
    echo "$AGENT_SERVICE_NAME is running: PID: $pids"
  else
    echo "$AGENT_SERVICE_NAME is not running"
  fi
}

# ----------------------------------------------------------------------------

f_version() {
  SOFTWARE_VERSION=$(f_find_software_version $AGENT_EXEC_PATH)
  echo "Version: $SOFTWARE_VERSION"
}

# ----------------------------------------------------------------------------

f_find_cmd_full_path_checked
f_find_cmd_full_path
AGENT_BUILD_TYPE=`f_get_build_type $AGENT_EXEC_PATH`
echo "AGENT_BUILD_TYPE=$AGENT_BUILD_TYPE"
ARG_SINGLE_MODE=0
ARG_INIT_SYSTEM_TYPE=""

case "$1" in
  start)
    ARG_INIT_SYSTEM_TYPE=$2
    f_start
    ;;
  stop)
    f_stop
    ;;
  restart)
    f_stop
    f_start
    ;;
  status)
    f_status
    ;;
  version)
    f_version
    ;;
  single-mode)
    ARG_SINGLE_MODE=1
    ARG_INIT_SYSTEM_TYPE=$2
    # We shift by 2 here: 1st argument is "single-mode" and 2nd argument is
    # "init system type". Rest of the argument we pass as it is to f_start().
    # These are expected to be extra linux_agent_exec options to pass when
    # starting.
    shift 2
    f_start "$@"
    ;;
  *)
    echo "Usage: $0 {start|stop|restart|status|version|single-mode}"
    exit 1
esac
exit 0
