#!/bin/bash
# Copyright 2019 Cohesity Inc.
#
# Author: Kiran Dodani
#
# This is a wrapper script to create clone from snapshot.
# It also appends the output at /tmp/rman_clone_of_clone_db_{TASK_ID}.INFO file.
#
# Exit from the script if any of the step fails.
# cclone1 -R $ORACLE_HOME/bin/rman -S $ORACLE_HOME/bin/sqlplus -A clone1 -B
# cs200 -T 01:11:2019-14:40:54 -O $COHESITY_VIEW -I cohesity_5270 -V 12.1.0.2
# -H $ORACLE_HOME -J $ORACLE_BASE -F /tmp
# -l $COHESITY_VIEW/4376_09asdrfg_1_1,$COHESITY_VIEW/4376_03tgghvj_1_1,$COHESITY_VIEW/4376_01tgghve_1_1,$COHESITY_VIEW/4376_02thggkf_1_1,$COHESITY_VIEW/4376_07tggi38_1_1
# -E /tmp/switch_clause_cohesity_26901.sql -N 1024

set -e

LOG_FILE=""

# Which OS Cohesity agent is running on.
OS=$(uname -a | cut -d " " -f 1)

if [ ${OS} = "AIX" ]; then
  # TODO(aabhas): find replacement for readlink on AIX.
  current_dir=$(dirname $0)
else
  current_dir=$(dirname `readlink -f $0`)
fi

SCRIPT_NAME=`basename "$0"`
# Source needed library functions.
. $current_dir/common_utils.sh

usage()
{
  echo "Invoke the script with following arguments :-"
  echo -e "\t -R|--rman-path                    Path to RMAN binary."
  echo -e "\t -S|--sqlplus-path                 Path to source SQLPLUS binary."
  echo -e "\t -A|--src-oracle-db-name           Oracle source database name."
  echo -e "\t -B|--dest-db-name                 Oracle destination database/SID value."
  echo -e "\t -T|--restore-time                 Restore time of database."
  echo -e "\t -O|--cohesity-view-location       Location of Cohesity view where all the files resides."
  echo -e "\t -I|--id                           Operation ID."
  echo -e "\t -F|--log-file-path                Rman log file path."
  echo -e "\t -H|--dst-oracle_home              Destination oracle home."
  echo -e "\t -J|--dst-oracle_base              Destination oracle base."
  echo -e "\t -E|--switch-sql-file              Path to the switch SQL file."
  echo -e "\t -N|--num-data-files               Number of data files to start with."
  echo -e "\t -V|--src-database-version         Oracle source database version."
  echo -e "\t -s|--shared_pool_size             Shared Pool Size."
  echo -e "\t -t|--sga_target                   SGA Target."
  echo -e "\t -i|--fra_size                     FRA size in MBs."
  echo -e "\t -o|--no_open                      Restore in NOOPEN mode."
  echo -e "\t -d|--sleep_sec                    Amount of time in seconds to sleep after changing permission needed for AIX."
}

# Shutdown the database. This routine is called as part of cleanup if clone
# fails at any stage.
shutdown_db()
{
  log_message "### Shutting down the database as part of cleanup."
  $SQLPLUS_PATH / $sqlplus_privilege << EOF >> ${LOG_FILE}
  WHENEVER OSERROR EXIT 11
  WHENEVER SQLERROR EXIT 22
  shutdown abort;
  exit;
EOF
  return $?
}

# This function cleans up the clone and exit with error code if caller routine failed.
exit_handler() {
  status=$1;
  if [ $status -eq 0 ]; then
    return;
  fi

  # Doing cleanup.
  log_message $2;
  shutdown_db;
  exit $status;
}

mod_owner_group_and_permission() {
  local dirpath=$1
  sudo chmod -R 775 ${dirpath}
  sudo chown -R ${ORACLE_USER}:${ORACLE_GROUP} ${dirpath}
  log_message "### Permission 775 and ownership ${ORACLE_USER}:${ORACLE_GROUP} changed for ${dirpath}"
}

dump_progress() {
  # Write progress number to tmp file first, then rename temp file
  # to actual progress file for Agent code to read the file. The 'mv'
  # command is an atomic file operation on Linux. This ensures a
  # concurrent reader of the file will only read either the old version
  # or the new version of the progress file, but never partial data in
  # a new progress file.
  tmpfile="${PROGRESS_FILE}.`date +%s`.$$"
  echo "$1" > ${tmpfile}
  mv "${tmpfile}" "$PROGRESS_FILE"
}
##### Main

while getopts "R:S:A:B:T:O:I:F:H:J:E:V:s:t:i:d:o" option; do
  case $option in
    R)
      RMAN_PATH=$OPTARG;;
    S)
      SQLPLUS_PATH=$OPTARG;;
    A)
      SRC_DB_NAME=$OPTARG;;
    B)
      DEST_NEW_NAME=$OPTARG;;
    T)
      RESTORE_TIME=$OPTARG;;
    O)
      COHESITY_VIEW=$OPTARG;;
    I)
      ID=$OPTARG;;
    F)
      LOG_FILE_PATH=$OPTARG;;
    H)
      DEST_ORACLE_HOME=$OPTARG;;
    J)
      DEST_ORACLE_BASE=$OPTARG;;
    E)
      SWITCH_SQL_FILENAME=$OPTARG;;
    N)
      NUM_DATA_FILES=$OPTARG;;
    V)
      SRC_DB_VERSION=$OPTARG;;
    s)
      SHARED_POOL_SIZE=$OPTARG;;
    t)
      SGA_TARGET_SIZE=$OPTARG;;
    i)
      FRA_SIZE=$OPTARG;;
    o)
      NO_OPEN=$OPTARG;;
    d)
      SLEEP_SEC=$OPTARG;;
    *) echo "Invalid Option $option passed"; usage; exit 1 ;;
  esac
done

export ORACLE_HOME=$DEST_ORACLE_HOME

if [[ "x$LOG_FILE_PATH" == "x" ]]; then
  LOG_FILE_PATH="/tmp"
fi
LOG_FILE=${LOG_FILE_PATH}/rman_clone_of_clone_db_${ID}.INFO
# Update rman log file permissions.
touch $LOG_FILE
sudo chmod 776 $LOG_FILE

# Modify how to invoke Oracle binaries if the agent is installed with 'root'
setup_invoke_oracle_binaries

# Create progress file
PROGRESS_FILE=${LOG_FILE_PATH}/${ID}_progress
touch $PROGRESS_FILE

if [ -z "$RMAN_PATH" ] || [ -z "$SQLPLUS_PATH" ] || [ -z "$SRC_DB_NAME" ] || [ -z "$SRC_DB_VERSION" ] || [ -z "$DEST_NEW_NAME" ]; then
  log_message "RMAN configuration parameters missing."
  log_message "#####$RMAN_PATH, $SQLPLUS_PATH, $SRC_DB_NAME, $SRC_DB_VERSION, $DEST_NEW_NAME";
  usage;
  exit 1;
fi

if [ -z "$RESTORE_TIME" ] || [ -z "$COHESITY_VIEW" ]; then
  log_message "RMAN script parameters missing.";
  log_message "######$COHESITY_VIEW";
  usage;
  exit 1;
fi

if [ -z "$DEST_ORACLE_HOME" ] || [ -z "$DEST_ORACLE_BASE" ]; then
  log_message "RMAN DST Oracle home/base parameter is missing.";
  log_message "#####$DEST_ORACLE_HOME, $DEST_ORACLE_BASE";
  usage;
  exit 1;
fi

if [ -z "$SWITCH_SQL_FILENAME" ]; then
  log_message "datafile/redolog is missing.";
  log_message "##### $SWITCH_SQL_FILENAME";
  usage;
  exit 1;
fi

start_time="`date "+%D %X"`"
log_message "### ============== Clone of Clone started for Database ${DEST_NEW_NAME} at : ${start_time} ============="

if [ ! -f ${DEST_ORACLE_HOME}/bin/oracle ]; then
  log_message "Unable to find ${DEST_ORACLE_HOME}/bin/oracle"
  exit 1;
fi

oracle_bin_permission=`ls -l ${DEST_ORACLE_HOME}/bin/oracle`
if [ $? -ne 0 ]; then
  log_message "Unable to find permissions for: ${DEST_ORACLE_HOME}/bin/oracle"
  exit 1;
fi

ORACLE_USER=`echo ${oracle_bin_permission} | awk '{print $3}'`
ORACLE_GROUP=`echo ${oracle_bin_permission} | awk '{print $4}'`

if [ -z ${ORACLE_USER} ] || [ -z ${ORACLE_GROUP} ]; then
  log_message "ORACLE_USER(${ORACLE_USER}) or ORACLE_GROUP(${ORACLE_GROUP}) not set."
  exit 1;
fi

# Giving permission to cohesity view.
log_message "### Running sudo chmod 1777 ${COHESITY_VIEW}"
sudo chmod 1777 ${COHESITY_VIEW};
sudo chown ${ORACLE_USER}:${ORACLE_GROUP} ${COHESITY_VIEW}
log_message "### Permission and ownership changed for ${COHESITY_VIEW}"

# Exporting PATH and LD_LIBRARY_PATH.
# export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:${DEST_ORACLE_HOME}/lib"
export PATH="$PATH:${DEST_ORACLE_HOME}/bin"

# Creating FRA directory
log_message "### Creating the directories as ${ORACLE_USER}:${ORACLE_GROUP}"
FRA_LOCATION="${COHESITY_VIEW}/fast_recovery_area"
sudo mkdir -p ${FRA_LOCATION}
DB_FRA_LOCATION="${FRA_LOCATION}/${DEST_NEW_NAME}"
sudo mkdir -p ${DB_FRA_LOCATION}

# Default FRA_SIZE set to 10240M.
if [ -z ${FRA_SIZE} ] || [ ${FRA_SIZE} -eq 0 ]; then
   FRA_SIZE=10240`
fi

#TODO(Sandeep) : Change this to $COHESITY_VIEW/controlfile after testing.
CONTROL_FILES_PATH="${DB_FRA_LOCATION}/controlfile"
sudo mkdir -p ${CONTROL_FILES_PATH}
CONTROL_FILES="'$CONTROL_FILES_PATH/control01.ctl'"
mod_owner_group_and_permission ${FRA_LOCATION}

# Creating parent directory for audit files.
DIAG_HOME="${COHESITY_VIEW}/diag"
DB_DIAG_LOCATION="${DIAG_HOME}/rdbms/${DEST_NEW_NAME}"
sudo mkdir -p ${DB_DIAG_LOCATION}
# Creating directory for audit file destination. The 2 step directory creation
# is required because {ORACLE_USER}:${ORACLE_GROUP} premission is needed for
# 'audit' and its parent dir '{DEST_NEW_NAME}'.
ADUMP_LOCATION="${DB_DIAG_LOCATION}/audit"
sudo mkdir -p ${ADUMP_LOCATION}
mod_owner_group_and_permission  ${DIAG_HOME}

# Creating directory for temp files.
TMP_LOCATION="$COHESITY_VIEW/tmp/"
sudo mkdir -p ${TMP_LOCATION}
mod_owner_group_and_permission ${TMP_LOCATION}

log_message "Switch SQL file located at ${SWITCH_SQL_FILENAME}"
log_message "### Printing content of ${SWITCH_SQL_FILENAME}"
cat ${SWITCH_SQL_FILENAME} >> ${LOG_FILE}

# Init Automatic Diagnostic Repository (ADR).
ADR_LOCATION=${DB_DIAG_LOCATION}/${DEST_NEW_NAME}
log_message "Setting up ADR at ${ADR_LOCATION}."
sudo mkdir -p ${ADR_LOCATION}
sudo mkdir -p ${ADR_LOCATION}/alert
sudo mkdir -p ${ADR_LOCATION}/trace
sudo mkdir -p ${ADR_LOCATION}/cdump
sudo mkdir -p ${ADR_LOCATION}/incident

mod_owner_group_and_permission ${ADR_LOCATION}
log_message "ADR setup done."
dump_progress "10"

echo "--------------------------------------------------------------------" >> ${LOG_FILE}

# Creating initora file.
TMP_PFILE="/tmp/init${DEST_NEW_NAME}.ora"
WHITELISTED_PATH_LIST+=( "${TMP_PFILE}" )
safe_rm_force "__USE_SUDO__" "$TMP_PFILE"
echo "db_name='${SRC_DB_NAME}'" > ${TMP_PFILE}
echo "db_unique_name=${DEST_NEW_NAME}" >> ${TMP_PFILE}
echo "UNDO_MANAGEMENT=MANUAL" >> ${TMP_PFILE}
if [ ! -z $NUM_DATA_FILES ]; then
  echo "db_files=${NUM_DATA_FILES}" >> ${TMP_PFILE}
  echo "open_cursors=${NUM_DATA_FILES}" >> ${TMP_PFILE}
  log_message "### Setting number of datafiles and open_cursors to ${NUM_DATA_FILES}";
fi
echo "db_create_file_dest='${COHESITY_VIEW}'" >> ${TMP_PFILE}
echo "db_recovery_file_dest='${COHESITY_VIEW}/fast_recovery_area'" >> ${TMP_PFILE}
if [ ! -z ${SGA_TARGET_SIZE} ]; then
  # If SGA_TARGET_SIZE is zero, skip adding it to pfile.
  if [[ ! ${SGA_TARGET_SIZE} =~ ^[0*] ]]; then
    echo "sga_target='${SGA_TARGET_SIZE}'" >> ${TMP_PFILE}
  else
    log_message "Not adding sga_target='${SGA_TARGET_SIZE}' to pfile."
  fi
fi

if [ ! -z ${SHARED_POOL_SIZE} ]; then
  # If SHARED_POOL_SIZE is zero, skip adding it to pfile.
  if [[ ! ${SHARED_POOL_SIZE} =~ ^[0*] ]]; then
    echo "shared_pool_size=${SHARED_POOL_SIZE}" >> $TMP_PFILE
  else
    log_message "Not adding shared_pool_size=${SHARED_POOL_SIZE} to pfile."
  fi
fi
echo "db_recovery_file_dest_size='${FRA_SIZE}M'" >> ${TMP_PFILE}
echo "undo_tablespace='UNDOTBS1'" >> ${TMP_PFILE}
echo "control_files=${CONTROL_FILES}" >> ${TMP_PFILE}
echo "diagnostic_dest=${COHESITY_VIEW}" >> ${TMP_PFILE}
echo "audit_file_dest=${ADUMP_LOCATION}" >> ${TMP_PFILE}
echo "cluster_database='FALSE'" >> ${TMP_PFILE}
echo "compatible=${SRC_DB_VERSION}" >> ${TMP_PFILE}
sudo chmod 777 ${TMP_PFILE}
export ORACLE_HOME=${DEST_ORACLE_HOME}
export NLS_DATE_FORMAT='DD:MM:YYYY-HH24:MI:SS'
sudo cp ${TMP_PFILE} ${ORACLE_HOME}/dbs/
P_FILE="${ORACLE_HOME}/dbs/init${DEST_NEW_NAME}.ora";
sudo chown ${ORACLE_USER}:${ORACLE_GROUP} ${P_FILE}
log_message "### Pfile created at ${P_FILE}, Starting database in 'mount' mode...";
dump_progress "20"

WHICH_OS=$(uname -a | cut -d " " -f 1)

if [ ${WHICH_OS} = "AIX" ]; then
  log_message "Sleeping for ${SLEEP_SEC} seconds after attribute modification ....."
  sleep ${SLEEP_SEC}
fi

# This script only uses OS authentication, just pass "/"
f_set_privilege "$DEST_ORACLE_HOME" "/"

echo "--------------------------------------------------------------------" >> ${LOG_FILE}

# After this point we want to handle the error.
set +e

# Starting new instance in mount mode.
export ORACLE_SID=${DEST_NEW_NAME}
$SQLPLUS_PATH / $sqlplus_privilege << EOF >> ${LOG_FILE}
WHENEVER OSERROR EXIT 11;
WHENEVER SQLERROR EXIT 22;
startup mount;
quit;
EOF
exit_handler $? "##### Startup db in 'mount' failed."
log_message "### Instance ${DEST_NEW_NAME} started in mount mode.";
dump_progress "40"

echo "--------------------------------------------------------------------" >> ${LOG_FILE}

# Running SQL to switch datafile/redolog file.
log_message "### Starting switch datafile/logfile";
$SQLPLUS_PATH / $sqlplus_privilege @${SWITCH_SQL_FILENAME} >> ${LOG_FILE}
exit_handler $? "##### Switch datafile/logfile failed for SID $ORACLE_SID"
log_message "### Finished switch datafile/logfile. Starting recovery...";

dump_progress "50"

echo "--------------------------------------------------------------------" >> ${LOG_FILE}

# Creating spfile.
$SQLPLUS_PATH / $sqlplus_privilege << EOF >> ${LOG_FILE}
WHENEVER OSERROR EXIT 11;
WHENEVER SQLERROR EXIT 22;
create spfile from pfile;
shutdown immediate;
STARTUP NOMOUNT;
ALTER SYSTEM SET DB_NAME=${DEST_NEW_NAME} SCOPE=SPFILE;
create pfile from spfile;
alter database mount;
exit;
EOF
exit_handler $? "##### SPFile creation failed."
log_message "### SPFile created successfully.";
dump_progress "60"

echo "--------------------------------------------------------------------" >> ${LOG_FILE}

#TODO(Kiran): Uncomment below code if NID required.

# Renaming the clone database. Please note that 'nid target=/' is broken in 11.2.0.3 with following Oracle bug reference.
#-- Bug 13366202 - DBNEWID [ nid ] does not allow TARGET=/ (NID-106) (Doc ID 13366202.8) --
# This bug has been fixd in 11.2.0.4 onwards.

# Make new LOG file for DB rename. nid utility cannot append to log file.
# NID_LOG_FILE=${LOG_FILE_PATH}/db_rename_${ID}.INFO

# log_message "### Renaming the database using 'nid' utility for user : ${ORACLE_USER}"
# ${ORACLE_HOME}/bin/nid TARGET=/ DBNAME=${DEST_NEW_NAME} LOGFILE=${NID_LOG_FILE};
# exit_handler $? "##### Rename failed."
# log_message "### Rename using 'nid' successful. Opening the database with RESETLOGS...";
# dump_progress "80"
# echo "--------------------------------------------------------------------" >> ${LOG_FILE}

# Check to leave the DB is open mode. if no open (Leave db is recovery mode) is
# selected, then the database will be left in mount mode.
if [ -z ${NO_OPEN} ]; then
  # Updating spfile and pfile with new name and starting database in specified mode.
  $SQLPLUS_PATH / $sqlplus_privilege << EOF >> ${LOG_FILE}
  WHENEVER OSERROR EXIT 11;
  WHENEVER SQLERROR EXIT 22;
  ALTER DATABASE OPEN;
  exit;
EOF
  exit_handler $? "##### Clone of clone failed on $ORACLE_SID at: ${RESTORE_TIME}"
  log_message "### Database ${DEST_NEW_NAME} started.";
fi

log_message "###NO_OPEN:: ${NO_OPEN}"

log_message "##### Clone of clone  completed on $ORACLE_SID at: ${RESTORE_TIME}";
dump_progress "90"
# Updating oratab file.
echo "$DEST_NEW_NAME:$ORACLE_HOME:N" | sudo tee -a /etc/oratab;

exit 0
