/**
 * Copyright 2018 Cohesity Inc.
 *
 * Author : Vivek Velankar
 *
 * Fetches the information for the database.
 *
 * The sql query returns a single column value, which is the
 * result of squashing all column values. This way we can avoid
 * dealing with formating the sql query output, which is difficult
 * to do. SQLPLUS does not do much formatting for concatendated
 * column values, so a returned row looks like the following:
 * -----------------------
 * The below line is on a single line, which is broken into 4 lines to
 * show a complete example:
 * >>>>>>>>>>>>>>>>>>>>>
 * 12.1.0.2.0#__ch__#ARCHIVELOG#__ch__#DISABLED#__ch__#7878672384#__ch__#
 * DEV#__ch__#dev#__ch__#4104508493#__ch__#PRIMARY#__ch__#0#__ch__#TRUE
 * __ch__#2368M#__ch__#0#__ch__#1#__ch__##__ch__# (ADDRESS=(PROTOCOL=TCP)
 * (HOST=10.2.203.119)(PORT=1521))#__ch__#Xeng.cohesity.com
 * <<<<<<<<<<<<<<<<<<<<<
 * -----------------------
 *
 * Note:
 * Please follow this pattern when introducing new columns:
 *   select /+ RULE/
 *    >>>>>>>>>>>>>
 *    existing columns
 *    <<<<<<<<<<<<<
 *    ||'&column_separator'||
 *    -- Column#: <next column number>
 *    <A new column goes here>
 *   from GV$database gdb;
 *
 * New appoarch for db_info file, with oracle 18 and 19 versions, the scripts
 * can potentially fail because of the pdb specific parameter, making the script
 * into a pl/sql block so that we can add more conditional statements.
*/
define common_path = '&5'
define column_separator = &4
@"&common_path/common.sql"

SET SERVEROUTPUT ON
SET AUTOPRINT ON;

/* refcursor to print the values to stdout */

var v_rc refcursor;
set echo on;
DECLARE
  sql_stmt   VARCHAR2(32767);
  cdb_clause VARCHAR2(100);
  V_CDB   varchar(3);
  V_VERSION INT;
BEGIN
  /* Check for pdb/cdb support and add a clause accordingly */
  select regexp_substr(version, '[0-9]+') into V_VERSION from v$instance;
  if V_VERSION >= 12 THEN
    /* Use Execute immediate to avoid pl/sql compile time error
     in older oracle versions. */
    execute immediate 'SELECT /*+ RULE*/ CDB FROM V$DATABASE' into V_CDB;
    IF V_CDB = 'YES' THEN
      cdb_clause := ' and con_id = 0 ';
    END IF;
  END IF;

  sql_stmt := '
  select /*+ RULE*/
  -- Eliminate any possible line feeds (carriage return characters)
  -- start REPLACE(REPLACE( <column>, CHR(10) ), CHR(13) )
  replace(replace(
  -- Column#: 1
  (select /*+ RULE*/ version from V$INSTANCE)
  ||''&column_separator''||
  -- Column#: 2
  gdb.LOG_MODE
  ||''&column_separator''||
  -- Column#: 3
  TO_CHAR((select /*+ RULE*/ STATUS from V$BLOCK_CHANGE_TRACKING))
  ||''&column_separator''||
  -- Column#: 4      "Size in Bytes"
  TO_CHAR(( select sum(BYTES) data_size from V$DATAFILE ) +
  ( select /*+ RULE*/ sum(BYTES) redo_size from V$LOG ) +
  ( select /*+ RULE*/ sum(BLOCK_SIZE*FILE_SIZE_BLKS) controlfile_size from V$CONTROLFILE))
  ||''&column_separator''||
  -- Column#: 5
  gdb.NAME
  ||''&column_separator''||
  -- Column#: 6
  gdb.DB_UNIQUE_NAME
  ||''&column_separator''||
  -- Column#: 7
  gdb.DBID
  ||''&column_separator''||
  -- Column#: 8
  gdb.DATABASE_ROLE
  ||''&column_separator''||
  -- Column#: 9
  TO_CHAR((select /*+ RULE*/ count(*) from v$archive_dest where status=''VALID'' and TARGET=''STANDBY''))
  ||''&column_separator''||
  -- Column#: 10     "RAC OR NOT"
  (select /*+ RULE*/ trim(VALUE) from V$PARAMETER where upper(name)=''CLUSTER_DATABASE'')
  ||''&column_separator''||
  -- Column#: 11     "SGA_TARGET"
  (select /*+ RULE*/ trim(display_value) from V$SYSTEM_PARAMETER where upper(name)=''SGA_TARGET'' and
  rownum=1 ' || cdb_clause || ')
  ||''&column_separator''||
  -- Column#: 12     "SHARED_POOL_SIZE"
  (select /*+ RULE*/ trim(display_value) from V$SYSTEM_PARAMETER where upper(name)=''SHARED_POOL_SIZE'' and
  rownum=1 ' || cdb_clause || ')
  ||''&column_separator''||
  -- Column#: 13
  (select /*+ RULE*/ max(FILE#) from V$TEMPFILE)
  ||''&column_separator''||
  -- Column#: 14
  (select /*+ RULE*/ space_limit from v$RECOVERY_FILE_DEST)
  ||''&column_separator''||
  -- Column#: 15      "PORTNUM"
  (select /*+ RULE*/ trim(value) from V$SYSTEM_PARAMETER where upper(name)=''LOCAL_LISTENER'' ' || cdb_clause || ')
  ||''&column_separator''||
  -- Column#: 16      "DOMAINNAME"
  (select /*+ RULE*/ ''X'' || trim(value) from V$SYSTEM_PARAMETER where upper(name)=''DB_DOMAIN'' ' || cdb_clause || ')
  ||''&column_separator''||
  -- Column#: 17      "TDE support"
  TO_CHAR((select /*+ RULE*/ count(*) from V$ENCRYPTED_TABLESPACES where ENCRYPTEDTS =''YES'' and rownum <= 1))
  ||''&column_separator''||
  -- Column#: 18      "DB Open Mode"
  gdb.open_mode
  -- Please do NOT add columns beyond this line: insert them before this line!
  --
  -- end REPLACE(REPLACE( <column>, CHR(10) ), CHR(13) )
  , CHR(10) ), CHR(13) )
  from GV$database gdb
  ';
  open :v_rc for sql_stmt;
END;
/
quit;
