#!/bin/bash
# Copyright 2018 Cohesity Inc.
#
# Author: Sandeep Tandekar
#
# This is a wrapper script to drop the database.
# It also appends the output at /tmp/drop_db_{SID}.INFO file.
#
# Exit from the script if any of the step fails.
# drop_db.sh -H $ORACLE_HOME -S $ORACLE_HOME/bin/sqlplus -A cs100 -H $ORACLE_HOME

set -e

LOG_FILE=""

# Which OS Cohesity agent is running on.
OS=$(uname -a | cut -d " " -f 1)

if [ ${OS} = "AIX" ]; then
  # TODO(aabhas): find replacement for readlink on AIX.
  current_dir=$(dirname $0)
else
  current_dir=$(dirname `readlink -f $0`)
fi

SCRIPT_NAME=`basename "$0"`
# Source needed library functions.
. $current_dir/common_utils.sh

usage()
{
  echo "Invoke the script with following arguments :-"
  echo -e "\t -H|--oracle_home                  Oracle home."
  echo -e "\t -S|--sqlplus-path                 Path to source SQLPLUS binary."
  echo -e "\t -A|--oracle-sid                   Oracle source database SID value."
  echo -e "\t -F|--log-file-path                Log file path."
  echo -e "\t -C|--cleanup                      Force cleanup."
  echo -e "\t -O|--oratab_list                  List of oratab file paths"
}

while getopts "H:S:A:F:C:O:" option; do
  case $option in
    H)
      ORACLE_HOME=$OPTARG;;
    S)
      SQLPLUS_PATH=$OPTARG;;
    A)
      SID=$OPTARG;;
    F)
      LOG_FILE_PATH=$OPTARG;;
    C)
      CLEANUP=$OPTARG;;
    O)
      ORATAB_LIST=$OPTARG;;
    *) echo "Invalid Option passed"; usage; exit 1 ;;
  esac
done

if [[ "x${LOG_FILE_PATH}" == "x" ]]; then
  LOG_FILE_PATH="/tmp"
fi
LOG_FILE=${LOG_FILE_PATH}/drop_db_${SID}_$$.INFO
# Update rman log file permissions.
touch $LOG_FILE

# Modify how to invoke Oracle binaries if the agent is installed with 'root'
setup_invoke_oracle_binaries

if [ -z "${SQLPLUS_PATH}" ] || [ -z "${SID}" ] ||  [ -z "${ORACLE_HOME}" ]; then
  log_message "SQL configuration parameters missing."
  log_message "#####${SQLPLUS_PATH}, ${SID}, ${ORACLE_HOME}";
  usage;
  exit 1;
fi

# This script only uses OS authentication, just pass "/"
f_set_privilege "$ORACLE_HOME" "/"

# Exporting PATH and LD_LIBRARY_PATH.
export NLS_DATE_FORMAT='DD:MM:YYYY-HH24:MI:SS'
export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:${DEST_ORACLE_HOME}/lib"
export PATH="$PATH:${DEST_ORACLE_HOME}/bin"

# After this point we want to handle the error.
set +e

start_time="`date "+%D %X"`"
log_message "### ============== Drop database :${SID} started at : ${start_time} ============="

export ORACLE_HOME=${ORACLE_HOME}
export ORACLE_SID=${SID}
# Shutdown and drop database
$SQLPLUS_PATH / $sqlplus_privilege << EOF >> ${LOG_FILE}
WHENEVER OSERROR EXIT 11
WHENEVER SQLERROR EXIT 22
shutdown abort;
startup mount exclusive restrict;
DROP DATABASE;
quit;
EOF
result=$?

if [[ "${ORACLE_HOME}" == *"/" ]]; then
  WHITELISTED_PATH_LIST+=( "${ORACLE_HOME}.*${SID}.*" )
else
  WHITELISTED_PATH_LIST+=( "${ORACLE_HOME}/.*${SID}.*" )
fi

# Removal of remnants after drop database.
remove_remnants() {
  safe_rm_force "__USE_SUDO__" "${ORACLE_HOME}/dbs/init${SID}.ora"
  safe_rm_force "__USE_SUDO__" "${ORACLE_HOME}/dbs/hc_${SID}.dat"
  safe_rm_force "__USE_SUDO__" "${ORACLE_HOME}/dbs/lk$(echo $SID | awk '{print toupper($0)}')"
}

remove_entry_from_oratab() {
  IFS=',' read -r -a ORATAB_FILES <<< "$ORATAB_LIST"
  # TODO(Aabhas) Need to handle the case where there exists multiple oratab files.
  ORATAB=${ORATAB_FILES[0]}
  # check if oratab db entry exists.
  grep -wq "${SID}:" $ORATAB
  if [ $? -eq 0 ]; then
    # remove line containing drop db entry.
    sudo sed -i "/${SID}:/d" $ORATAB
    # check if entry is removed.
    grep -wq "${SID}:" $ORATAB
    if [ $? -eq 0 ]; then
      log_message "Error removing oratab entry for db:$SID"
    else
      log_message "Successfully removed oratab entry for db:$SID"
    fi
  else
    log_message "Oratab entry not found for db $SID.";
  fi
}

end_time="`date "+%D %X"`"
if [ ${result} -eq 0 ];then
  remove_remnants
  log_message "##### Drop database :$SID completed. Start time :$start_time, End time :$end_time";
  remove_entry_from_oratab
else
  if [ ! -z "${CLEANUP}" ]; then
    log_message "##### ============= Trying to force cleanup of database files ============="
    safe_rm_force "__USE_SUDO__" "${ORACLE_HOME}/dbs/spfile${SID}.ora"
    safe_rm_force "__USE_SUDO__" "${ORACLE_HOME}/dbs/snapcf_${SID}.f"
    remove_remnants
    remove_entry_from_oratab
  fi
  log_message "##### Drop database :$SID failed. Start time :$start_time, End time :$end_time, Exit status :$result";
fi

exit ${result}
