#!/bin/bash
# Copyright 2020 Cohesity Inc.
#
# Author: Steve Lee
#
# This is the shell script that handles the killing of RMAN sessions.
#
export SQLPLUS_PATH=$1
export CONNECT_STRING=$2
export OSERROR=$3
export SQLERROR=$4
export SID_SERIAL_LIST=$5
export COL_SEPARATOR=$6
export COMMON_PATH=$7
export OSPID_LIST=$8

# Parse $SID_SERIAL_LIST
# It should be a list of '#' separated number pairs E.g.
#
#  24,138#14,284#134,23
#
# Each 2 values comma separated list describes one RMAN session in the
# following format:
#
#  <Session ID>,<Session number>
#
IFS='#' read -r -a sid_serial_vec <<< "$SID_SERIAL_LIST"

#
# Loop through all sessions and disconnect them. Session should have
# status "KILLED" after disconnected. ALTER SYSTEM DISCONNECT SESSION
# is preferred over ALTER SYSTEM KILL SESSION. The latter command
# only sends the kill request to the session and leave it up to the
# session to choose when to terminate itself. DISCONNECT on the other
# hand kill the Oracle foreground process for that session immediately.
for index in "${!sid_serial_vec[@]}"
do
    echo "Attempt to request Oracle to disconnect and kill the session: ${sid_serial_vec[$index]}"
    # Execute sqlplus to run DISCONNECT SESSION for each RMAN session, followed
    # by KILL SESSION just to make sure.
    $SQLPLUS_PATH -S <<EOF
        $CONNECT_STRING
        @$COMMON_PATH/common.sql $OSERROR $SQLERROR "dummy" $COL_SEPARATOR
        SET SERVEROUTPUT ON
        BEGIN
          -- Use 'IMMEDIATE' to disconnect session without waiting for
          -- in-flight transaction (if any) to finish. This is an RMAN session.
          -- We do not expect it will be in the middle of a database transaction.
          EXECUTE IMMEDIATE 'ALTER SYSTEM DISCONNECT SESSION ''${sid_serial_vec[$index]}'' IMMEDIATE';
        EXCEPTION
          -- Best try effort to disconnect session. If we hit errors, ignores it.
          WHEN others THEN
            DBMS_OUTPUT.PUT_LINE('ORA error when disconnect session. Skip session ${sid_serial_vec[$index]}.');
        END;
        /
        BEGIN
          -- Send KILL SESSION signal to the RMAN process request the process to die
          EXECUTE IMMEDIATE 'ALTER SYSTEM KILL SESSION ''${sid_serial_vec[$index]}'' IMMEDIATE';
        EXCEPTION
          -- Best try effort to kill session. If we hit errors, ignores it.
          WHEN others THEN
            DBMS_OUTPUT.PUT_LINE('ORA error when kill session. Skip session ${sid_serial_vec[$index]}.');
        END;
        /
EOF
done

#
# Issue OS 'kill -9' to remove the top level RMAN process.
#
# This will work for MNMC even there are remote RMAN channels created to
# perform the backup. Magneto slave uses the SCAN address to land itself
# to one of the RAC node and start coordinating backup with that Agent.
# That particular Agent allocates remote RMAN channel via SQLNet to activate
# other remote RMAN channels to participate in the backup. The top level OS
# process is on this Agent host. We just need to kill this top level process.
#
IFS=',' read -r -a ospid_vec <<< "${OSPID_LIST}"
#
# Loop through unique RMAN top level OS pid
#
# Attempt to kill the OS process directly. This is a best attempt. If the
# Agent is run as 'root' or the 'oracle' user, kill will work. Otherwise,
# we will just have to leave the top level RMAN process. Normally, the
# above combination of DISCONNECT SESSION, then KILL SESSION should
# have already killed the RMAN process.
for index in "${!ospid_vec[@]}"
do
    pid=${ospid_vec[$index]}
    # Sanity check if it is a number
    [ "$pid" -eq "$pid" ] 2>/dev/null || continue
    echo "Attempt to kill RMAN process with pid: $pid"

    # Best attempt to kill the OS process now
    set +e
    kill -9 ${pid}
    set -e
done

exit 0
