#!/bin/bash
# Copyright 2018 Cohesity Inc.
#
# Author: Soumya Poddar
#
# This is a wrapper script to take care of the operations needed after the
# alternate restore of Oracle DB
#
# Currently following operations are handled in this script.
#
# 1. Disable archivelog if mentioned.
# 2. Set the mentioned BCT file path.
#
# It also appends the output at /tmp/rman_post_restore_shell.INFO file
# Exit from the script if any of the step fails.
set -e

LOG_FILE=""

# Which OS Cohesity agent is running on.
OS=$(uname -a | cut -d " " -f 1)

if [ ${OS} = "AIX" ]; then
  # TODO(aabhas): find replacement for readlink on AIX.
  current_dir=$(dirname $0)
else
  current_dir=$(dirname `readlink -f $0`)
fi

SCRIPT_NAME=`basename "$0"`
# Source needed library functions.
. $current_dir/common_utils.sh

##### Main

usage()
{
  echo "Invoke the script with following arguments :-"
  echo -e "\t -R|--rman-path                    Path to RMAN binary."
  echo -e "\t -S|--sqlplus-path                 Path to source SQLPLUS binary."
  echo -e "\t -D|--dest-db-name                 Oracle destination database/SID value."
  echo -e "\t -L|--log-file-path                Rman log file path."
  echo -e "\t -H|--dst-oracle_home              Destination oracle home."
  echo -e "\t -B|--dst-oracle_base              Destination oracle base."
  echo -e "\t -I|--id                           Operation ID."
  echo -e "\t -b|--bct_file_path                BCT file location."
  echo -e "\t -e|--disable_archive_log_mode     If set, disable archive log mode."
}


# Variables with default value set.
BCT_FILE_PATH=""

while getopts "R:S:D:L:H:B:I:b:e" option; do
  case $option in
    R)
      RMAN_PATH=$OPTARG;;
    S)
      SQLPLUS_PATH=$OPTARG;;
    D)
      DEST_NEW_NAME=$OPTARG;;
    L)
      LOG_FILE_PATH=$OPTARG;;
    H)
      DEST_ORACLE_HOME=$OPTARG;;
    B)
      DEST_ORACLE_BASE=$OPTARG;;
    I)
      ID=$OPTARG;;
    b)
      BCT_FILE_PATH=$OPTARG;;
    e)
      IS_LOG_MODE_DISABLED=1;;
    *) echo "Invalid Option passed"; usage; exit 1 ;;
  esac
done

export ORACLE_HOME=$DEST_ORACLE_HOME

if [[ "x$LOG_FILE_PATH" == "x" ]]; then
  LOG_FILE_PATH="/tmp"
fi
LOG_FILE=${LOG_FILE_PATH}/rman_shell_${ID}.INFO
# Update rman log file permissions.
sudo touch $LOG_FILE
sudo chmod 766 $LOG_FILE

# Modify how to invoke Oracle binaries if the agent is installed with 'root'
setup_invoke_oracle_binaries

log_message "Shell Environment:"
log_message "`env`"

start_time="`date "+%D %X"`"
log_message "Post Restore Operation started at: $start_time"

if [ -z "$RMAN_PATH" ] || [ -z "$SQLPLUS_PATH" ] || [ -z "$DEST_NEW_NAME" ]; then
  log_message "Configuration parameters missing."
  log_message "#####$RMAN_PATH, $SQLPLUS_PATH, $DEST_NEW_NAME";
  usage;
  exit 1;
fi

if [ -z "$DEST_ORACLE_HOME" ] || [ -z "$DEST_ORACLE_BASE" ]; then
  log_message "DST Oracle home/base parameter is missing.";
  log_message "#####$DEST_ORACLE_HOME, $DEST_ORACLE_BASE";
  usage;
  exit 1;
fi

if [ ! -f ${DEST_ORACLE_HOME}/bin/oracle ]; then
  log_message "Unable to find ${DEST_ORACLE_HOME}/bin/oracle"
  exit 1;
fi

oracle_bin_permission=`ls -l ${DEST_ORACLE_HOME}/bin/oracle`
if [ $? -ne 0 ]; then
  log_message "Unable to find permissions for: ${DEST_ORACLE_HOME}/bin/oracle"
  exit 1;
fi

ORACLE_USER=`echo ${oracle_bin_permission} | awk '{print $3}'`
ORACLE_GROUP=`echo ${oracle_bin_permission} | awk '{print $4}'`

if [ -z ${ORACLE_USER} ] || [ -z ${ORACLE_GROUP} ]; then
  log_message "ORACLE_USER(${ORACLE_USER}) or ORACLE_GROUP(${ORACLE_GROUP}) not set."
  exit 1;
fi

export ORACLE_SID=$DEST_NEW_NAME
export ORACLE_HOME=$DEST_ORACLE_HOME

# This script only uses OS authentication, just pass "/"
f_set_privilege "$DEST_ORACLE_HOME" "/"

# Exporting PATH and LD_LIBRARY_PATH.
export NLS_DATE_FORMAT='DD:MM:YYYY-HH24:MI:SS'
export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:${DEST_ORACLE_HOME}/lib"
export PATH="$PATH:${DEST_ORACLE_HOME}/bin"


# Disabling archive log mode on user selection
if [ ! -z ${IS_LOG_MODE_DISABLED} ]; then
 $SQLPLUS_PATH / $sqlplus_privilege << EOF
 shutdown immediate;
 startup mount;
 alter database noarchivelog;
 alter database open;
 quit;
EOF
log_message  "ARCH log disable exited with status: $?"
fi

result=$?

# Enable block change tracking for restore database
if [ ! -z ${BCT_FILE_PATH} ] && [ ! -f ${BCT_FILE_PATH} ]; then
  log_message "Enabling BCT with file ${BCT_FILE_PATH}"
  $SQLPLUS_PATH / $sqlplus_privilege <<EOF
  ALTER DATABASE DISABLE BLOCK CHANGE TRACKING;
  ALTER DATABASE ENABLE BLOCK CHANGE TRACKING USING FILE '${BCT_FILE_PATH}';
  quit;
EOF
log_message "BCT tracking file set exited with status: $?"
fi

result=$?

if [ $result -eq 0 ];then
    log_message "#####Restore post op completed on $ORACLE_SID";
else
    log_message "#####Restore post op failed on $ORACLE_SID";
fi

exit $result
