#!/bin/bash
# Copyright 2019 Cohesity Inc.
#
# Author:Soumya Poddar
#

## ----------------------------------------------------------------------------

## *NIX Permission bits copied from man 2 stat, in Octal.

## S_ISUID=04000   #set-user-ID bit
## S_ISGID=02000   #set-group-ID bit (see below)
## S_ISVTX=01000   #sticky bit (see below)
##
## S_IRWXU=0700   #owner has read, write, and execute permission
## S_IRUSR=0400   #owner has read permission
## S_IWUSR=0200   #owner has write permission
## S_IXUSR=0100   #owner has execute permission
##
## S_IRWXG=070    #group has read, write, and execute permission
## S_IRGRP=040    #group has read permission
## S_IWGRP=020    #group has write permission
## S_IXGRP=010    #group has execute permission
##
## S_IRWXO=07     #others (not in group) have read, write, and execute permission
## S_IROTH=04     #others have read permission
## S_IWOTH=02     #others have write permission
## S_IXOTH=01     #others have execute permission

## ----------------------------------------------------------------------------

## *NIX Error codes.
ENOENT=2
EACCES=13
ENOSPC=28

## ----------------------------------------------------------------------------


# --------------------------------------------------------------------------------------------------

RDPERM=04      #Read permission [ This is same as RD perm for others ]
WRPERM=02      #Write permission [ This is same as WR perm for others ]
EXEPERM=01     #Execute permission [ This is same as EXE perm for Others ]

USR_ACCESS_SHIFT=6 #Shift RDPERM,WRPERM,EXEPERM by 2^6 to get User acess bits.
GRP_ACCESS_SHIFT=3 #Shift RDPERM,WRPERM,EXEPERM by 2^3 to get Group acess bits.

CommonOracleBinaryArray=(
"rman"
"sqlplus"
"nid"
)

RDBMS_LIB_LOC="rdbms/lib"


# According to the follownig Oracle metalink. The OS DBA groups are defined in
# $ORACLE_HOME/rdbms/lib/config.[cs] ( in AIX/Solaris config.s other *NIX
# platforms config.c).
#
# [ doc-id ] Doc ID 730067.1
# [ LINK ]https://support.oracle.com/epmos/faces/DocumentDisplay?id=730067.1&displayIndex=3&_adf.ctrl-state=bv4e5fwrq_73&_afrLoop=121418734686865
#
# Greping this file, to get the DBA group names.
#

ORACLE_RDBMS_CONFIG_FILE_NAME_LINUX="config.c"
ORACLE_RDBMS_CONFIG_FILE_NAME_AIX="config.s"

UNIQ_ORACLE_HOMES=()
ORACLE_DBA_GROUP_LIST=()
TR_BIN_LOC=$(which tr)

# --------------------------------------------------------------------------------------------------

# Add an element to a given array only if the element doesn't already exists
# there. Essentially creating a SET.
# The SET w/wo modification is printed, so that the caller function can
# update the array given as an argument.
f_add_in_array()
{
  new_elem=$1 && shift
  array=($@)
  for iter in ${array[@]}; do
      if [ $iter == $new_elem ]; then
         echo ${array[@]}
         return
      fi
  done
  array[${#array[@]}]=$new_elem
  echo ${array[@]}
}

# --------------------------------------------------------------------------------------------------

## Check If the Source DB version is same with the Oracle in the current host. The version check
## is done by getting version from SQLPLUS binary residing in the ${ORACLE_HOME}/bin.
f_oracle_version_check() {
  if [ ! -f $ORACLE_HOME/bin/sqlplus ]; then
      log_message "Unable to find $ORACLE_HOME/bin/sqlplus"
      f_mark_script_failed_with_reason "Unable to find $ORACLE_HOME/bin/sqlplus"
      return;
  fi

  local oracle_sqlplus_version=$($ORACLE_HOME/bin/sqlplus -V | grep Release | cut -d " " -f 3)
  if [ ${oracle_sqlplus_version} != ${SRC_DB_VERSION} ]; then
      log_message "Oracle version mismatch ${oracle_sqlplus_version} VS ${SRC_DB_VERSION}"
      f_mark_script_failed_with_reason "Oracle version mismatch ${oracle_sqlplus_version} VS ${SRC_DB_VERSION}"
      return;
  fi
}

# --------------------------------------------------------------------------------------------------

## Check if the Source DB version is over 12c. The version check
## is done by getting version from SQLPLUS binary residing in the ${ORACLE_HOME}/bin.
f_oracle_set_privilege() {
  if [ ! -f $ORACLE_HOME/bin/sqlplus ]; then
      log_message "Unable to find $ORACLE_HOME/bin/sqlplus"
      f_mark_script_failed_with_reason "Unable to find $ORACLE_HOME/bin/sqlplus"
      return 1;
  fi

  if [[ "X${RMAN_PATH}" == "X" ]]; then
    log_message "RMAN_PATH is not set, cannot choose between SYSDBA or SYSBACKUP"
    return 1;
  fi

  local local_connect_string="$1"

  local use_sysbackup=0
  local oracle_sqlplus_version=$($ORACLE_HOME/bin/sqlplus -V | grep Release | cut -d " " -f 3)
  IFS='.' read -ra ADDR <<< "$oracle_sqlplus_version"
  if [ ${ADDR[0]} -gt 12 ] || { [ ${ADDR[0]} -eq 12 ] && [ ${ADDR[1]} -ge 1 ]; }; then
    # Test if we can connect with SYSBACKUP, if not, fallback automatically to
    # sysdba. Some customer environment that is >=12c may not be able to
    # set up SYSBACKUP in their OS authentication or DB authentication. For
    # example, if SYSBACKUP OS group is not specified when the oracle image is
    # built previously, this will require production database down time to
    # add the SYSBACKUP OS group and relink of the Oracle binary. Customer
    # may not be able to do that. Cohesity should not impose SYSBACKUP as a
    # hard requirement with database >=12c.
    $RMAN_PATH <<EOF > /dev/null
      connect target '$local_connect_string as sysbackup';
EOF
    [ $? = 0 ] && use_sysbackup=1
  fi

  if [ ${use_sysbackup} -eq 1 ]; then
    sqlplus_privilege="as sysbackup"
    rman_privilege=" as sysbackup"
    return 0;
  else
    sqlplus_privilege="as sysdba"
    # 11c don't support 'as sysdba when using RMAN'
    rman_privilege=""
    return 1;
  fi
}

# --------------------------------------------------------------------------------------------------

## Check If the a given file has READ/WRITE [ in some case EXCUTE ] permission bit set.
## The Permission bits are passed as arguments.
f_check_perm() {
  local file_path=$1
  local permission=""
  if [ ${OS} = "AIX" ]; then
    # Taken from http://agileadam.com/2011/02/755-style-permissions-with-ls/
    # for example it will iterate over "-rw-rw-r--" given from ls -ld command
    # and convert them to their respective octal values.
    # "-rw-rw-r--" -> respective values of k at each value of i is
    # i=0 k=400
    # i=1 k=600
    # i=2 k=600
    # i=3 k=640
    # i=4 k=660
    # i=5 k=660
    # i=6 k=664
    # i=7 k=664
    # i=8 k=664

    local ls_out=$(ls -ld ${file_path} | awk '{
      k=0;
      for(i=0;i<=8;i++)
        k+=((substr($1,i+2,1)~/[rwx]/)*2^(8-i));
        if(k)
          printf("%0o\n",k);
        }';)
    permission=0$(echo ${ls_out})
  else
    local stat_out=$(stat -L --format=%a ${file_path})
    permission=0$(echo ${stat_out})
  fi
  local R_PERM=$2
  local W_PERM=$3
  local EXE_PERM=$4
  local is_exe_perm_needed=false
  local result=0
  local perm_user_or_group=$5

  if [ -d ${file_path} ]; then
     is_exe_perm_needed=true;
  fi

  if [ $R_PERM -ne 0 ] && [ $(($permission & $R_PERM)) -eq 0 ]; then
     log_message "${perm_user_or_group} Do not have RD access to ${file_path}"
     result=1
  fi

  if [ $W_PERM -ne 0 ] && [ $(($permission & $W_PERM)) -eq 0 ]; then
     log_message "${perm_user_or_group} Do not have WR access to ${file_path}"
     result=1
  fi

  if [ $EXE_PERM -ne 0 ] && [ $(($permission & $EXE_PERM)) -eq 0 ]; then
     log_message "${perm_user_or_group} Do not have EX access to ${file_path}"
     result=1
  fi
  return ${result}
}

# --------------------------------------------------------------------------------------------------

# Check If a give Group is the owning Group of a file or not.
# Example output of comamd "stat -L format=%G ${file_path}" this command prints Owning Group name
# of the file.
#
# [oracle@OracleLinuxVM4 ~]$ stat -L --format=%U /u01/app/oracle/product/12.2.0.1/db_1
# oinstall
#
f_is_same_group() {
  local file_path=$1
  local group_name=$2
  local stat_out=""
  if [ ${OS} = "AIX" ]; then
    stat_out=$(istat  ${file_path}  | grep Owner | xargs | cut -d "(" -f3 | cut -d ")" -f1)
  else
    stat_out=$(stat -L --format=%G ${file_path})
  fi
  local result=0
  if [ $group_name != $stat_out ]; then
      result=1
      log_message "Group ${group_name} is not the same group of ${file_path}"
  fi
  return ${result}
}

# --------------------------------------------------------------------------------------------------

# Check If a given User is the owner of a file or not.
# Example output of comamd "stat -L format=%U ${file_path}" this command prints Owner User name of
# the file.
#
# [oracle@OracleLinuxVM4 ~]$ stat -L --format=%U /u01/app/oracle/product/12.2.0.1/db_1
# oracle
#
f_is_same_user() {
  local file_path=$1
  local owner_name=$2
  local stat_out=""
  if [ ${OS} = "AIX" ]; then
    stat_out=$(istat  ${file_path}  | grep Owner | xargs | cut -d "(" -f2 | cut -d ")" -f1)
  else
    stat_out=$(stat -L --format=%U ${file_path})
  fi
  local result=0
  if [ $owner_name != $stat_out ]; then
     result=1
     log_message "Owner ${owner_name} is not the same owner of ${file_path}"
  fi
  return ${result}
}

# --------------------------------------------------------------------------------------------------

## Check if it is possible to gain Read,Write [ in some case Execute ] permission of a given file.
##
## If strict checking is enabled then return failure if required user/group
## doesn't match or their required permission doesn't match.
##
##
## 1. Try, if the file can be accessed with Owner User priviledge [ ORACLE_USER ]
## 2. Try, if the file can be accessed with Owning Group priviledges.
## 3. Lastly try if the file can be accessed with Other [ common to all ] priviledges.
##
## If anyone of the above passed, return 0 (Success) else return non-zero.
f_check_file_can_be_accessed() {
    local file_path=$1
    local user=$2
    local group=$3
    local access_bits=$4
    local result=0

    local R_PERM=$(($access_bits & $RDPERM))
    local W_PERM=$(($access_bits & $WRPERM))
    local E_PERM=$(($access_bits & $EXEPERM))

    f_is_same_user ${file_path} ${user}
    if [ $? -eq 0 ]; then

      U_RDPERM=$(eval printf "%o" $(($R_PERM << $USR_ACCESS_SHIFT)))
      U_WRPERM=$(eval printf "%o" $(($W_PERM << $USR_ACCESS_SHIFT)))
      U_EXPERM=$(eval printf "%o" $(($E_PERM << $USR_ACCESS_SHIFT)))

      f_check_perm ${file_path} $U_RDPERM $U_WRPERM $U_EXPERM ${user}
      if [ $? -eq 0 ]; then
         return 0
      fi
      log_message "User ${user} do not have Pemission for ${file_path}"
    fi

    f_is_same_group ${file_path} ${group}
    if [ $? -eq 0 ]; then

      G_RDPERM=$(eval printf "%o" $(($R_PERM << $GRP_ACCESS_SHIFT)))
      G_WRPERM=$(eval printf "%o" $(($W_PERM << $GRP_ACCESS_SHIFT)))
      G_EXPERM=$(eval printf "%o" $(($E_PERM << $GRP_ACCESS_SHIFT)))

      f_check_perm ${file_path} $G_RDPERM $G_WRPERM $G_EXPERM ${group}
      if [ $? -eq 0 ] ; then
         return 0
      fi

      log_message "Group ${group} do not have Pemission for ${file_path}"
    fi

    f_check_perm ${file_path} $RDPERM $WRPERM $EXEPERM $(whoami)

    if [ $? -eq 0 ]; then
      log_message "Accessing file ${file_path} with permission for Others"
      return 0
    fi

    return 1
}

# --------------------------------------------------------------------------------------------------

# Given a path or set of paths, the following function checks the following.
#
#1. The given path exists or not, if exists it is valid or not.
#2. Can the file be accessed with the given access mode in the argument.
f_check_file_path_and_perm_UNIX_FS() {
  local file_path_list=$1
  local user=$2
  local group=$3
  local permission=$4 # Mode of access required.
  local is_dir=$5
  local context=$6

  local failure_msg="Access denied for"

  if [ ! -z ${context} ]; then
    failure_msg="$failure_msg ${context} ="
  fi

  IFS=',' read -r -a file_path_arr <<< "$file_path_list"

  for index in "${!file_path_arr[@]}"
  do
      log_message "Checking for path ... ${file_path_arr[$index]} .."
      if ${is_dir} && [ ! -d ${file_path_arr[$index]} ]; then
         log_message "Dir ${file_path_arr[$index]} Not found ${is_dir}"
         f_mark_script_failed_with_reason \
         "Dir ${context} = ${file_path_arr[$index]} either do not exists or Not a directory"
         return
      fi

      if ! ${is_dir} && [ ! -f ${file_path_arr[$index]} ]; then
         log_message "File ${file_path_arr[$index]} Not found"
         f_mark_script_failed_with_reason \
         "File ${context} = ${file_path_arr[$index]} either do not exists or Not a file"
         return
      fi

      f_check_file_can_be_accessed ${file_path_arr[$index]} ${user} \
      ${group} ${permission} ${strict_check}

      if [ $? -ne 0 ]; then
         log_message "Do not have access permission for ${file_path_arr[$index]}"
         f_mark_script_failed_with_reason "${failure_msg} ${file_path_arr[$index]}"
      fi

      ## TODO(soumyap):
      ## hash the permission check result.
      ## which might be used in the later result.
  done
  return
}

# --------------------------------------------------------------------------------------------------

# Grep for the DB pmon process in ps output.
f_check_for_running_db_process() {
  local db_name=$1
  local output=$(ps -ef| grep -v grep | grep -e "ora_pmon_${db_name}$" | awk '{print $2}')
  local result=1

  if [ ! -z ${output} ]; then
    result=0
    log_message "found ${db_name} in ps output ===> ${output}"
  fi
  return $result
}

# --------------------------------------------------------------------------------------------------

# Grep for the DB_SID in /etc/oratab file.
f_check_for_db_in_oratab() {
  local db_name=$1
  local oratab_file=$2
  log_message "searching for ${db_name} and in ${oratab_file}..."
  local output=$(grep -e "^${db_name}:" ${oratab_file})
  local result=1

  if [[ "x${output}" != "x" ]]; then
    result=0
    log_message "found ${db_name} in ${oratab_file}==> ${output}"
  fi
  return $result
}

# --------------------------------------------------------------------------------------------------

# Check any init<DBSID>.ora or spfile<DBSID> exists or not.
# Alternate restore or Clone will fail if an old DB already exists with same
# name.
f_check_same_DBSID() {
  local DBSID=$1
  local ORATAB_FILE_LOC=$2
  log_message "searching for ${DBSID}"
  f_check_for_running_db_process ${DBSID}
  local pmon_grep_result=$?
  f_check_for_db_in_oratab ${DBSID} ${ORATAB_FILE_LOC}
  oratab_grep_result=$?
  if [ $pmon_grep_result -eq 0 ] || [ $oratab_grep_result -eq 0 ]; then
     f_mark_script_failed_with_reason "DB ${DBSID} might already be \
         present in the system or in ${ORATAB_FILE_LOC}"
  fi
}

# --------------------------------------------------------------------------------------------------

# Assign and export the appropriate space check function, depending upon the
# path is in UNIX FS or in ASM.
f_get_space_check_func() {
  local path=$1

  if f_check_asm_path $1 ; then
    export f_check_available_space=f_check_available_space_ASM
  else
    export f_check_available_space=f_check_available_space_UNIX
  fi
}

# --------------------------------------------------------------------------------------------------

# Used for space checks. Give a path returns the free space in the filesystem
# in which the file/dir resides.
# Used for Alternate restore to check space for coping data/log files.
f_check_available_space_UNIX()
{
  local path=$1
  local space_needed=$2
  local df_string="df -P"
  local format_specifier=""

  if [ ! -d ${path} ]; then
    log_message "${path} do not exists on the system"
    return 2 #ENOENT
  fi

  if [ ${OS} = "AIX" ]; then
     format_specifier="g"
  else
     format_specifier="B 1G"
  fi

  space_remained=$(${df_string}${format_specifier} ${path} | tail -n 1 | awk '{print $4}')

  if [ ${OS} = "AIX" ]; then
    # converting to integer for AIX
    space_remained=$(echo $space_remained/1 | bc)
  fi

  if [ $space_remained -lt $space_needed ]; then
    return ${ENOSPC}
  fi

  log_message "Space needed ${space_needed}G and available space ${space_remained}G"
  return 0
}

# --------------------------------------------------------------------------------------------------

# Placeholder function for space checking in case ASM path is given.
f_check_available_space_ASM() {
  local path=$1
  local space_needed=$2
  log_message "The Alternate restore space $path is in ASM, skipping space check"
  return 0
}

# --------------------------------------------------------------------------------------------------

# Check that a given amount of memory passed as an argument is available in the
# in the system or not.
f_check_required_mem()
{
  local required_mem=$1

  if [ -z ${required_mem} ]; then
    log_message "Required mem is not passed memory check is not done"
    return 0
  fi

  f_check_required_mem_${OS} ${required_mem}
  return $?
}

# --------------------------------------------------------------------------------------------------

# Sample output for free command in Linux. We extract the amount of memory
# showing under Total.
#
#cohesity@ubuntu:~$ free -g
#              total        used        free      shared  buff/cache   available
#Mem:             15           1          12           0           1      13
#Swap:            15           0          15
f_check_required_mem_Linux()
{

  local required_mem=$1
  # Extract the last charecter in the given required memory.
  # the required_memory should look like 1024M/10G
  local switch=$(echo ${required_mem: -1} | awk '{print tolower($0)}')
  required_mem=$(echo "${required_mem}"|sed 's/.$//')

  # This is to print the memory with M/G/K
  local memory_suffix=$(echo $switch | awk '{print toupper($0)}')

  # Use the last charecter extracted as switch i.e. -m/-g in free command.
  local available_mem=$(free -${switch} | grep Mem |awk '{print $2}')
  if [ ${available_mem} -gt 0 ] && [ ${available_mem} -gt ${required_mem} ]; then
    return 0
  fi
  f_mark_script_failed_with_reason "Not enough memory: Avail \
  ${available_mem}${memory_suffix} Need: ${required_mem}${memory_suffix}"
  return 1
}

# --------------------------------------------------------------------------------------------------

# Sample output for svmon command in AIX. We extract the amount of memory
# showing under size.
#
# bash-4.4# svmon -G -O unit=GB
#Unit: GB
#--------------------------------------------------------------------------------------
#               size       inuse        free         pin     virtual  available   mmode
#memory         32.0        4.23        27.8        3.18        3.73       27.8     Ded
#pg space       0.50        0.01
#
#               work        pers        clnt       other
#pin            1.99           0           0        1.19
#in use         3.73           0        0.51

f_check_required_mem_AIX()
{
  local required_mem=$1
  # Extract the last charecter in the given required memory.
  # the required_memory should look like 1024M/10G
  local switch=$(echo ${required_mem: -1} | awk '{print $0}')
  required_mem=$(echo "${required_mem}"|sed 's/.$//')

  local available_mem=$(svmon -G -O unit=${switch}B | grep memory | xargs | cut -d " " -f2)
  # converting to integer for AIX
  available_mem=$(echo $available_mem/1 | bc)
  if [ ${available_mem} -gt 0 ] && [ ${available_mem} -gt ${required_mem} ]; then
    return 0
  fi
  f_mark_script_failed_with_reason "Not enough memory: Avail \
  ${available_mem}${switch} Need: ${required_mem}${switch}"
  return 1
}

# --------------------------------------------------------------------------------------------------

# Following function checks that a given PATH is ASM path or not.
# An ASM path will be staring with '+' sign.
f_check_asm_path() {
  local var1=$1
  if [[ $var1 == +* ]]; then
     log_message "Found ASM path ${var1} "
     return 0
  fi
  return 1
}

# --------------------------------------------------------------------------------------------------

# Placeholder function, for testing ASM path validation.
f_validate_asm_path() {
  local path=$1
  log_message "Skipping validation for ASM path $path"
}

# --------------------------------------------------------------------------------------------------

# Following function sets the appropriate function which should be used for
# permission checks for a given path.
#
# If the path is in UNIX fs them UNIX permission checks will be performed.
# If the path is in ASM, then permission checks are not implemented now.
f_get_path_check_function() {
  local path=$1
  if f_check_asm_path $1 ; then
    export f_check_file_path_and_perm=f_validate_asm_path
  else
    export f_check_file_path_and_perm=f_check_file_path_and_perm_UNIX_FS
  fi
}

# --------------------------------------------------------------------------------------------------

# Following function parses the "/etc/oratab" (configurable) file and call a
# callback function on each valid entry line of the "/etc/oratab".
# This function takes following arguments.
# 1. oratab file location.
# 2. callback function to be called.
# 3. extra arguemnts to be passed to the callback function.
f_parse_oratab_file() {
  local oratab_file_path=$1
  local f_name=$2

  # Shift the argv by 2 as first 2 arguemnt will be used by the parser.
  shift 2

  if [[ "$f_name" == "f_pass_function1" ]];then
     log_message "Calling the same function will result in infinite loop"
     return
  fi

  if [ ! -f $oratab_file_path ]; then
     log_message "Oratab file $oratab_file_path not present"
  fi

  log_message "Parsing $oratab_file_path as ORATAB_FILE"

  while IFS='' read -r line || [[ -n "$line" ]]; do

    # Commented line oratab file skip.
    if [[ "$line" =~ ^#.* ]]; then
       continue;
    fi

    # Empty line in oratab file skip.
    if [[ "x{$line}" == "x" ]]; then
       continue;
    fi

    oracle_sid=`echo $line | awk -F':' '{print $1}'`
    oracle_home=`echo $line | awk -F':' '{print $2}'`

    # empty ORACLE_HOME skip.
    if [[ "x${oracle_home}" == "x" ]]; then
       continue
    fi

    # empty ORACLE_SID skip.
    if [[ "x${oracle_sid}" == "x" ]]; then
       continue;
    fi

    # Calling callback function on the oratab entry.
    $f_name "$line" $@
  done < $oratab_file_path
}

# --------------------------------------------------------------------------------------------------

# Parse through each valid line in ORATAB (/etc/oratab) file.
# Add the Oracle home to SET which contains unique Oracle_Home paths.
f_adding_Uniq_Oracle_Home_arr() {
  local oratab_line=$1
  oracle_home=$(echo $oratab_line | awk -F':' '{print $2}')
  for iter in ${UNIQ_ORACLE_HOMES};do
      if [[ ${oracle_home} = $iter ]];then
         return
      fi
  done
  local rdbms_lib_path="${oracle_home}/${RDBMS_LIB_LOC}/"

  if [ ! -d ${oracle_home} ] || [ ! -d ${rdbms_lib_path} ]; then
     log_message "${oracle_home} is not a valid Oracle Home dir, hence skipping it"
     return
  fi

  ## TODO(soumyap) Check for +GRID entries and skip them.
  UNIQ_ORACLE_HOMES=($(f_add_in_array ${oracle_home} ${UNIQ_ORACLE_HOMES[@]}))
}

# --------------------------------------------------------------------------------------------------
# Get all the Oracle homes in a global SET. Which contain all unique Oracle
# Homes.

f_get_all_oracle_homes() {
  local oratab_file=$1
  f_parse_oratab_file $oratab_file f_adding_Uniq_Oracle_Home_arr
}

# --------------------------------------------------------------------------------------------------
# Go through each Oracle HOME (in case there are multiple installation of
# Oracle exits in a single system), parse config.[cs] file inside rdbms/lib
# to figure out the SET of unique OS group names which holds SYSDBA privileges.
f_get_all_dba_groups_from_ORACLE_HOME() {
  local oratab_file=$1

  if [ -z ${oratab_file} ]; then
     log_message "Missing Oratab file information, skipping DBA checks"
     return 1
  fi

  f_get_all_oracle_homes $oratab_file

  if [ ${#UNIQ_ORACLE_HOMES[@]} -eq 0 ]; then
     log_message "Not able to find the Oracle homes from ${oratab_file}"
     return 1
  fi

  log_message "Trying to find DBA adn BKP groups from ${UNIQ_ORACLE_HOMES[@]}"
  f_get_Oracle_DBA_group_list

  if [ ${#DBA_GROUP_LIST[@]} -eq 0 ]; then
     log_message "Not able find the DBA groups from configs files \
                  in ${UNIQ_ORACLE_HOMES[@]} Oracle homes"
     f_get_Oracle_BKP_group_list
     if [ ${#BKP_GROUP_LIST[@]} -eq 0 ]; then
        log_message "Not able find the BKP groups from configs files \
                      in ${UNIQ_ORACLE_HOMES[@]} Oracle homes"
        return 1
     fi
  fi



  log_message "List of Unique DBA groups .. ${DBA_GROUP_LIST[@]} "
  log_message "List of Unique BKP groups .. ${BKP_GROUP_LIST[@]} "
  return 0
}

# --------------------------------------------------------------------------------------------------
# In case of Linux the DBA group name is mentioned in the config.c file
# inside rdbms/lib.
f_get_dba_group_from_Oracle_home_Linux() {
  local Oracle_Home=$1
  local rdbms_lib_path="${Oracle_Home}/${RDBMS_LIB_LOC}/"
  local config_file_path="${rdbms_lib_path}/${ORACLE_RDBMS_CONFIG_FILE_NAME_LINUX}"
  local output=$(grep "define SS_DBA_GRP" ${config_file_path} | awk '{print $3}')

  if [ -z ${output} ]; then
     log_message "Failed to find SS_DBA_GROUP entry in file ${config_file_path}"
     return
  fi

  echo $output | ${TR_BIN_LOC} -d '"'
}

# --------------------------------------------------------------------------------------------------
# In case of Linux the DBA group name is mentioned in the config.c file
# inside rdbms/lib.
f_get_bkp_group_from_Oracle_home_Linux() {
  local Oracle_Home=$1
  local rdbms_lib_path="${Oracle_Home}/${RDBMS_LIB_LOC}/"
  local config_file_path="${rdbms_lib_path}/${ORACLE_RDBMS_CONFIG_FILE_NAME_LINUX}"
  local output=$(grep "define SS_BKP_GRP" ${config_file_path} | awk '{print $3}')

  if [ -z ${output} ]; then
     log_message "Failed to find SS_BKP_GROUP entry in file ${config_file_path}"
     return
  fi

  echo $output | ${TR_BIN_LOC} -d '"'
}

# --------------------------------------------------------------------------------------------------

f_get_Oracle_DBA_group_list() {
  for oracle_home in ${UNIQ_ORACLE_HOMES[@]}; do
      local dba_group_name=$(f_get_dba_group_from_Oracle_home_${OS} ${oracle_home})
      if [ -z ${dba_group_name} ]; then
         continue
      fi
      DBA_GROUP_LIST=($(f_add_in_array ${dba_group_name} ${DBA_GROUP_LIST[@]}))
  done
}

# --------------------------------------------------------------------------------------------------

f_get_Oracle_BKP_group_list() {
  for oracle_home in ${UNIQ_ORACLE_HOMES[@]}; do
      local bkp_group_name=$(f_get_bkp_group_from_Oracle_home_${OS} ${oracle_home})
      if [ -z ${bkp_group_name} ]; then
         continue
      fi
      BKP_GROUP_LIST=($(f_add_in_array ${bkp_group_name} ${BKP_GROUP_LIST[@]}))
  done
}

# --------------------------------------------------------------------------------------------------

f_cur_user_has_SYSDBA_PRIV() {
  local user_name=$1

  if [ -z ${user_name} ];then
     log_message "Empty username returning without check"
  fi

  local cohesity_agent_groups=$(id -Gn ${user_name})

  if [ $? -ne 0 ]; then
    log_message "Not a valid User id ${user_name} Not able to run group check"
    return 0
  fi

  if [[ "x" == "x${cohesity_agent_groups}" ]];then
     local msg="Empty group information found for the user.\
                User might not be part of DBA group"
     log_message "${msg}"

     ## Fail the script from here.
     f_mark_script_failed_with_reason "${msg}"
  fi

  if [ ${#DBA_GROUP_LIST[@]} -eq 0 ]; then
     log_message "DBA_GROUP_LIST empty please init the DBA_GROUP_LIST by calling \
                  f_get_all_dba_groups_from_ORACLE_HOME"
     return
  fi

  for group in ${DBA_GROUP_LIST[@]};do
      local grep_out=$(echo ${cohesity_agent_groups} | grep -w ${group})
      if [ -z ${grep_out} ];then
         local msg="Cohesityagent user ${user_name} is not part of DBA group ${group}"

         log_message "$msg"
         ## Fail the script from here.
         ## Not failing the check as we also need to check SYSBACKUP
         ## f_mark_script_failed_with_reason "${msg}"

         return 1
      fi
  done

  log_message "Cohesityagent user ${user_name} \
               is part of all the DBA groups ${DBA_GROUP_LIST[@]}"
  return 0
}

# --------------------------------------------------------------------------------------------------

f_cur_user_has_SYSBACKUP_PRIV() {
  local user_name=$1

  if [ -z ${user_name} ];then
     log_message "Empty username returning without check"
  fi

  local cohesity_agent_groups=$(id -Gn ${user_name})

  if [ $? -ne 0 ]; then
    log_message "Not a valid User id ${user_name} Not able to run group check"
    return 0
  fi

  if [[ "x" == "x${cohesity_agent_groups}" ]];then
     local msg="Empty group information found for the user.\
                User might not be part of BKP group"
     log_message "${msg}"

     ## Fail the script from here.
     f_mark_script_failed_with_reason "${msg}"
  fi

  if [ ${#BKP_GROUP_LIST[@]} -eq 0 ]; then
     log_message "BKP_GROUP_LIST empty please init the BKP_GROUP_LIST by calling \
                  f_get_all_dba_groups_from_ORACLE_HOME"
     return
  fi

  for group in ${BKP_GROUP_LIST[@]};do
      local grep_out=$(echo ${cohesity_agent_groups} | grep -w ${group})
      if [ -z ${grep_out} ];then
         local msg="Cohesityagent user ${user_name} is not part of BKP group ${group}"

         log_message "$msg"
         ## Fail the script from here.
         f_mark_script_failed_with_reason "${msg}"

         return 1
      fi
  done

  log_message "Cohesityagent user ${user_name} \
               is part of all the BKP groups ${BKP_GROUP_LIST[@]}"
  return 0
}

# --------------------------------------------------------------------------------------------------

# This is a callback function. This function takes a valid "/etc/oratab" entry
# and checks that the DB mentioned in the entry is currently running or not.
# Print in the STDOUT if the DB is running.
f_test_for_running_db() {
  local oratab_line=$1
  local db_running_string=$2
  oracle_sid=$(echo $oratab_line | awk -F':' '{print $1}')
  oracle_home=$(echo $oratab_line | awk -F':' '{print $2}')
  f_check_for_running_db_process $oracle_sid
  if [ $? -eq 0 ]; then
   log_message "found $oratab_line to be running"
   echo "$db_running_string $oracle_sid"
   return
  fi

  log_message "DB: $oracle_sid is not running"
}

# --------------------------------------------------------------------------------------------------

# Check for all the DBs which are currently running on the host.
# This is done by calling ORATAB parser with a callback function which
# checks that the.
f_check_for_all_dbs_running() {
  local db_running_string=$1
  log_message "Calling the test for Running DB"
  log_message "The DB running string is $db_running_string"
  f_parse_oratab_file $ORATAB_FILE f_test_for_running_db "$db_running_string"
}

# --------------------------------------------------------------------------------------------------

# This is a callback function which takes an oratab (valid) entry anc check for
# GRID_HOME.
f_check_for_grid_home() {
  local line=$1
  local grid_home_prefix=$2
  local grid_home_found=$3
  local grid_home_oratab_file=$4
  local grid_db_name=$(echo $line | cut -d ":" -f 1)
  log_message "grid_db_name is $grid_db_name  and grid_home_prefix is $grid_home_prefix"
  if [[ "$grid_db_name" == $grid_home_prefix* ]]; then
     eval $grid_home_found=1
     eval $grid_home_oratab_file=$line
  fi
  return 1
}

# --------------------------------------------------------------------------------------------------

# Following function checks that any entry in the "/etc/oratab" which starts
# with GRID_HOME prefix.
f_find_grid_home_in_oratab() {
  local oratab_file=$1
  local grid_home_prefix=$2
  local grid_home_found=0
  local grid_home_oratab_line=""

  f_parse_oratab_file $oratab_file f_check_for_grid_home "$grid_home_prefix" \
  grid_home_found grid_home_oratab_line

  if [ $grid_home_found -ne 0 ]; then
     echo "Found grid home in Oratab ${grid_home_oratab_line}"
     return 0
  fi

  return 1
}

# --------------------------------------------------------------------------------------------------

# Check that the current installation binary is RAC enabled or not.
# Returns NON-ZERO value if RAC supported in the binary level.
# Else returns ZERO.
f_check_for_RAC_capability() {
  local oracle_home=$1
  local ar_location=$(which ar)
  local ar_switch="-t"

  if [ $? -ne 0 ]; then
     return 1
  fi

  if [ ${OS} = "AIX" ]; then
     ar_switch="-X32_64 $ar_switch"
  fi

  local RAC_KERNEL_CSM_OPTION="kcsm"
  local ORACLE_KERNEL_OPTION_LIB=${oracle_home}/rdbms/lib/libknlopt.a
  local binary_is_rac_enabled=$($ar_location ${ar_switch} ${ORACLE_KERNEL_OPTION_LIB} | grep -c $RAC_KERNEL_CSM_OPTION)
  log_message "Output of binary RAC capability $($ar_location ${ar_switch} ${ORACLE_KERNEL_OPTION_LIB} | grep -c $RAC_KERNEL_CSM_OPTION)"
  return $binary_is_rac_enabled
}

# --------------------------------------------------------------------------------------------------

# Verify that the RAC is running on the current system.
# 1. If binaries are RAC enabled &&
# 2. CSRD (Cluster Ready Service deamon is running).
f_check_RAC_running() {
  local oracle_home=$1
  f_check_for_RAC_capability ${oracle_home}

  if [ $? -eq 0 ]; then
   return 1
  fi

  # Grep for cluster service deamon is running or not. Following returns a
  # non-zero result if CSRD is running.
  local is_crsd_running=$(ps -ef | grep -v grep | grep -c "crsd")

  if [ $is_crsd_running -eq 0 ];then
     log_message "RAC capable system but CRSD is not running"
     return 1
  fi

  return 0
}

# --------------------------------------------------------------------------------------------------

# Check for RAC compatability.
# This is verification function gets called, if a potential RAC host
# discovered as a StandAlone Oracle Host. Following checks are performed as
# part of this function.
#
# Check RAC is running on the current host or not.
# If RAC is running check that the "/etc/oratab" file has the GRID Home entry
# or not. If "/etc/oratab" does not have a GRID home entry flag and Error.
f_validate_RAC() {
  local oracle_home=$1
  local oratab_file=$2
  local grid_home_prefix=$3

  f_check_RAC_running $oracle_home
  local is_rac_running=$?

  if [ $is_rac_running -ne 0 ]; then
     log_message "RAC is not running"
     return 0
  fi

  f_find_grid_home_in_oratab $oratab_file "$grid_home_prefix"
  local grid_home_in_oratab=$?

  if [ $grid_home_in_oratab -ne 0 ];then
    log_message "Current Host is RAC enabled, but couldn't find
        ${grid_home_prefix} in ORATAB_FILE=${oratab_file}"
    return 1
  fi
  return 0
}

# --------------------------------------------------------------------------------------------------
