#!/bin/bash
# Copyright 2019 Cohesity Inc.
#
# Author:Soumya Poddar
#
# This script performs the pre validation checks before starting Oracle Database
# Alternate restore or clone.
#
# It takes the Source DB version, and other configuration paramertes and checks
# compatability and permission in the config directories.
#
# /home/oracle/cohesityagent/software/crux/bin/oracle_scripts/precheck_validation.sh
# -H /u01/app/oracle/product/12.2.0.1/db_1 -B /u01/app/oracle -F /tmp -f
# /home/oracle/Oracle_DATA/fdest -d /home/oracle/Oracle_DATA/dpath -b /home/oracle/Oracle_DATA/bpath -c
# /home/oracle/Oracle_DATA/cpath -V 12.2.0.1.0

# -------------------------------------------------------------------------------------------------

LOG_FILE=""
script_out=0
failure_reasons=""

#
# Example RMAN Error Stack.
#RMAN-00571: ===========================================================
#RMAN-00569: =============== ERROR MESSAGE STACK FOLLOWS ===============
#RMAN-00571: ===========================================================
#
RMAN_ERR_MSG_CODE="RMAN-00569"

# Which OS Cohesity agent is running on.
OS=$(uname -a | cut -d " " -f 1)

PRECHECK_START_MESSAGE="Starting Precheck validation for"
PRECHECK_PASS_MESSAGE="PreCheck Validation passed for"
PRECHECK_FAIL_MESSAGE="PreCheck Validation script failed for"
PRECHECK_END_MESSAGE="Completed Precheck validation for"
ORACLE_LOGIN_ERR_STRING="ORA-01017:"

if [ ${OS} = "AIX" ]; then
  # TODO(aabhas): find replacement for readlink on AIX.
  current_dir=$(dirname $0)
else
  current_dir=$(dirname `readlink -f $0`)
fi

SCRIPT_NAME=`basename "$0"`
# Source needed library functions.
. $current_dir/precheck_libs.sh
. $current_dir/common_utils.sh

f_create_log_file()
{
  if [[ "x$LOG_FILE_PATH" == "x" ]]; then
     LOG_FILE_PATH="/tmp/precheck_logs.INFO"
  fi
  LOG_FILE=${LOG_FILE_PATH}

  # This is a precheck log file, setting generic permission it.
  if [ ! -f ${LOG_FILE} ]; then
    touch ${LOG_FILE}
    chmod 777 ${LOG_FILE}
  fi
}

log_message()
{
  msg=$*
  TS=`date "+I%m%d %H:%M:%S"`
  echo "$TS ${msg}" >> ${LOG_FILE}
}

# Mark the script failed.
# Add the failure reason in the Failure reason string.
f_mark_script_failed_with_reason()
{
  local message=$*
  failure_reasons="$failure_reasons  {$message}"
  script_out=1
}

f_export_env() {
  export ORACLE_HOME=${ORACLE_HOME}
  export ORACLE_SID=${ORACLE_DB_SID}
  export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:${ORACLE_HOME}/lib"
  export PATH="$PATH:${ORACLE_HOME}/bin"
  export NLS_DATE_FORMAT='DD:MM:YYYY-HH24:MI:SS'
}

# -------------------------------------------------------------------------------------------------

# Following function populate the global shell environment variables.
# These are Oracle specific environemnt variable like ORACLE_HOME, ORACLE_BASE
# oracle_user, oracle_group etc.
#
f_populate_oracle_env() {

  # If oracle binary is absent on the ORACLE_HOME. Exit with error.
  if [ ! -f ${ORACLE_HOME}/bin/oracle ]; then
    log_message "Unable to find ${ORACLE_HOME}/bin/oracle"
    # In case of ENVCHECK, do not exit with error.
    if [ ${WORKFLOW_TYPE} == "ENVCHECK" ]; then
      return
    fi
    exit 1;
  fi

  oracle_bin_permission=`ls -l ${ORACLE_HOME}/bin/oracle`

  # If failed to get the permission for oracle binary exit with error.
  if [ $? -ne 0 ]; then
    log_message "Unable to find permissions for: ${ORACLE_HOME}/bin/oracle"
    exit 1;
  fi

  ORACLE_USER=`echo ${oracle_bin_permission} | awk '{print $3}'`
  ORACLE_GROUP=`echo ${oracle_bin_permission} | awk '{print $4}'`

  # If not able ORACLE_USER or ORACLE_GROUP is not set exit with error.
  if [ -z ${ORACLE_USER} ] || [ -z ${ORACLE_GROUP} ]; then
    log_message "ORACLE_USER(${ORACLE_USER}) or ORACLE_GROUP(${ORACLE_GROUP}) not set."
    exit 1;
  fi

  # If both ORACLE_HOME and ORACLE_SID are provided do export them
  if [ ! -z ${ORACLE_HOME} ] && [ ! -z ${ORACLE_DB_SID} ]; then
    f_export_env
  fi
}

# -------------------------------------------------------------------------------------------------

# Following function does the validation of the various Database config files.
# It checks the existence and the permission of those file and directories
# needed as part of Backup/Restore/clone functionality.
#
# To add checks for any new DB config file, add the checks in the following
# function.

f_db_path_verification() {
  if [ ! -z ${BCT_FILE_PATH} ]; then
     if ! f_check_asm_path ${BCT_FILE_PATH}; then
       # Check for the bct filepath's parent folder as BCT file will be created
       # inside it.
       BCT_FILE_PATH=`dirname ${BCT_FILE_PATH}`;
     fi
     permission=$(eval printf "%o" $(($RDPERM | $WRPERM | $EXEPERM)))
     f_get_path_check_function ${BCT_FILE_PATH}
     ${f_check_file_path_and_perm}\
     ${BCT_FILE_PATH} ${ORACLE_USER} ${ORACLE_GROUP} ${permission} true "BCT_FILE_DIR"
  fi

  if [ ! -z ${DB_CONFIG_FILE_PATH} ]; then
     permission=$(eval printf "%o" $(($RDPERM | $WRPERM)))
     f_get_path_check_function ${DB_CONFIG_FILE_PATH}
     ${f_check_file_path_and_perm}\
     ${DB_CONFIG_FILE_PATH} ${ORACLE_USER} ${ORACLE_GROUP} ${permission} false "DB_CONFIG_FILE"
  fi

  if [ ! -z ${DIAG_DEST} ]; then
     permission=$(eval printf "%o" $(($RDPERM | $WRPERM | $EXEPERM)))
     f_get_path_check_function ${DIAG_DEST}
     ${f_check_file_path_and_perm}\
     ${DIAG_DEST} ${ORACLE_USER} ${ORACLE_GROUP} ${permission} true "DIAG_DEST"
  fi

  if [ ! -z ${FRA_DEST} ]; then
     permission=$(eval printf "%o" $(($RDPERM | $WRPERM | $EXEPERM)))
     f_get_path_check_function ${FRA_DEST}
     ${f_check_file_path_and_perm}\
     ${FRA_DEST} ${ORACLE_USER} ${ORACLE_GROUP} ${permission} true "FRA_DEST"
  fi

  if [ ! -z ${AUDIT_LOG_DEST} ]; then
     permission=$(eval printf "%o" $(($RDPERM | $WRPERM | $EXEPERM)))
     f_get_path_check_function ${AUDIT_LOG_DEST}
     ${f_check_file_path_and_perm}\
     ${AUDIT_LOG_DEST} ${ORACLE_USER} ${ORACLE_GROUP} ${permission} true \
     "AUDIT_LOG_DEST"
  fi


  if [ ! -z ${ALT_RESTORE_DATAFILE_PATH} ]; then
     permission=$(eval printf "%o" $(($RDPERM | $WRPERM | $EXEPERM)))
     f_get_path_check_function ${ALT_RESTORE_DATAFILE_PATH}
     ${f_check_file_path_and_perm}\
     ${ALT_RESTORE_DATAFILE_PATH} ${ORACLE_USER} ${ORACLE_GROUP} ${permission} \
     true "ALTERNATE_RESTORE_LOCATION"
  fi
}

# -------------------------------------------------------------------------------------------------

# Following are the very basic checks, which confirms that the any Oracle
# workflow can run on the given ebvironment.
#
# 1. Check rman,sqlplus, oracle bin, oracle lib can be accessed with proper
#    permissions.
# 2. Check if ORACLE_BASE can be accessed by the current user/group with the
#    given permission.
f_basic_checks() {
  f_validate_agent_user_OS_Auth

  if [ $? -ne 0 ]; then
     log_message "Credential Check failed"
  fi

  # Check binaries oracle,rman,sqlplus,nid can be accessed.
  oracle_bin_file_list=""
  for key in ${CommonOracleBinaryArray[@]}
  do
    if [[ "x"${oracle_bin_file_list} == "x" ]]; then
      oracle_bin_file_list=${ORACLE_HOME}/bin/$key
    else
      oracle_bin_file_list=${oracle_bin_file_list},${ORACLE_HOME}/bin/$key
    fi
  done

  # Check ORACLE_HOME/bin has the read and list permission.
  permission=$(eval printf "%o" $(($RDPERM | $EXEPERM)))

  f_check_file_path_and_perm_UNIX_FS ${oracle_bin_file_list} \
  ${ORACLE_USER} ${ORACLE_GROUP} ${permission} false # Not dir

  # Check ORACLE_HOME/lib has the read and list permission.
  permission=$(eval printf "%o" $(($RDPERM | $WRPERM | $EXEPERM)))

  f_check_file_path_and_perm_UNIX_FS ${ORACLE_HOME}/lib \
  ${ORACLE_USER} ${ORACLE_GROUP} ${permission} true # Dir

  # Pending Checks for the libsql path.

  # We need not check for oracle base when workflow is backup.
  if [ ${WORKFLOW_TYPE} == "BACKUP" ]; then
    return 0;
  fi

  permission=$(eval printf "%o" $(($RDPERM | $WRPERM | $EXEPERM)))
  f_check_file_path_and_perm_UNIX_FS ${ORACLE_BASE} ${ORACLE_USER} ${ORACLE_GROUP} ${permission} true

  if [ $? -ne 0 ]; then
     log_message "Do not have access permission for Oracle BASE=${ORACLE_BASE}"
     f_mark_script_failed_with_reason "Access denied for ${ORACLE_BASE}"
  fi
}

# -------------------------------------------------------------------------------------------------

# Clone or Restore to Alternate specific checks.
f_verify_ALTERNATE_RESTORE()
{
  # First initialize Oracle environment specific values before calling workload
  # specific valisation.
  f_populate_oracle_env
  if [ -z "$SRC_DB_VERSION" ]; then
     log_message "Oracle Source DB Version parameter is missing."
     usage $LINENO;
     exit 1;
  fi

  if [ -z ${ALT_RESTORE_DATAFILE_PATH} ] || [ $DB_DATAFILE_SIZE -eq 0 ]; then
    log_message "Alternate Restore path or Total DB size not specified"
    usage $LINENO;
    exit 1;
  fi

  if [ -z ${ORATAB_FILE} ] || [ ! -f ${ORATAB_FILE} ] ; then
    log_message "ORATAB filename not passed or ${ORATAB_FILE} is invalid"
    usage $LINENO;
    exit 1;
  fi

  f_basic_checks
  f_oracle_version_check
  f_db_path_verification

  f_get_space_check_func ${ALT_RESTORE_DATAFILE_PATH}
  ${f_check_available_space} ${ALT_RESTORE_DATAFILE_PATH} ${DB_DATAFILE_SIZE}
  local space_check_res=$?
  if [ $space_check_res -ne 0 ] && [ $space_check_res -ne ${ENOENT} ]; then
     f_mark_script_failed_with_reason "Not enough space in ${ALT_RESTORE_DATAFILE_PATH}\
          need ${DB_DATAFILE_SIZE}GB space for restore"
  fi

  if [[ "x${SGA_TARGET}" != "x" ]] && [[ "${SGA_TARGET}" != "0" ]];then
    f_check_required_mem ${SGA_TARGET}
  fi

  ## Add same DB SID checks as well.
  f_check_same_DBSID ${ORACLE_DB_SID} ${ORATAB_FILE}
}

# -------------------------------------------------------------------------------------------------

# Clone or Restore to Alternate specific checks.
f_verify_CLONE()
{
  # First initialize Oracle environment specific values before calling workload
  # specific valisation.
  f_populate_oracle_env
  if [ -z "$SRC_DB_VERSION" ]; then
     log_message "Oracle Source DB Version parameter is missing."
     usage $LINENO;
     exit 1;
  fi

  if [ -z ${ORATAB_FILE} ] || [ ! -f ${ORATAB_FILE} ] ; then
    log_message "ORATAB filename not passed or ${ORATAB_FILE} is invalid"
    usage $LINENO;
    exit 1;
  fi

  f_basic_checks
  f_oracle_version_check
  f_db_path_verification

  if [[ "x${SGA_TARGET}" != "x" ]] && [[ "${SGA_TARGET}" != "0" ]];then
    f_check_required_mem ${SGA_TARGET}
  fi

  ## Add same DB SID checks as well.
  f_check_same_DBSID ${ORACLE_DB_SID} ${ORATAB_FILE}
}

# -------------------------------------------------------------------------------------------------

# Following function validates that the current Effective user is able to run
# Oracle wrokflow with OS Authentication.
f_validate_agent_user_OS_Auth() {
  local oracle_bin_file=${ORACLE_HOME}/bin/oracle
  local cur_uname=$(id -un) # Initialize as current username.
  local cur_euid=$EUID # Effecting User ID

  f_get_all_dba_groups_from_ORACLE_HOME ${ORATAB_FILE}

  if [ $? -ne 0 ]; then
     log_message "Failed to get DBA group information, skipping checks"
     return 0
  fi

  # If we succeed get the username from the current effective user id, then we
  # would user the username from the effective user id.
  id -un $cur_euid > /dev/null 2>&1
  if [ $? -eq 0 ];then
    cur_uname=$(id -un $cur_euid)
  fi

  f_is_same_user $oracle_bin_file ${cur_uname}
  if [ $? -eq 0 ]; then
    return
  fi

  f_cur_user_has_SYSDBA_PRIV $cur_uname
  local has_sysdba_priv=$?

  if [ ${has_sysdba_priv} -ne 0 ]; then
    # TODO(Chenyao): remove this check once we support clone on sysbackup priv
    if [ ${WORKFLOW_TYPE} == "CLONE" ]; then
      log_message "Clone without SYSDBA privilege is not supported"
      return has_sysdba_priv
    fi
    f_cur_user_has_SYSBACKUP_PRIV $cur_uname
    return $?
  fi
  return has_sysdba_priv
}

# -------------------------------------------------------------------------------------------------

# Perform the environment releated prechecks.
f_verify_ENVCHECK() {
  if [ -z ${ORACLE_HOME} ] || [ ! -d ${ORACLE_HOME} ]; then
     log_message "Failed to find ORACLE_HOME or ${ORACLE_HOME} is not a direcotry"
  fi

  f_populate_oracle_env
  # If GROUP_CHECK is not enabled, skip the check.
  if [ ! -z ${GROUP_CHECK} ] && [ ${GROUP_CHECK} -ne 0 ];then
     # First initialize Oracle environment specific values before calling workload
     # specific valisation.
     f_validate_agent_user_OS_Auth
  fi

  if [ -z ${RAC_CHECK} ] || [ ${RAC_CHECK} -eq 0 ];then
    log_message "RAC check option not passed, skipping RAC check"
    return
  fi

  if [ -z ${ORATAB_FILE} ] || [ ! -f ${ORATAB_FILE} ]; then
    log_message "ORATAB_FILE not passed correctly not able to run RAC \
        validation."
    exit 1
  fi

  if [ -z $ORACLE_GRID_HOME_PREFIX ];then
    log_message "GRID home prefix not sent in argument, not running RAC validation"
    exit 1
  fi

  f_validate_RAC ${ORACLE_HOME} ${ORATAB_FILE} ${ORACLE_GRID_HOME_PREFIX}

  if [ $? -ne 0 ]; then
    f_mark_script_failed_with_reason "RAC discovery Failed.\
        Current host is a RAC instance but \
        Entry with ${ORACLE_GRID_HOME_PREFIX} not found in ${ORATAB_FILE}"
  fi
}

# -------------------------------------------------------------------------------------------------

# Perform DB discovery related prechecks.
f_verify_DISCOVERY() {
  # For now only print out the DBs which are not running.
  if [ -z ${ORATAB_FILE} ]; then
     log_message "ORATAB_FILE is not provided for discovery precheck"
     usage $LINENO;
     exit 1;
  fi

  if [ ! -f ${ORATAB_FILE} ]; then
     log_message "ORATAB_FILE ${ORATAB_FILE} is not present "
     usage $LINENO;
     exit 1;
  fi
  f_check_for_all_dbs_running $DB_RUNNING_STRING
}

# -------------------------------------------------------------------------------------------------

f_verify_BACKUP() {
  # First initialize Oracle environment specific values before calling workload
  # specific valisation.
  f_populate_oracle_env
  f_basic_checks

  # connect string is passsed as stdin.
  read connect_string;

  f_oracle_set_privilege "$connect_string"

  # using `` instead of $() because of bash 4.1 version compatibility.
  local output=`$SQLPLUS_PATH -S $connect_string $sqlplus_privilege << EOF 2>&1
set verify off;
set heading off;
select open_mode,database_role from v\\$database;
EOF`

  local login_failed=$(echo "$output" | grep -w $ORACLE_LOGIN_ERR_STRING)

  if [[ "x${login_failed}" != "x" ]]; then
    f_mark_script_failed_with_reason "${login_failed}"
    return 1;
  fi

  # The query output can have following formats for example:
  # MOUNTED              PHYSICAL STANDBY
  # MOUNTED              LOGICAL STANDBY
  # READ ONLY WITH APPLY PHYSICAL STANDBY
  # READ ONLY            PHYSICAL STANDBY
  # READ WRITE           PRIMARY
  # READ ONLY            PRIMARY(Not allowed)
  # MOUNTED              PRIMARY(Not allowed)
  upcased_output=`echo "$output" | awk '{print toupper($0)}'`
  echo ${upcased_output} | grep "PRIMARY" > /dev/null
  if [ $? -ne 0 ]; then
    # Not primary, should be physical/logical standby
    echo ${upcased_output} | egrep "LOGICAL STANDBY|PHYSICAL STANDBY" > /dev/null
    if [ $? -ne 0 ]; then
      # Not primary and not physical/logical standby, fail
      # TODO(Vivek) May be add a check for OPEN mode in standby as well.
      local msg="Database ${DB_UNIQUE_NAME} should be in Primary or Physical/Logical Standby Role."
      log_message "${msg}"
      f_mark_script_failed_with_reason "${msg}"
      return 1
    fi
  else
    echo ${upcased_output} | grep "READ WRITE" > /dev/null
    if [ $? -ne 0 ]; then
      # Its a primary not opened in READ WRITE mode, fail
      local msg="Database ${DB_UNIQUE_NAME} in Primary role should be opened in READ WRITE mode."
      log_message "${msg}"
      f_mark_script_failed_with_reason "${msg}"
      return 1;
    fi
  fi

  if [ -z "$REMOTE_NODE_INFO_LIST" ]; then
    log_message "Remote node info is not passed Skipping MNMC check."
    return
  fi

  if [[ "X${RMAN_PATH}" == "X" ]]; then
    log_message "-R RMAN_PATH is not supplied cannot verify Easy-connect check"
    return
  fi

  # MNMC remote channel connection check
  IFS=',' read -r -a node_info_array <<< "$REMOTE_NODE_INFO_LIST"

  # Deserialize comma separated remote node info, into respective vectors:
  # - fqdn vector
  # - port number vector
  # - remote mount path vector
  # - remote Oracle Sid vector
  fqdn_vec=""
  port_num_vec=""
  remote_mount_pts=""
  remote_channel_oracle_sid_vec=""

  for index in "${!node_info_array[@]}"
  do
    echo "node $index is ${node_info_array[$index]}";
    IFS=':' read -r -a temp_info <<< "${node_info_array[$index]}"
    # Older Magneto may not have sent the 4th element - Oracle SID
    if [ ${#temp_info[@]} -lt 3 ]; then
      local msg="RAC Database ${DB_UNIQUE_NAME} has incorrect remote mount info vector: ${temp_info[@]}"
      log_message "${msg}"
      f_mark_script_failed_with_reason "${msg}"
      return 1;
    fi

    fqdn_vec=(${fqdn_vec[@]} ${temp_info[0]});
    port_num_vec=(${port_num_vec[@]} ${temp_info[1]});
    remote_mount_pts=(${remote_mount_pts[@]} ${temp_info[2]});
    # If older Magneto did not send the Oracle SID, the following Bash variable
    # will accummulate empty string. The command will not fail.
    # 'remote_channel_oracle_sid_vec' will just be an empty string.
    remote_channel_oracle_sid_vec=(${remote_channel_oracle_sid_vec[@]} ${temp_info[3]});
  done

  #
  # Compose RMAN remote channel naming method, then test connect them one
  # by one.
  #
  for index in "${!node_info_array[@]}"
  do
    # Compose MNMC remote channel naming method, which can be one of the following:
    # (a) Oracle Connect Descriptor:
    #      (DESCRIPTION=(ADDRESS=(PROTOCOL=tcp)(HOST=10.2.103.25)(PORT=1521))(CONNECT_DATA=(SID=cdb1)))
    # (b) Oracle EZConnect form:
    #      10.2.150.25:1521/cdb1.cohesity.com
    if [ -z ${MNMC_USE_CONNECT_DSC} ]; then
       ORACLE_NAMING_METHOD="${connect_string}@${fqdn_vec[$index]}:${port_num_vec[$index]}/$DB_UNIQUE_NAME"
    else
       remote_channel_oracle_sid=${remote_channel_oracle_sid_vec[$index]}
       log_message "remote channel ${index} oracle sid=$remote_channel_oracle_sid"
       if [ -z ${MNMC_USE_SID} ] || [ -z ${remote_channel_oracle_sid} ]; then
         # Use Oracle Service Name to connect if Agent gflag linux_agent_oracle_mnmc_use_sid says so.
         # Fall back to use service name if gflag asks for SID but we do not have SID provided (old Magneto does not send SID)
         CONNECT_DATA_OPTION="(SERVICE_NAME=$DB_UNIQUE_NAME)"
       else
         # Use Oracle SID to connect.
         # Oracle SID ensures RAC listener will not be able to re-route the connection to a different RAC node.
         CONNECT_DATA_OPTION="(SID=${remote_channel_oracle_sid})"
       fi
       ORACLE_NAMING_METHOD="${connect_string}@(DESCRIPTION=(ADDRESS=(PROTOCOL=tcp)(HOST=${fqdn_vec[$index]})(PORT=${port_num_vec[$index]}))(CONNECT_DATA=$CONNECT_DATA_OPTION))"
    fi
    # Log the naming method used to set up remote RMAN channel (masking the
    # sensitive connect string)
    masked_oracle_naming_method="Checking connect to remote channel $((num_local_mounts+index+1)) using: `sed -n 's/.*\(@.*\)/<connect-string>\1/p'<<<${ORACLE_NAMING_METHOD}`"
    log_message ${masked_oracle_naming_method}

    # Indicates that remote login with SYSDBA priv failed.
    # 1: means there is login failure.
    # 0: means pass.
    local rman_login_err_sysdba=1

    # Indicates that remote login with SYSBACKUP priv failed.
    # 1: means there is login failure.
    # 0: means pass.
    local rman_login_err_sysbackup=1

    # First try remote login with SYSDBA priv.
    local r_conn_out=`$RMAN_PATH << EOF 2>&1
connect target "${ORACLE_NAMING_METHOD}";
quit;
EOF`

    # grep RMAN Generic error code "RMAN-00569", which indicates RMAN failure.
    local rman_err_sysdba=$(echo "${r_conn_out}" | grep "${RMAN_ERR_MSG_CODE}")
    if [[ "X${rman_err_sysdba}" == "X" ]]; then
       # If there is no failure in remote login using SYSDBA priv, set the appropriate
       # then set rman_login_err_sysdba to be 0.
       rman_login_err_sysdba=0
    fi

    # Example Oracle error String.
    # ORA-01017: invalid username/password; logon denied'
    local ora_err_sysdba=$(echo "${r_conn_out}" | grep -w "ORA-[0-9]*:")

    # If rman_privilege ( mostly SYSBACKUP ) is set, Try to login using that.
    if [[ "X${rman_privilege}" != "X" ]]; then
       local r_conn_out_sysbackup=`$RMAN_PATH << EOF 2>&1
connect target '${ORACLE_NAMING_METHOD}${rman_privilege}';
quit;
EOF`
       # grep RMAN Generic error code "RMAN-00569", which indicates RMAN failure.
       local rman_err_sysbkp=$(echo "${r_conn_out_sysbackup}" | grep "${RMAN_ERR_MSG_CODE}")
       if [[ "X${rman_err_sysbkp}" == "X" ]]; then
          rman_login_err_sysbackup=0
       fi

       # Example Oracle error String.
       # ORA-01017: invalid username/password; logon denied'
       local ora_err_sysbkp=$(echo "${r_conn_out_sysbackup}" | grep -w "ORA-[0-9]*:")
    fi

    local msg="Remote connect to database:"$DB_UNIQUE_NAME" failed. Reason:";

    if [[ "${ora_err_sysdba}" != "${ora_err_sysbkp}" ]]; then
       msg="${msg} [SYSDBA]${ora_err_sysdba} [SYSBACKUP]${ora_err_sysbkp}"
    else
       msg="${msg} ${ora_err_sysdba}"
    fi

    if [ ${rman_login_err_sysdba} -ne 0 ] && [ ${rman_login_err_sysbackup} -ne 0 ]; then
      log_message "$msg"
      f_mark_script_failed_with_reason "${msg}"
    fi
  done
}

# -------------------------------------------------------------------------------------------------

f_verify_OVERWRITE_RESTORE() {
  log_message "------- Not Implemented"
}

# -------------------------------------------------------------------------------------------------

f_arg_env_check() {
  if [ -z "$ORACLE_HOME" ]; then
     log_message "Oracle Home parameter missing."
     usage $LINENO;
     exit 1;
  fi

  if [ -z ${ORACLE_DB_SID} ]; then
    log_message "ORACLE DB SID is missing "
    usage $LINENO;
    exit 1;
  fi

  # If the given ORACLE_HOME is not present, no Oracle workload
  # can run. Break early.
  if [ ! -d ${ORACLE_HOME} ]; then
     local msg="Oracle home ${ORACLE_HOME} doesn't exists or not directory"
     log_message "${msg}"

     f_mark_script_failed_with_reason "${msg}"
     return 1;
  fi

  # We need not check for oracle base when workflow is backup.
  if [ ${WORKFLOW_TYPE} == "BACKUP" ]; then
    return 0;
  fi

  if [ -z "$ORACLE_BASE" ]; then
     log_message "Oracle Base parameter missing."
     usage $LINENO;
     exit 1;
  fi

  # If the given ORACLE_BASE is not present, no Oracle workload
  # can run. Break early.
  if [ ! -d ${ORACLE_BASE} ]; then
     local msg="Oracle Base ${ORACLE_BASE} doesn't exists or not directory"
     # log_message "Oracle Base ${ORACLE_BASE} doesn't exists or not directory"
     log_message "${msg}"

     f_mark_script_failed_with_reason "${msg}"
     return 1;
  fi
}

# -------------------------------------------------------------------------------------------------

# Check the basic sanity of the command line arguments.
f_arg_check() {
  # Type of operation should be mentioned in the command line argument.
  if [ -z ${WORKFLOW_TYPE} ]; then
     log_message "WORKFLOW_TYPE .. means precheck running for which operation is missing"
     usage $LINENO;
     exit 1
  fi

  if [ ${WORKFLOW_TYPE} != "BACKUP" ] &&
     [ ${WORKFLOW_TYPE} != "ALTERNATE_RESTORE" ] &&
     [ ${WORKFLOW_TYPE} != "CLONE" ] &&
     [ ${WORKFLOW_TYPE} != "OVERWRITE_RESTORE" ] &&
     [ ${WORKFLOW_TYPE} != "DISCOVERY" ] &&
     [ ${WORKFLOW_TYPE} != "ENVCHECK" ]; then
    log_message "WORKFLOW_TYPE ${WORKFLOW_TYPE} not matched ..."
    usage $LINENO;
    exit 1
  fi

  PRECHECK_START_MESSAGE="${PRECHECK_START_MESSAGE} ${WORKFLOW_TYPE}"
  PRECHECK_PASS_MESSAGE="${PRECHECK_PASS_MESSAGE} ${WORKFLOW_TYPE}"
  PRECHECK_FAIL_MESSAGE="${PRECHECK_FAIL_MESSAGE} ${WORKFLOW_TYPE}"
  PRECHECK_END_MESSAGE="${PRECHECK_END_MESSAGE} ${WORKFLOW_TYPE}"
  if [ ! -z ${TASK_ID} ];then
    PRECHECK_START_MESSAGE="${PRECHECK_START_MESSAGE} with task_id=${TASK_ID}"
    PRECHECK_PASS_MESSAGE="${PRECHECK_PASS_MESSAGE} with task_id=${TASK_ID}"
    PRECHECK_FAIL_MESSAGE="${PRECHECK_FAIL_MESSAGE} with task_id=${TASK_ID}"
    PRECHECK_END_MESSAGE="${PRECHECK_END_MESSAGE} with task_id=${TASK_ID}"
  fi

  local argument_check_result=0
  if [ ${WORKFLOW_TYPE} != "DISCOVERY" ] &&
     [ ${WORKFLOW_TYPE} != "ENVCHECK" ]; then
     f_arg_env_check
     argument_check_result=$?
  fi

  return ${argument_check_result}
}

# -------------------------------------------------------------------------------------------------

f_do_precheck() {
  echo "----------------------------------------------------------" >> ${LOG_FILE}
  echo "" >> ${LOG_FILE} # newline

  log_message "Starting precheck with ${AGENT_VERSION_STR}"
  log_message "${PRECHECK_START_MESSAGE}"

  # First initialize Oracle environment specific values before calling workload
  # specific valisation.
  f_verify_${WORKFLOW_TYPE}
}

# -------------------------------------------------------------------------------------------------

usage()
{
  local line_no=$1
  echo "script failure at $line_no"
  echo "Invoke the script with following arguments :-"
  echo -e "\t -A|--Alternate-location Path to Alternate location where DATAFILES are stored."
  echo -e "\t -B|--oracle-base        Path to Oracle BASE directory."
  echo -e "\t -D|--Data-FIle-size     Total size of the Database in GB"
  echo -e "\t -F|--log-file-path      Rman log file path."
  echo -e "\t -G|--Grid-Home-Prefix   Oracle GRID Home prefix."
  echo -e "\t -H|--oracle-home        Path to ORACLE home directory."
  echo -e "\t -I|--task-id            optional task-id"
  echo -e "\t -M|--Memory-Needed      SGA_TARGET size in case of ALT Restore"
  echo -e "\t -N|--remote-node-info   CSV Hostname followed by port number followed"
  echo -e "\t -O|--oratab-file        File location of /etc/oratab file"
  echo -e "\t -P|--sqlplus-path       Path to SQLPLUS binary."
  echo -e "\t -R|--rman-path          Path to RMAN binary."
  echo -e "\t -S|--oracle-sid         ORACLE DB SID"
  echo -e "\t -U|--db-unique-name     Unique name for database.It can be appeneded by db_domain in some cases."
  echo -e "\t -f|--fra-dest-path      Fast recovery Dest path."
  echo -e "\t -d|--diag-dest          Path to diag dest dir."
  echo -e "\t -b|--bct-file-path      Path to Block Change Tracking directory."
  echo -e "\t -c|--db-config-file     Path to db-config-file."
  echo -e "\t -V|--SRC-ORCL-Version   Oracle Version where DB instantiated"
  echo -e "\t -T|--Workflow-Type      The type of workflow [DISCOVERY/BACKUP/ALTERNATE_RESTORE
  /CLONE/OVERWRITE_RESTORE]"
  echo -e "\t -y|--mnmc-use-connect-dsc MNMC to use Oracle Connect Descriptor, instead of EZconnect."
  echo -e "\t -z|--mnmc_use_sid         MNMC to use SID instead of service name."
}

# -------------------------------------------------------------------------------------------------

## MAIN

while getopts "A:B:D:F:G:H:I:M:N:O:P:R:S:T:U:V:a:b:c:d:e:f:g:r:s:yz" option; do
  case $option in
    A)
      ALT_RESTORE_DATAFILE_PATH=$OPTARG;;
    B)
      ORACLE_BASE=$OPTARG;;
    D)
      DB_DATAFILE_SIZE=$OPTARG;;
    F)
      LOG_FILE_PATH=$OPTARG;;
    G)
      ORACLE_GRID_HOME_PREFIX=$OPTARG;;
    H)
      ORACLE_HOME=$OPTARG;;
    I)
      TASK_ID=$OPTARG;;
    M)
      SGA_TARGET=$OPTARG;;
    N)
      REMOTE_NODE_INFO_LIST=$OPTARG;;
    O)
      ORATAB_FILE=$OPTARG;;
    P)
      SQLPLUS_PATH=$OPTARG;;
    R)
      RMAN_PATH=$OPTARG;;
    S)
      ORACLE_DB_SID=$OPTARG;;
    T)
      WORKFLOW_TYPE=$OPTARG;;
    U)
      DB_UNIQUE_NAME=$OPTARG;;
    V)
      SRC_DB_VERSION=$OPTARG;;
    a)
      DB_RUNNING_STRING=$OPTARG;;
    b)
      BCT_FILE_PATH=$OPTARG;;
    c)
      DB_CONFIG_FILE_PATH=$OPTARG;;
    d)
      DIAG_DEST=$OPTARG;;
    e)
      AUDIT_LOG_DEST=$OPTARG;;
    f)
      FRA_DEST=$OPTARG;;
    g)
      GROUP_CHECK=$OPTARG;;
    r)
      RAC_CHECK=$OPTARG;;
    s)
      AGENT_VERSION_STR=$OPTARG;;
    y)
      MNMC_USE_CONNECT_DSC=1;;
    z)
      MNMC_USE_SID=1;;
    *) echo "Invalid Option passed"; usage $LINENO; exit 1 ;;
  esac
done

# Modify how to invoke Oracle binaries if the agent is installed with 'root'
setup_invoke_oracle_binaries

# -------------------------------------------------------------------------------------------------

f_create_log_file
if [ $? -ne 0 ]; then
  msg="Failed to create the log file: $LOG_FILE"
  log_message "$msg"
  f_mark_script_failed_with_reason "$msg"
fi

f_arg_check

if [ $? -eq 0 ]; then
   f_do_precheck
else
   log_message "Argument check failed, failing the precheck"
fi

if [ ${script_out} -ne 0 ]; then
   log_message "${PRECHECK_FAIL_MESSAGE} due to ${failure_reasons}"
else
   log_message "${PRECHECK_PASS_MESSAGE}"
fi

log_message "${PRECHECK_END_MESSAGE}"
echo "" >> ${LOG_FILE} # newline
echo "----------------------------------------------------------" >> ${LOG_FILE}

## Exit with result code.
echo $failure_reasons
exit $script_out
