#!/bin/bash
# Copyright 2019 Cohesity Inc.
#
# Author: Sandeep Tandekar
#
# This script performs the restore of database control file.
# It takes command ID, control file path as input and database id.
# Example :
# /home/oracle/cohesityagent/software/crux/bin/oracle_scripts//restore_control_file.sh
# -R /u01/app/oracle/product/11.2.0.3/db_1/bin/rman -H /u01/app/oracle/product/11.2.0.3/db_1
# -S v1 -C / -I cohesity_5704 -F /tmp -B 3312305298 -T True
# -A /home/oracle/cohesityagent/data/agent/tmp/nfs_oracle_mounts/oracle_5704_5_path0/5700_33toh3m8_1_1

# Read the stdin as first step.
read connect_string;

# Exit from the script if any of the step fails.
set -e

LOG_FILE=""

# Which OS Cohesity agent is running on.
OS=$(uname -a | cut -d " " -f 1)

if [ ${OS} = "AIX" ]; then
  # TODO(aabhas): find replacement for readlink on AIX.
  current_dir=$(dirname $0)
else
  current_dir=$(dirname `readlink -f $0`)
fi

SCRIPT_NAME=`basename "$0"`
# Source needed library functions.
. $current_dir/common_utils.sh


usage()
{
  echo "Invoke the script with following arguments :-"
  echo -e "\t -R|--rman-path        Path to RMAN binary."
  echo -e "\t -H|--oracle-home      Path to ORACLE home directory."
  echo -e "\t -S|--oracle-sid       Oracle database SID value."
  echo -e "\t -T|--stop-db          If set then stop the dababase."
  echo -e "\t -M|--start-mount-db   If set then start the database in mount mode after control file restore."
  echo -e "\t -I|--id               Operation ID."
  echo -e "\t -F|--log-file-path    Rman log file path."
  echo -e "\t -A|--control-file     Database control file path."
  echo -e "\t -B|--db-id            Database Id."
  echo -e "\t -s|--is-standby       It is set if the DB is working in standby role."
}

##### Main


while getopts "R:H:S:T:M:I:F:A:B:s" option; do
  case $option in
    R)
      RMAN_PATH=$OPTARG;;
    H)
      export ORACLE_HOME=$OPTARG;;
    S)
      export ORACLE_SID=$OPTARG;;
    T)
      STOP_DB=$OPTARG;;
    M)
      START_MOUNT_DB=$OPTARG;;
    I)
      ID=$OPTARG;;
    F)
      LOG_FILE_PATH=$OPTARG;;
    A)
      CONTROL_FILE=$OPTARG;;
    B)
      DATABASE_ID=$OPTARG;;
    s)
      IS_STANDBY=1;;
    *) echo "Invalid Option passed"; usage; exit 1 ;;
  esac
done

if [ -z "${RMAN_PATH}" ] || [ -z "${ORACLE_HOME}" ] || [ -z "${ORACLE_SID}" ] ||  [ -z "${connect_string}" ]; then
  echo "RMAN configuration parameters or authentication is missing."
  echo "*** ${RMAN_PATH}, ${ORACLE_HOME}, ${ORACLE_SID}";
  usage;
  exit 1;
fi

if [ -z "${CONTROL_FILE}" ] || [ -z "${ID}" ] || [ -z "${DATABASE_ID}" ]; then
  echo "Database parameters missing."
  echo "*** ${CONTROL_FILE}, ${ID}, ${DATABASE_ID}";
  usage;
  exit 1;
fi

f_set_privilege "$ORACLE_HOME" "$connect_string"

# Export PATH and other enviroment params.
export NLS_DATE_FORMAT='DD:MM:YYYY-HH24:MI:SS'
export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:${DEST_ORACLE_HOME}/lib"
export PATH="$PATH:${DEST_ORACLE_HOME}/bin"

if [[ "x$LOG_FILE_PATH" == "x" ]]; then
  LOG_FILE_PATH="/tmp"
fi
LOG_FILE=${LOG_FILE_PATH}/rman_shell_${ID}.INFO

# Modify how to invoke Oracle binaries if the agent is installed with 'root'
setup_invoke_oracle_binaries

echo "-------------------------" >> ${LOG_FILE}
log_message "Date: " `date`
log_message "Wrapper script args: $@"
log_message "Shell environment:"
log_message "`env`"

# After this point we want to handle the error.
set +e


log_message "Shutting down the database before control file restore."
if [ ! -z "${STOP_DB}" ]; then
  ${RMAN_PATH} >>${LOG_FILE} <<EOF
  connect target '${connect_string}${rman_privilege}';
  set command id to '${ID}';
  SHUTDOWN ABORT;
EOF

  log_message "Sleeping for 10 seconds."
  sleep 10;
fi

if [ ! -z ${IS_STANDBY} ]; then
  RESTORE_CLAUSE="RESTORE STANDBY CONTROLFILE FROM '${CONTROL_FILE}';"
  log_message "Restoring the standby control file ${CONTROL_FILE} for database sid ${ORACLE_SID}."
else
  RESTORE_CLAUSE="RESTORE PRIMARY CONTROLFILE FROM '${CONTROL_FILE}';"
  log_message "Restoring the primary control file ${CONTROL_FILE} for database sid ${ORACLE_SID}."
fi

# In case of Standalone and Active-Passive first start the database in no mount
# mode, restore the control file and then change database open_mode to mount.
if [ ! -z "${START_MOUNT_DB}" ]; then
  ${RMAN_PATH} >>${LOG_FILE} <<EOF
  connect target '${connect_string}${rman_privilege}';
  SET ECHO ON;
  SET DBID ${DATABASE_ID};
  STARTUP NOMOUNT;
  ${RESTORE_CLAUSE}
  STARTUP MOUNT;
EOF
# In case of RAC we will issue command like stop, startup mount, startup
# nomount using srvctl. In such case we will not do startup here.
else
  ${RMAN_PATH} >>${LOG_FILE} <<EOF
  connect target '${connect_string}${rman_privilege}';
  SET ECHO ON;
  SET DBID ${DATABASE_ID};
  ${RESTORE_CLAUSE}
EOF
fi

result=$?

if [ ${result} -eq 0 ];then
    log_message "***Operation completed on ${ORACLE_SID} at: $(date)"
else
    log_message "***Operation failed on ${ORACLE_SID} at: $(date)"
    log_message "Starting up the database in mount mode.";
    ${RMAN_PATH} >>${LOG_FILE} <<EOF
    connect target '${connect_string}${rman_privilege}';
    # Attempt to start the RMAN.
    startup mount
EOF
fi

exit ${result}
