#!/bin/bash
# Copyright 2018 Cohesity Inc.
#
# Author: Vivek Velankar
#
# This is a wrapper script to restore the Oracle database to alternate
# location.
# It also appends the output at /tmp/rman_remote_restore.INFO file
#
# $0 -R $ORACLE_HOME/rman -A ORA3 -B VVV -T 15:03:2018-17:19:13
# -S /u01/app/oracle/product/11.2.0.3/db_1/bin/sqlplus -L /mnt/Cohesity1
# -O /home/oracle/dst_base

# Exit from the script if any of the step fails.
set -e

LOG_FILE=""

# Which OS Cohesity agent is running on.
OS=$(uname -a | cut -d " " -f 1)

if [ ${OS} = "AIX" ]; then
  # TODO(aabhas): find replacement for readlink on AIX.
  current_dir=$(dirname $0)
else
  current_dir=$(dirname `readlink -f $0`)
fi

SCRIPT_NAME=`basename "$0"`
# Source needed library functions.
. $current_dir/common_utils.sh

cleanup () {
  result=$?
  log_message "exit status is $result";

  if [ $RESTORE_COMPLETED -eq 1 ]; then
    $RMAN_PATH msglog=${LOG_FILE} append <<EOF
      connect auxiliary '/${rman_privilege}';
      set echo on;
      list backup;
      list datafilecopy all;
EOF
  fi

  if [ -d ${TMP_BACKUP_LOCATION} ]; then
    # ignore stderr from rm
    log_message "removing the soft links directory: ${TMP_BACKUP_LOCATION}";
    safe_rm_force_recursive "$TMP_BACKUP_LOCATION" &> /dev/null
  fi
  if [ $KILL_PMON -ne 0 ]; then
    log_message "Looking for pmon process.."
    pid_op=`ps xu | grep -e "ora_pmon_${ORACLE_SID}$"`
    log_message "Found output as $pid_op"
    num_pid_to_kill=`echo -n "$pid_op"|grep -c '^'`
    if [ $num_pid_to_kill -eq 1 ];then
      to_kill_pid=`echo $pid_op | awk '{print $2}'`
      log_message "Found $to_kill_pid to kill"
      sudo kill -9 $to_kill_pid
    fi
  fi
  exit $result
}

# register the cleanup function for all these signal types
trap cleanup EXIT ERR INT TERM

usage()
{
  echo "Invoke the script with following arguments :-"
  echo -e "\t -R|--rman-path                    Path to RMAN binary."
  echo -e "\t -S|--sqlplus-path                 Path to source SQLPLUS binary."
  echo -e "\t -A|--src-oracle-sid               Oracle source database SID value."
  echo -e "\t -B|--dest-db-name                 Oracle destination database/SID value."
  echo -e "\t -T|--restore-time                 Restore time of database."
  echo -e "\t -L|--restore-destination-location Location of data files files."
  echo -e "\t -M|--mount-points                 CSV mount point list."
  echo -e "\t -I|--id                           Operation ID."
  echo -e "\t -F|--log-file-path                Rman log file path."
  echo -e "\t -D|--data-file-convertor          Data file convertor."
  echo -e "\t -E|--log-file-convertor           Log file convertor."
  echo -e "\t -H|--dst-oracle_home              Destination oracle home."
  echo -e "\t -J|--dst-oracle_base              Destination oracle base."
  echo -e "\t -P|--PIT                          If provided, the restore is done to specified PIT."
  echo -e "\t -n|--num-tempfiles                Number of tempfiles to create."
  echo -e "\t -c|--control_files                CSV control file list."
  echo -e "\t -r|--redo_log_members             Redo log members."
  echo -e "\t -g|--redo_log_num_groups          Num redo log groups."
  echo -e "\t -p|--redo_log_prefix              Redo Member name prefix."
  echo -e "\t -m|--redo_log_size                Redo Member size in MB."
  echo -e "\t -o|--no_open                      Restore in NOOPEN mode."
  echo -e "\t -a|--audit                        Audit log destination."
  echo -e "\t -d|--diag                         Diagnostic log destination."
  echo -e "\t -l|--fra                          FRA location."
  echo -e "\t -C|--cdump                        cdump location."
  echo -e "\t -i|--fra_size                     FRA size in MBs."
  echo -e "\t -f|--config_file                  Path to config file."
  echo -e "\t -s|--shared_pool_size             Shared Pool Size."
  echo -e "\t -t|--sga_target                   SGA Target."
  echo -e "\t -k|--cdb_restore                  CDB Restore workflow."
  echo -e "\t -x|--pdb_list                     List of pdbs to restore."
  echo -e "\t -e|--pfile_path                   If set to valid file path, contents of this file will be appended to pfile created by agent"
  echo -e "\t -h|--duplicate_files              If the backup has duplicate datafiles."
  echo -e "\t -Y|--controlfile_prefix           Prefix to identify the controlfile."
  echo -e "\t -G|--spfile_prefix                Prefix to identify the spfile."
  echo -e "\t -Z|--pfile_location               location of custom pfile created by agent."
  echo -e "\t -j|--source_parameters            Pfile parameters that we propagate here from source db."
  echo -e "\t -N|--user_newname_clause          User specified SET NEWNAME caluse"
  echo -e "\t -k|--nofilenamecheck              NOFILENAME check option for DUPLICATE command"
}

##### Main

# Variables with default value set.
IS_PIT=0
NUM_TEMPFILES=5
CONTROL_FILES=""
REDO_CONF_CLAUSE=""
NUM_LOG_ARCHIVE_DEST=5
KILL_PMON=1
PFILE_LOCATION=""

# Remaining alphabets for future use: K O Q U V W b q u v w x y
# Please remove alphabets from the above list if u are using it.
while getopts "R:S:A:B:T:L:M:Z:I:F:D:E:H:J:P:Y:G:n:c:r:g:p:m:of:a:d:e:l:i:j:C:s:t:z:XhN:k" option; do
  case $option in
    R)
      RMAN_PATH=$OPTARG;;
    S)
      SQLPLUS_PATH=$OPTARG;;
    A)
      SRC_SID=$OPTARG;;
    B)
      DEST_NEW_NAME=$OPTARG;;
    T)
      RESTORE_TIME=$OPTARG;;
    L)
      RESTORE_DEST_LOCATION=$OPTARG;;
    I)
      ID=$OPTARG;;
    F)
      LOG_FILE_PATH=$OPTARG;;
    D)
      DATA_FILE_CONVERTOR=$OPTARG;;
    Y)
      CONTROLFILE_PREFIX=$OPTARG;;
    G)
      SPFILE_PREFIX=$OPTARG;;
    Z)
      PFILE_LOCATION=$OPTARG;;
    E)
      LOG_FILE_CONVERTOR=$OPTARG;;
    H)
      DEST_ORACLE_HOME=$OPTARG;;
    J)
      DEST_ORACLE_BASE=$OPTARG;;
    P)
      IS_PIT=$OPTARG;;
    n)
      NUM_TEMPFILES=$OPTARG;;
    c)
      CONTROL_FILES=$OPTARG;;
    r)
      REDO_LOG_MEMBERS=$OPTARG;;
    g)
      REDO_NUM_GROUPS=$OPTARG;;
    p)
      REDO_MEMBER_PREFIX=$OPTARG;;
    m)
      REDO_MEMBER_SIZE=$OPTARG;;
    o)
      NO_OPEN=1;;
    a)
      ADUMP_LOCATION=$OPTARG;;
    C)
      CDUMP_LOCATION=$OPTARG;;
    d)
      DIAG_LOCATION=$OPTARG;;
    l)
      FRA_LOCATION=$OPTARG;;
    i)
      FRA_SIZE=$OPTARG;;
    j)
      SOURCE_PARAMETERS=$OPTARG;;
    f)
      CONFIG_FILE=$OPTARG;;
    s)
      SHARED_POOL_SIZE=$OPTARG;;
    t)
      SGA_TARGET=$OPTARG;;
    M)
      CHANNEL_VEC=$OPTARG;;
    X)
      CDB_RESTORE=1;;
    z)
      PDB_LIST=$OPTARG;;
    e)
      CUSTOM_PFILE_PATH=$OPTARG;;
    h)
      DUPLICATE_FILES=1;;
    N)
      USER_NEWNAME_CLAUSE=$OPTARG;;
    k)
      NOFILENAMECHECK=1;;
    *) echo "Invalid Option passed"; usage; exit 1 ;;
  esac
done

export ORACLE_HOME=$DEST_ORACLE_HOME
export ORACLE_SID=$DEST_NEW_NAME
export NLS_DATE_FORMAT='DD:MM:YYYY-HH24:MI:SS'

if [[ "x$LOG_FILE_PATH" == "x" ]]; then
  LOG_FILE_PATH="/tmp"
fi
LOG_FILE=${LOG_FILE_PATH}/rman_shell_${ID}.INFO
# Update rman log file permissions.
sudo touch $LOG_FILE
sudo chmod 766 $LOG_FILE

# Modify how to invoke Oracle binaries if the agent is installed with 'root'
setup_invoke_oracle_binaries

log_message "Shell Environment:"
log_message "`env`"

# This restore script only use OS authentication, just pass "/"
f_set_privilege "$DEST_ORACLE_HOME" "/"

if [ -z "$RMAN_PATH" ] || [ -z "$SQLPLUS_PATH" ] || [ -z "$SRC_SID" ] || [ -z "$DEST_NEW_NAME" ]; then
  log_message "RMAN configuration parameters missing."
  log_message "#####$RMAN_PATH, $SQLPLUS_PATH, $SRC_SID, $DEST_NEW_NAME";
  usage;
  exit 1;
fi

if [ -z "$RESTORE_TIME" ] || [ -z "$RESTORE_DEST_LOCATION" ] || [ -z "$CHANNEL_VEC" ]; then
  log_message "RMAN script parameters missing.";
  log_message "######$RESTORE_DEST_LOCATION, $CHANNEL_VEC";
  usage;
  exit 1;
fi

if [ ! -z ${CONFIG_FILE} ] && [ ! -f ${CONFIG_FILE} ]; then
  log_message "Config file: ${CONFIG_FILE} does not exist."
  exit 1;
fi

if [ -z "$DEST_ORACLE_HOME" ] || [ -z "$DEST_ORACLE_BASE" ]; then
  log_message "RMAN DST Oracle home/base parameter is missing.";
  log_message "#####$DEST_ORACLE_HOME, $DEST_ORACLE_BASE";
  usage;
  exit 1;
fi

if [ -z "$SHARED_POOL_SIZE" ] || [ -z "$SGA_TARGET" ]; then
  log_message "SGA_TARGET and SHARED_POOL_SIZE missing.";
  log_message "SHARED_POOL_SIZE: $SHARED_POOL_SIZE";
  log_message "SGA_TARGET: $SGA_TARGET";
fi

# Agent will create this PFILE and also set all the parameters.
if [ -z ${PFILE_LOCATION} ]; then
  log_message "Pfile location not passed."
  exit 1;
else
  TMP_PFILE=$PFILE_LOCATION
fi

sudo chmod 777 $TMP_PFILE

if [[ "${RESTORE_DEST_LOCATION}" == *"/" ]]; then
  WHITELISTED_PATH_LIST+=( "${RESTORE_DEST_LOCATION}.*" )
else
  WHITELISTED_PATH_LIST+=( "${RESTORE_DEST_LOCATION}/.*" )
fi

num_mounts=0
# Parse the mount points information
if [ ! -z "$CHANNEL_VEC" ]; then
  log_message "Multiple mount point information provided.";
  IFS=',' read -r -a mount_info_array <<< "$CHANNEL_VEC"
  num_mounts=${#mount_info_array[@]}
fi

log_message "num mount is $num_mounts";
aux_channel_vec=""
for index in "${!mount_info_array[@]}"; do
    aux_channel_vec="${aux_channel_vec} allocate auxiliary channel aux$((index+1)) device type disk format '${mount_info_array[$index]}/%U'; ";
done

log_message "aux channel vec is $aux_channel_vec";

if [ ${num_mounts} -gt 1 ]; then
    TMP_BACKUP_LOCATION="/tmp/restore_soft_links_${ID}"
    log_message "creating dir ${TMP_BACKUP_LOCATION}"
    safe_mkdir "${TMP_BACKUP_LOCATION}"
    # The tmp restore location should be made read accessible to other users
    sudo chmod 755 ${TMP_BACKUP_LOCATION}
    index=0
    for dirname in `ls ${mount_info_array[0]}`;
    do
      log_message "Linking $dirname";
      CMD="ln -s ${mount_info_array[$index]}/$dirname ${TMP_BACKUP_LOCATION}/$dirname"
      log_message "$CMD";
      `$CMD`
      index=$((index + 1));
    done
    FINAL_BACKUP_CLAUSE="BACKUP LOCATION '$TMP_BACKUP_LOCATION'"
    COHESITY_VIEW="$TMP_BACKUP_LOCATION"
else
    FINAL_BACKUP_CLAUSE="BACKUP LOCATION '${mount_info_array[0]}'"
    COHESITY_VIEW="${mount_info_array[0]}"
fi


start_time="`date "+%D %X"`"
log_message "Restore started at: $start_time"

if [ ! -f ${DEST_ORACLE_HOME}/bin/oracle ]; then
  log_message "Unable to find ${DEST_ORACLE_HOME}/bin/oracle"
  exit 1;
fi

export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:${DEST_ORACLE_HOME}/lib"
export PATH="$PATH:${DEST_ORACLE_HOME}/bin"

oracle_bin_permission=`ls -l ${DEST_ORACLE_HOME}/bin/oracle`
if [ $? -ne 0 ]; then
  log_message "Unable to find permissions for: ${DEST_ORACLE_HOME}/bin/oracle"
  exit 1;
fi

ORACLE_USER=`echo ${oracle_bin_permission} | awk '{print $3}'`
ORACLE_GROUP=`echo ${oracle_bin_permission} | awk '{print $4}'`

if [ -z ${ORACLE_USER} ] || [ -z ${ORACLE_GROUP} ]; then
  log_message "ORACLE_USER(${ORACLE_USER}) or ORACLE_GROUP(${ORACLE_GROUP}) not set."
  exit 1;
fi

log_message "Creating the directories as ${ORACLE_USER}:${ORACLE_GROUP}"

# Create destination database directories.
if [ -z ${ADUMP_LOCATION} ] || [ -z ${DIAG_LOCATION} ]; then
  sudo mkdir -p "$DEST_ORACLE_BASE/admin/$DEST_NEW_NAME"
  sudo chown ${ORACLE_USER}:${ORACLE_GROUP} $DEST_ORACLE_BASE/admin
  sudo chown ${ORACLE_USER}:${ORACLE_GROUP} $DEST_ORACLE_BASE/admin/$DEST_NEW_NAME
fi

if [ -z ${ADUMP_LOCATION} ]; then
  ADUMP_LOCATION="$DEST_ORACLE_BASE/admin/$DEST_NEW_NAME/adump"
  sudo mkdir -p ${ADUMP_LOCATION}
  sudo chown ${ORACLE_USER}:${ORACLE_GROUP} ${ADUMP_LOCATION}
fi

if [ -z ${DIAG_LOCATION} ]; then
  DIAG_LOCATION="$DEST_ORACLE_BASE/admin/$DEST_NEW_NAME/diag"
  sudo mkdir -p ${DIAG_LOCATION}
  sudo chown ${ORACLE_USER}:${ORACLE_GROUP} ${DIAG_LOCATION}
fi

if [ -z ${CDUMP_LOCATION} ]; then
  CDUMP_LOCATION="$DEST_ORACLE_BASE/admin/$DEST_NEW_NAME/cdump"
  sudo mkdir -p ${CDUMP_LOCATION}
  sudo chown ${ORACLE_USER}:${ORACLE_GROUP} ${CDUMP_LOCATION}
fi

if [ -z ${CONTROL_FILES} ]; then
  CONTROL_FILES_PATH="$DEST_ORACLE_HOME/$DEST_NEW_NAME"
  sudo mkdir -p ${CONTROL_FILES_PATH}
  sudo chown ${ORACLE_USER}:${ORACLE_GROUP} ${CONTROL_FILES_PATH}
  CONTROL_FILES="'$DEST_ORACLE_HOME/$DEST_NEW_NAME/control01.ctl'"
fi

if [ -z ${FRA_LOCATION} ]; then
  sudo mkdir -p $DEST_ORACLE_HOME/fast_recovery_area
  sudo chown ${ORACLE_USER}:${ORACLE_GROUP} $DEST_ORACLE_HOME/fast_recovery_area
  FRA_LOCATION="$DEST_ORACLE_HOME/fast_recovery_area/$DEST_NEW_NAME"
  FRA_SIZE=2048
  sudo mkdir -p ${FRA_LOCATION}
  sudo chown ${ORACLE_USER}:${ORACLE_GROUP} ${FRA_LOCATION}

  SID=`echo "$DEST_NEW_NAME" | awk '{print toupper($0)}'`
  sudo mkdir -p $DEST_ORACLE_HOME/fast_recovery_area/$SID
  sudo chown ${ORACLE_USER}:${ORACLE_GROUP} $DEST_ORACLE_HOME/fast_recovery_area/$SID
fi

# Creating temporary initora file for reading SPFILE.
TMP_PFILE_TO_READ_SPFILE="/tmp/init${DEST_NEW_NAME}_2read_spfile.ora"
WHITELISTED_PATH_LIST+=( "${TMP_PFILE_TO_READ_SPFILE}" )
safe_rm_force "__USE_SUDO__" ${TMP_PFILE_TO_READ_SPFILE}

# We need to start a dummy db to extract pfile from spfile
# we stored in backup.
cp ${TMP_PFILE} ${TMP_PFILE_TO_READ_SPFILE}

# Evaluate Spfile path. It is a backup piece with a known prefix in the
# recovery view.
if [ ! -z ${SPFILE_PREFIX} ]; then
  SPFILE_PATH=`find ${COHESITY_VIEW} -name "*${SPFILE_PREFIX}*" | head -n 1`
  PFILE_FROM_SPFILE_LOCATION="/tmp/PFILE_FROM_SPFILE_${ID}.ora"
  WHITELISTED_PATH_LIST+=( "${PFILE_FROM_SPFILE_LOCATION}" )
  safe_rm_force "__USE_SUDO__" ${PFILE_FROM_SPFILE_LOCATION}

  # If found SPFILE, restore its content to a PFILE
  if [ ! -z $SPFILE_PATH ]; then
    RESTORE_SP_FILE="RESTORE SPFILE TO PFILE '${PFILE_FROM_SPFILE_LOCATION}' from '${SPFILE_PATH}';"
    export ORACLE_SID=${DEST_NEW_NAME}

    set +e # Control error handling
    trap - EXIT ERR INT TERM
    $RMAN_PATH msglog=${LOG_FILE} append <<EOF
      connect target '/${rman_privilege}';
      set echo on;
      startup nomount pfile=${TMP_PFILE_TO_READ_SPFILE};
      ${RESTORE_SP_FILE}
      shutdown abort;
      exit;
EOF
    restore_spfile_status=$?
    set -e
    trap cleanup EXIT ERR INT TERM

    if [ ${restore_spfile_status} != 0 ]; then
      if grep 'ORA-19687' ${LOG_FILE}; then
        #
        # For older backup, we cannot determine the backup set that contains the spfile for all cases.
        # Handle the edge case where we have the wrong spfile backup set. Tolerate this case and
        # allow the clone workflow to continue.
        #
        log_message "###Warning: Skip SPFILE proccessing, as ${SPFILE_PATH} does not contain spfile."
        log_message "###         For older backup taken in the past, it is not possible to determine "
        log_message "###         which backupset contains the SPFILE."
      else
        exit_handler ${restore_spfile_status} "#####Error: Extracting pfile from backup spfile failed"
      fi
    else
      log_message "### Successfully extract pfile from backup spfile";
    fi
  else
    log_message "No spfile found in cohesity view. Probably db was started with pfile while taking backup."
    log_message "Hence cannot propagate any parameters from source db."
  fi
fi

# Find parameter value from source db's spfile and add it to pfile.
if [[ -f ${PFILE_FROM_SPFILE_LOCATION} ]]; then
  IFS=',' read -r -a SOURCE_PARAMETER_ARRAY <<< "$SOURCE_PARAMETERS"
  for PARAMETER in "${SOURCE_PARAMETER_ARRAY[@]}"
  do
    # if parameter already set in pfile give more preference to it.
    count=`grep -i "${PARAMETER}=" ${TMP_PFILE} | wc -l`
    if [[ $count -gt 0 ]]; then
      log_message "${PARAMETER} already set in pfile, not adding value from spfile."
      continue
    fi
    log_message "Searching for ${PARAMETER} in SPFILE."
    PARAMETER_VALUE=$(grep -0 -i "\.${PARAMETER}=" ${PFILE_FROM_SPFILE_LOCATION} | head -n 1 | cut -d'=' -f2)
    [ -z $PARAMETER_VALUE ] || echo "${PARAMETER}=${PARAMETER_VALUE}" >> ${TMP_PFILE}
  done
fi

# If the CUSTOM_PFILE_PATH is valid, append the contents to pfile.
# Note that it should come at the end after handling all the other pfile
# options.
if [ ! -z ${CUSTOM_PFILE_PATH} ] ; then
  if [ -f ${CUSTOM_PFILE_PATH} ]; then
    log_message "Appending contents of ${CUSTOM_PFILE_PATH} to ${TMP_PFILE}"
    echo "# Appending contents from ${CUSTOM_PFILE_PATH}" >> ${TMP_PFILE}
    cat ${CUSTOM_PFILE_PATH} >> ${TMP_PFILE}
  else
    log_message "Custom pfile ${CUSTOM_PFILE_PATH} doesn't exist."
  fi
fi

$SQLPLUS_PATH / $sqlplus_privilege << EOF >> ${LOG_FILE}
startup nomount pfile='$TMP_PFILE';
quit;
EOF

CLEAR_CONF_CLAUSE="CONFIGURE DEVICE TYPE DISK CLEAR;\
                   CONFIGURE DEVICE TYPE SBT clear;\
                   CONFIGURE DEVICE TYPE SBT_TAPE clear;\
                   CONFIGURE CHANNEL DEVICE TYPE SBT CLEAR;\
                   CONFIGURE CHANNEL DEVICE TYPE SBT_TAPE CLEAR;"
log_message "CLEAR_CONF_CLAUSE: ${CLEAR_CONF_CLAUSE}"

UPCASED_DEST_NEW_NAME=`echo "$DEST_NEW_NAME" | awk '{print toupper($0)}'`

# Create a destination directory for tempfiles for the database to be created,
# only if non ASM.
if [[ ${RESTORE_DEST_LOCATION} != +*  ]] || [ -d ${RESTORE_DEST_LOCATION} ]; then
  safe_mkdir "__USE_SUDO__" "${RESTORE_DEST_LOCATION}/${UPCASED_DEST_NEW_NAME}"
  sudo chown ${ORACLE_USER}:${ORACLE_GROUP} ${RESTORE_DEST_LOCATION}/${UPCASED_DEST_NEW_NAME}
  safe_mkdir "__USE_SUDO__" "${RESTORE_DEST_LOCATION}/${UPCASED_DEST_NEW_NAME}/TEMPFILE"
  sudo chown ${ORACLE_USER}:${ORACLE_GROUP} ${RESTORE_DEST_LOCATION}/${UPCASED_DEST_NEW_NAME}/TEMPFILE
  NON_ASM=1
fi

TEMPFILE_CLAUSE="";
for num in $(eval echo {1..${NUM_TEMPFILES}})
do
  TEMPFILE_CLAUSE="${TEMPFILE_CLAUSE} SET NEWNAME FOR TEMPFILE $num to \
                  '${RESTORE_DEST_LOCATION}/${UPCASED_DEST_NEW_NAME}/TEMPFILE/temp0${num}.dbf';";
done
log_message "TEMPFILE_CLAUSE: ${TEMPFILE_CLAUSE}"

# By default, use SET NEWNAME FOR DATABASE for renaming all datafiles to
# the single restore destination.
#
# Only SET NEWNAME FOR DATABASE if pfile does not have DB_FILE_NAME_CONVERT. If
# it does, user implies they want full control of how datafile filename
# conversion is done. We should not do SET NEWNAME FOR DATABASE as this will
# effectively ignore user's DB_FILE_NAME_CONVERT setting.
#
# User can also have the option to specify BOTH 'SET NEWNAME' and
# DB_FILE_NAME_CONVERT option. In this case, we use both. We assume the user
# specified SET NEWNAME is not for the whole database, but rather for specific
# datafiles. The restore use case for this is when customer has a few non ASM
# OMF datafiles mixed in with non OMF/ASM files. DB_FILE_NAME_CONVERT can take
# care of the conversion of OMF/ASM files, but will error out if it converts
# the non-ASM OMF files. Oracle recommends to use SET NEWNAME separately for
# these non-ASM OMF files.
#
# Other database files conversion are specifically handle in this script
# individually:
# (a) TEMPFILE name: see TEMPFILE_CLAUSE above
# (b) Online redo log file name: Magneto always set up LOG_FILE_NAME_CONVERT.
#     User can further change LOG_FILE_NAME_CONVERT.
NEWNAME_CLAUSE=""

if [ -z "$USER_NEWNAME_CLAUSE" ]; then
  log_message "Check if DB_FILE_NAME_CONVERT in defined in the PFILE"
  DB_CONVERT_PARAMETER_VALUE=$(grep -0 -i "DB_FILE_NAME_CONVERT" ${TMP_PFILE} | head -n 1 | cut -d'=' -f2)

  # Restore payload does not have SET NEWNAME clause on input. We will
  # determine the SET NEWNAME clause to be used in the following order of
  # steps:
  # (a) Check if user has configured hidden shell environment variable for
  #     Cohesity - "_COHESITY_USE_THIS_SET_NEWNAME". If so use the value of
  #     this hidden shell environment as is, i.e. no sanitization of the
  #     value from this shell environment variable will be done.
  # (b) If not, check if DB_FILE_NAME_CONVERT is configured by user in the
  #     pfile customization. If so, do not configured SET NEWNAME
  # (c) If (a) and (b) are not configured, this script will use the default
  #     SET NEWNAME FOR DATABASE clause.
  if [ ! -z "${_COHESITY_USE_THIS_SET_NEWNAME}" ]; then
    NEWNAME_CLAUSE="${_COHESITY_USE_THIS_SET_NEWNAME}"
    log_message "Use user supplied shell environment to SET NEWNAME for this restore workflow."
  elif [ -z "${DB_CONVERT_PARAMETER_VALUE}" ]; then
     NEWNAME_CLAUSE="SET NEWNAME FOR DATABASE to '${RESTORE_DEST_LOCATION}"

     if [ -z $NON_ASM ]; then
       NEWNAME_CLAUSE="${NEWNAME_CLAUSE}/${UPCASED_DEST_NEW_NAME}/DATAFILE"
     fi

     if [ -z $DUPLICATE_FILES ]; then
       NEWNAME_CLAUSE="${NEWNAME_CLAUSE}/%b';"
     else
       log_message "Duplicate datafile names present, appending file number \
          to the file names."
       NEWNAME_CLAUSE="${NEWNAME_CLAUSE}/%f_%b';"
     fi
  fi

else
  NEWNAME_CLAUSE="$USER_NEWNAME_CLAUSE"
fi

log_message "NEWNAME_CLAUSE: $NEWNAME_CLAUSE"

LOG_ARCHIVE_DEST="";
for num in $(eval echo {1..${NUM_LOG_ARCHIVE_DEST}})
do
  LOG_ARCHIVE_DEST="${LOG_ARCHIVE_DEST} set log_archive_dest_${num}=''";
done
log_message "LOG_ARCHIVE_DEST: ${LOG_ARCHIVE_DEST}"

if [ ! -z ${REDO_LOG_MEMBERS} ]; then

  if [ -z ${REDO_NUM_GROUPS} ] || [ -z ${REDO_MEMBER_PREFIX} ] || [ -z ${REDO_MEMBER_SIZE} ]; then
    log_message " ####Insufficient redo log configuration specified.";
    exit 1;
  fi

  MEMBER_PATHS=($(echo ${REDO_LOG_MEMBERS} | tr "," "\n"))
  for group in $(eval echo {1..${REDO_NUM_GROUPS}}); do
    if [ $group -ne 1 ]; then
      REDO_CONF_CLAUSE="${REDO_CONF_CLAUSE},"
    fi
    REDO_CONF_CLAUSE="${REDO_CONF_CLAUSE} GROUP $group ("
    redo_name="${REDO_MEMBER_PREFIX}${group}.log"
    for member in ${!MEMBER_PATHS[@]}; do
      if [ $member -ne 0 ]; then
        REDO_CONF_CLAUSE="${REDO_CONF_CLAUSE},"
      fi
      REDO_CONF_CLAUSE="${REDO_CONF_CLAUSE}'${MEMBER_PATHS[$member]}/${redo_name}'"
    done
    REDO_CONF_CLAUSE="${REDO_CONF_CLAUSE}) SIZE ${REDO_MEMBER_SIZE}M"
  done
fi

if [ ! -z "${REDO_CONF_CLAUSE}" ]; then
  REDO_CONF_CLAUSE="LOGFILE ${REDO_CONF_CLAUSE}"
  log_message "REDO_CONF_CLAUSE: ${REDO_CONF_CLAUSE}"
fi

if [ -z ${NO_OPEN} ]; then
  FINAL_BACKUP_CLAUSE="${FINAL_BACKUP_CLAUSE}"
else
  FINAL_BACKUP_CLAUSE="${FINAL_BACKUP_CLAUSE} NOOPEN"
fi

if [ ! -z $NOFILENAMECHECK ]; then
  FINAL_BACKUP_CLAUSE="${FINAL_BACKUP_CLAUSE} NOFILENAMECHECK"
fi

FINAL_BACKUP_CLAUSE="${FINAL_BACKUP_CLAUSE};"

log_message "FINAL_BACKUP_CLAUSE: ${FINAL_BACKUP_CLAUSE}"

# If List of PDBs are specified, restore only the subset of the PDBs.
if [ ! -z ${PDB_LIST} ]; then
  PDB_CLAUSE="PLUGGABLE DATABASE ${PDB_LIST}"
fi
log_message "PDB_CLAUSE: ${PDB_CLAUSE}"

if [ $IS_PIT -eq 1 ]; then
  $RMAN_PATH msglog=${LOG_FILE} append <<EOF
  connect auxiliary '/${rman_privilege}';
  set echo on;
  $CLEAR_CONF_CLAUSE
  run {
  SET COMMAND ID TO '$ID';
  ${aux_channel_vec}
  $TEMPFILE_CLAUSE
  $NEWNAME_CLAUSE
  # Alter some SPFILE parameters during the clone.
  DUPLICATE DATABASE TO '$DEST_NEW_NAME'
  ${PDB_CLAUSE}
  UNTIL TIME "TO_DATE('$RESTORE_TIME', 'DD:MM:YYYY-HH24:MI:SS')"
  ${REDO_CONF_CLAUSE}
  ${FINAL_BACKUP_CLAUSE}
  #NOFILENAMECHECK;
  }
EOF
else
  $RMAN_PATH msglog=${LOG_FILE} append <<EOF
  connect auxiliary '/${rman_privilege}';
  set echo on;
  $CLEAR_CONF_CLAUSE
  run {
  SET COMMAND ID TO '$ID';
  ${aux_channel_vec}
  $TEMPFILE_CLAUSE
  $NEWNAME_CLAUSE
  # Alter some SPFILE parameters during the clone.
  DUPLICATE DATABASE TO '$DEST_NEW_NAME'
  ${PDB_CLAUSE}
  ${REDO_CONF_CLAUSE}
  ${FINAL_BACKUP_CLAUSE}
  #NOFILENAMECHECK;
  }
EOF
fi

result=$?
RESTORE_COMPLETED=1

# Setting db_unique_name to new sid.
$SQLPLUS_PATH / $sqlplus_privilege << EOF >> ${LOG_FILE}
alter system set db_unique_name='$DEST_NEW_NAME' scope=spfile;
quit;
EOF

if [ ! -z ${NO_OPEN} ]; then
  log_message "#####No Open Restore completed on $ORACLE_SID at: ${RESTORE_TIME} with status $result";
  KILL_PMON=0
  exit $result;
fi

# Open all the PDBs after the restore is done.
if [ ! -z ${CDB_RESTORE} ]; then
  PDB_OPEN_CLAUSE="ALTER PLUGGABLE DATABASE ALL OPEN;"
fi
# Starting up the Database.
$SQLPLUS_PATH / $sqlplus_privilege << EOF >> ${LOG_FILE}
shutdown immediate;
startup;
${PDB_OPEN_CLAUSE}
quit;
EOF

if [ $result -eq 0 ];then
    log_message "#####Restore completed on $ORACLE_SID at: ${RESTORE_TIME}";
    KILL_PMON=0
else
    log_message "#####Restore failed on $ORACLE_SID at: ${RESTORE_TIME}";
fi

exit $result
