#!/bin/bash
# Copyright 2018 Cohesity Inc.
#
# Author: Vivek Velankar
#
# This is a wrapper script to invoke the rman scripts.
# It also appends the output at rman_shell.INFO file.

# Read the stdin as first step.
read connect_string;

# Exit from the script if any of the step fails.
set -e

LOG_FILE=""

# Which OS Cohesity agent is running on.
OS=$(uname -a | cut -d " " -f 1)

if [ ${OS} = "AIX" ]; then
  # TODO(aabhas): find replacement for readlink on AIX.
  current_dir=$(dirname $0)
else
  current_dir=$(dirname `readlink -f $0`)
fi

SCRIPT_NAME=`basename "$0"`
# Source needed library functions.
. $current_dir/common_utils.sh

usage()
{
  echo "Invoke the script with following arguments :-"
  echo -e "\t -R|--rman-path            Path to RMAN binary."
  echo -e "\t -H|--oracle-home          Path to ORACLE home directory."
  echo -e "\t -S|--oracle-sid           Oracle database SID value."
  echo -e "\t -T|--tag                  Operation TAG."
  echo -e "\t -L|--script-location      Location of RMAN script to execute."
  echo -e "\t -I|--id                   Operation ID."
  echo -e "\t -M|--mount_points         CSV Mount point list."
  echo -e "\t -D|--db-incarnation       DB Incarnation ID used while restore."
  echo -e "\t -F|--log-file-path        RMAN log file path."
  echo -e "\t -X|--log-file-copy-path   RMAN log file path second copy location."
  echo -e "\t -Y|--log-file-copy-keep_days Retention for second copy RMAN log files. "
  echo -e "\t -B|--db-id                Database Id."
  echo -e "\t -P|--restore-pt           Point In Time for Restore"
  echo -e "\t -N|--remote-node-info     CSV Hostname followed by port number followed by mount point."
  echo -e "\t -U|--db-unique-name       Unique name for database.It can be appeneded by db_domain in some cases."
  echo -e "\t -s|--is_standby           Source database is an Oracle Standby database."
  echo -e "\t -A|--archivelog-keep-days Number of days to keep source archived logs after they are backed up."
  echo -e "\t -C|--pdb-list             List of pdbs to restore."
  echo -e "\t -r|--rman-backup-type     Type of rman backup(ImageCopy or BackupSets)."
  echo -e "\t -b|--backup-level         Backup level, 0 for Full and 1 for incremental."
  echo -e "\t -O|--sbt-lib-path         SBT library path on this Oracle host."
  echo -e "\t -V|--vips-list            CSV VIPs list."
  echo -e "\t -W|--full-view-path       Full view path where SBT is going to dump data."
  echo -e "\t -d|--mnmc_use_sid         MNMC to use SID instead of service name."
  echo -e "\t -e|--mnmc-use-connect-dsc MNMC to use Oracle Connect Descriptor, instead of EZconnect."
}

##### Main

#
# Maintain all options in an exported shell variable. Shell scripts called
# by this script will reference the complete list of options via this exported
# Bash variable
#
export RMAN_SHELL_SCRIPT_OPTIONS="R:H:S:T:L:I:M:D:F:B:P:U:N:A:C:O:V:W:r:b:scX:Y:de"

while getopts "${RMAN_SHELL_SCRIPT_OPTIONS}" option; do
  case $option in
    R)
      RMAN_PATH=$OPTARG;;
    H)
      export ORACLE_HOME=$OPTARG;;
    S)
      export ORACLE_SID=$OPTARG;;
    T)
      TAG=$OPTARG;;
    L)
      script_location=$OPTARG;;
    I)
      ID=$OPTARG;;
    M)
      mount_point_list=$OPTARG;;
    D)
      db_incarnation=$OPTARG;;
    F)
      LOG_FILE_PATH=$OPTARG;;
    X)
      LOG_FILE_COPY_PATH=$OPTARG;;
    Y)
      LOG_FILE_COPY_PATH_KEEP_DAYS=$OPTARG;;
    B)
      DATABASE_ID=$OPTARG;;
    P)
      POINT_IN_TIME=$OPTARG;;
    N)
      remote_node_info_list=$OPTARG;;
    U)
      db_unique_name=$OPTARG;;
    s)
      is_physical_standby=1;;
    A)
      archivelog_keep_days=$OPTARG;;
    C)
      PDB_LIST=$OPTARG;;
    c)
      CROSSCHECK=1;;
    O)
      SBT_LIB_PATH=$OPTARG;;
    V)
      VIPS_LIST=$OPTARG;;
    W)
      VIEW_PATH=$OPTARG;;
    r)
      RMAN_BACKUP_TYPE=$OPTARG;;
    b)
      BACKUP_LEVEL=$OPTARG;;
    d)
      MNMC_USE_SID=1;;
    e)
      MNMC_USE_CONNECT_DSC=1;;
    *) echo "Invalid Option passed"; usage; exit 1 ;;
  esac
done

if [ -z "$RMAN_PATH" ] || [ -z "$ORACLE_HOME" ] || [ -z "$ORACLE_SID" ]; then
  echo "RMAN configuration parameters missing."
  echo "*** $RMAN_PATH, $ORACLE_HOME, $ORACLE_SID";
  usage;
  exit 1;
fi

if [ -z "$TAG" ] || [ -z "$connect_string" ] || [ -z "$script_location" ]; then
  echo "RMAN script parameters or authentication is missing.";
  echo "*** $TAG, $script_location";
  usage;
  exit 1;
fi

# Checking SBT params.
if [ "${RMAN_BACKUP_TYPE}" = "${SBT_BACKUP}" ]; then
  if [ -z "$VIEW_PATH" || -z "$SBT_LIB_PATH" || -z "$VIPS_LIST" ]; then
    echo "SBT configuration parameters missing."
    echo "*** $VIEW_PATH, $SBT_LIB_PATH, $VIPS_LIST";
    usage;
    exit 1;
 fi
fi

export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:${ORACLE_HOME}/lib"
export PATH="$PATH:${ORACLE_HOME}/bin"
export NLS_DATE_FORMAT='DD:MM:YYYY-HH24:MI:SS'

if [ -z "$mount_point_list" ]; then
    mount_point_list="/tmp"
fi

# Parse the mount point list.
IFS=',' read -r -a mount_array <<< "$mount_point_list"
num_local_mounts=${#mount_array[@]}

num_remote_mounts=0
# Parse the nodes information
if [ ! -z "$remote_node_info_list" ]; then
  echo "Nodes information provided.";
  IFS=',' read -r -a node_info_array <<< "$remote_node_info_list"
  num_remote_mounts=$((num_remote_mounts+${#node_info_array[@]}))
fi

# Deserialize comma separated remote node info, into respective vectors:
# - fqdn vector
# - port number vector
# - remote mount path vector
# - remote Oracle Sid vector
fqdn_vec=""
port_num_vec=""
remote_mount_pts=""
remote_channel_oracle_sid_vec=""

for index in "${!node_info_array[@]}"
do
  echo "node $index is ${node_info_array[$index]}";
  IFS=':' read -r -a temp_info <<< "${node_info_array[$index]}"
  # Older Magneto may not have sent the 4th element - Oracle SID
  if [ ${#temp_info[@]} -lt 3 ]; then
    echo "Incorrect syntax for node's information";
    usage
    exit 1
  fi

  fqdn_vec=(${fqdn_vec[@]} ${temp_info[0]});
  port_num_vec=(${port_num_vec[@]} ${temp_info[1]});
  remote_mount_pts=(${remote_mount_pts[@]} ${temp_info[2]});
  # If older Magneto did not send the Oracle SID, the following Bash variable
  # will accummulate empty string. The command will not fail.
  # 'remote_channel_oracle_sid_vec' will just be an empty string.
  remote_channel_oracle_sid_vec=(${remote_channel_oracle_sid_vec[@]} ${temp_info[3]});
done

f_set_privilege "$ORACLE_HOME" "$connect_string"

if [ "$rman_privilege" == " as sysbackup" ]; then
  # Add this check as we are not able to easy connect to sysdba user
  # with sysbackup privilege. Revert to use the default of no additional 'as'
  # clause to connect the remote RMAN channel.
  if [ ! -z ${node_info_array[0]} ]; then
    set +e
    $RMAN_PATH <<EOF > /dev/null
      connect target '$connect_string@${fqdn_vec[0]}:${port_num_vec[0]}/$db_unique_name$rman_privilege';
EOF
    if [ $? != 0 ]; then
      rman_privilege=""
    fi
    set -e
  fi
fi

if [[ "x$LOG_FILE_PATH" == "x" ]]; then
  LOG_FILE_PATH="${mount_array[0]}"
  # We want single file in mount path(in case of backup since there are lot of
  # runs), assumption is if we don't pass log file path, we will always pass
  # mount paths.
  LOG_FILE=${LOG_FILE_PATH}/rman_shell.INFO
else
  LOG_FILE=${LOG_FILE_PATH}/rman_shell_${ID}.INFO
fi
touch $LOG_FILE
sudo chmod -R 766 $LOG_FILE

# Modify how to invoke Oracle binaries if the agent is installed with 'root'
setup_invoke_oracle_binaries

log_message "Shell Environment:"
log_message "`env`"

# Parallelism is number of mount points.
parallelism=$((num_local_mounts+num_remote_mounts))

channel_vec=""
channel_clear_vec=""

for index in "${!mount_array[@]}"
do
  if [ ! -d "${mount_array[$index]}" ]; then
      echo "Mount point ${mount_array[$index]} does not exist."
      usage
      exit 1
  fi

  channel_vec="${channel_vec} allocate channel coh$((index+1)) device type disk format \"${mount_array[$index]}/%U\";";
  channel_clear_vec="${channel_clear_vec} release channel coh$((index+1));";
done

# This contains password so dont print this to logs
remote_channel_vec=""
remote_channel_clear_vec=""

# Compose RMAN command to configure RMAN RAC remote channels
for index in "${!node_info_array[@]}"
do
  # Compose MNMC remote channel naming method, which can be one of the following:
  # (a) Oracle Connect Descriptor:
  #      (DESCRIPTION=(ADDRESS=(PROTOCOL=tcp)(HOST=10.2.103.25)(PORT=1521))(CONNECT_DATA=(SID=cdb1)))
  # (b) Oracle EZConnect form:
  #      10.2.150.25:1521/cdb1.cohesity.com
  if [ -z ${MNMC_USE_CONNECT_DSC} ]; then
     ORACLE_NAMING_METHOD="${connect_string}@${fqdn_vec[$index]}:${port_num_vec[$index]}/$db_unique_name"
  else
     remote_channel_oracle_sid=${remote_channel_oracle_sid_vec[$index]}
     log_message "remote_channel_oracle_sid=$remote_channel_oracle_sid, index=$index"
     if [ -z ${MNMC_USE_SID} ] || [ -z ${remote_channel_oracle_sid} ]; then
       # Use Oracle Service Name to connect if Agent gflag linux_agent_oracle_mnmc_use_sid says so.
       # Fall back to use service name if gflag asks for SID but we do not have SID provided (old Magneto does not send SID)
       CONNECT_DATA_OPTION="(SERVICE_NAME=$db_unique_name)"
     else
       # Use Oracle SID to connect.
       # Oracle SID ensures RAC listener will not be able to re-route the connection to a different RAC node.
       CONNECT_DATA_OPTION="(SID=${remote_channel_oracle_sid})"
     fi
     ORACLE_NAMING_METHOD="${connect_string}@(DESCRIPTION=(ADDRESS=(PROTOCOL=tcp)(HOST=${fqdn_vec[$index]})(PORT=${port_num_vec[$index]}))(CONNECT_DATA=$CONNECT_DATA_OPTION))"
  fi
  # Log the naming method used to set up remote RMAN channel (masked connect string)
  log_message "remote channel $((num_local_mounts+index+1)) uses ORACLE_NAMING_METHOD: `sed -n 's/^.*\(@.*\)/<connect-string>\1/p'<<<${ORACLE_NAMING_METHOD}`, AS CLAUSE: ${rman_privilege}"
  remote_channel_vec="${remote_channel_vec} allocate channel cohr$((num_local_mounts+index+1)) device type disk connect \"${ORACLE_NAMING_METHOD}$rman_privilege\" format \"${remote_mount_pts[$index]}/%U\";";
  remote_channel_clear_vec="${remote_channel_clear_vec} release channel cohr$((num_local_mounts+index+1));";
done

# Are we invoking an RMAN script or a shell script to do the actual work?
# If the script file extension is '.rman', it is an rman script, otherwise
# we assume it is a shell script
script_file_ext=${script_location##*.}
script_file_ext=`echo "$script_file_ext" | awk '{print toupper($0)}'`

# Build the RMAN script command with argument list
# The following logic assumes the argument passed into this script matches
# the script passed in via the -L switch.
if [ "$script_file_ext" = "RMAN" ]; then
  rman_cmd="@$script_location '$ID' '$TAG'"

  if [[ "x$POINT_IN_TIME" != "x" ]] ; then
    rman_cmd="$rman_cmd '$POINT_IN_TIME'"
  fi

  if [[ "x$db_incarnation" != "x" ]] ; then
    rman_cmd="$rman_cmd $db_incarnation";
  elif [[ "x$DATABASE_ID" != "x" ]] ; then
    rman_cmd="$rman_cmd $DATABASE_ID";
  fi
  if [[ "x$PDB_LIST" != "x" ]] ; then
    rman_cmd="$rman_cmd $PDB_LIST";
  fi
  # Passing this unconditionally to avoid rman syntax error in non sbt case.
  rman_cmd="$rman_cmd '$sbt_channel_vec' '$sbt_channel_clear_vec' '$channel_vec' '$channel_clear_vec'";
fi

echo "-------------------------" >> ${LOG_FILE}
log_message "Date: " `date`
log_message "Wrapper script args: $@"
if [ "$script_file_ext" = "RMAN" ]; then
    log_message "RMAN Script command: $rman_cmd"
else
    log_message "Shell script command: $script_location $@"
fi
log_message "Parallelism is set to: $parallelism";

# Parse the mount point list.
IFS=',' read -r -a vips_array <<< "$VIPS_LIST"
num_sbt_channels=${#vips_array[@]}

# Setting device type base on rman backup(imagecopy, sbt).
device_type="DISK"
control_file_format_clause="CONFIGURE CONTROLFILE AUTOBACKUP FORMAT FOR DEVICE TYPE DISK TO '${mount_array[0]}/cf_%F';"
if [ "${RMAN_BACKUP_TYPE}" = "${SBT_BACKUP}" ]; then
  device_type="SBT"
  control_file_format_clause="";
  parallelism=${num_sbt_channels};
  channel_vec=""
  channel_clear_vec=""
fi

# After this point we want to handle the error.
set +e

# Get the channel information from DB.
RMAN_CHANNEL_OUTPUT=$($RMAN_PATH <<EOF
connect target '${connect_string}${rman_privilege}';
show channel;
quit;
EOF
)

STALE_CHANNEL_CONFIG_STR=$(echo "$RMAN_CHANNEL_OUTPUT" | egrep "CONFIGURE CHANNEL [0-9]+")
CLEAR_CHANNEL_LIST=""

if [[ "x${STALE_CHANNEL_CONFIG_STR}" != "x" ]];then
  CHANNEL_LIST=$( echo "$STALE_CHANNEL_CONFIG_STR" | cut -d " " -f 3)
  for i in $(eval echo ${CHANNEL_LIST});
  do
    CLEAR_CHANNEL_LIST="${CLEAR_CHANNEL_LIST} CONFIGURE CHANNEL ${i} DEVICE TYPE DISK CLEAR;";
  done
  log_message "Stale channels to clear are: ${CLEAR_CHANNEL_LIST}"
else
  log_message "No Stale Channel configuration found in the DB."
fi

$RMAN_PATH msglog=${LOG_FILE} APPEND <<EOF
    connect target '${connect_string}${rman_privilege}';

    # Configuration Section:
    CONFIGURE DEVICE TYPE DISK CLEAR;
    CONFIGURE DEVICE TYPE SBT CLEAR;
    CONFIGURE DEVICE TYPE SBT_TAPE CLEAR;
    CONFIGURE CHANNEL DEVICE TYPE SBT CLEAR;
    CONFIGURE CHANNEL DEVICE TYPE SBT_TAPE CLEAR;
    CONFIGURE DEFAULT DEVICE TYPE TO DISK;

    CONFIGURE DEVICE TYPE ${device_type} PARALLELISM $parallelism;
    ${CLEAR_CHANNEL_LIST}
    ${control_file_format_clause}

    # Maintenance Section: ***Uncomment this to enable the CROSSCHECK***
    # -------------------
    # Crosscheck Archivelogs to avoid the backup failure:
    # CHANGE ARCHIVELOG ALL CROSSCHECK;
    # DELETE NOPROMPT EXPIRED ARCHIVELOG ALL;
EOF

result=$?

if [ $result -eq 0 ]; then
    #
    # Invoking bash script or rman script?
    #
    if [ "$script_file_ext" = "RMAN" ]; then
        # Call rman script, passing it the rman argument
        # list as bind variables
        $RMAN_PATH msglog=${LOG_FILE} APPEND <<EOF
            connect target '${connect_string}${rman_privilege}';
            # Run the custom script provided by the user.
            $rman_cmd
EOF
    else
        # Call shell script, with the original full argument list
        export LOG_FILE
        export RMAN_PATH
        export connect_string
        export rman_privilege
        export sqlplus_privilege
        export channel_vec
        export channel_clear_vec
        export remote_channel_vec
        export remote_channel_clear_vec
        $script_location "$@"
    fi

    result=$?
fi

if [ $result -eq 0 ]; then
    $RMAN_PATH msglog=${LOG_FILE} APPEND <<EOF
        connect target '${connect_string}${rman_privilege}';
        # Clear the rman job configuration - channels, control file format
        CONFIGURE CONTROLFILE AUTOBACKUP FORMAT FOR DEVICE TYPE DISK clear;
EOF

    result=$?
fi

if [ $result -eq 0 ]; then
    log_message "***Operation completed on $ORACLE_SID at: $(date)"
else
    log_message "***Operation failed on $ORACLE_SID at: $(date)"
    log_message "Starting up the database.";
    $RMAN_PATH msglog=${LOG_FILE} APPEND <<EOF
    connect target '${connect_string}${rman_privilege}';
EOF
fi

# Copy log file to user specified location
if [ "x$LOG_FILE_COPY_PATH" != "x" ]; then
  if [ -d ${LOG_FILE_COPY_PATH} ]; then
    LOG_FILE_COPY_PATH_PREFIX=${ORACLE_SID}
    [ "x$db_unique_name" == "x" ] || LOG_FILE_COPY_PATH_PREFIX=${db_unique_name}
    # Remember this prefix for later retention enforcement
    PURGE_PREFIX=${LOG_FILE_COPY_PATH_PREFIX}

    STRIPED_ID=`cut -d '_' -f2-3 <<< ${ID}`
    LOG_FILE_COPY_PATH_PREFIX="${LOG_FILE_COPY_PATH_PREFIX}_${STRIPED_ID}"

    # For image copy backup, is this a FULL, INCR, or LOG backup?
    if [ "${RMAN_BACKUP_TYPE}" == "${IMAGECOPY_BACKUP}" ]; then
      TYPE_PREFIX="LOG_BACKUP"

      # Is it INCR? Check if incremental merge is triggered
      grep 'recovering datafile copy file number' $LOG_FILE > /dev/null
      if [ $? -eq 0 ]; then
        TYPE_PREFIX="INCR_BACKUP"
      else
        grep 'Starting recover at' $LOG_FILE > /dev/null
        [ $? -eq 0 ] && TYPE_PREFIX="FULL_BACKUP"
      fi
      LOG_FILE_COPY_PATH_PREFIX="${LOG_FILE_COPY_PATH_PREFIX}_${TYPE_PREFIX}"
    fi

    # Assemble final RMAN log file name
    LOG_FILE_2=${LOG_FILE_COPY_PATH}/${LOG_FILE_COPY_PATH_PREFIX}_rman_shell.INFO

    # Expose the RMAN log at the location the customer specified
    log_message "Copying RMAN backup log to ${LOG_FILE_2}"
    cp $LOG_FILE $LOG_FILE_2
    # Age out older logs that past the retention period
    # TODO (Manipal):  Find a way to add below to safe_rm.
    find ${LOG_FILE_COPY_PATH} -mtime +${LOG_FILE_COPY_PATH_KEEP_DAYS} -name "${PURGE_PREFIX}*" -exec rm {} \;
  fi
fi

exit $result
