#!/bin/bash
# Copyright 2018 Cohesity Inc.
#
# Author: Soumya Poddar
#
# This is a wrapper script to invoke crsctl command on Oracle RAC.
# crsctl command on RAC used to get info about the RAC cluster status
# cluster health
#

usage() {
  echo "Invoke the script with following arguments :-"
  echo -e "\t -P|--crsctl-path           Path to CRSCTL binary"
}

#
# Following function runs 'crsctl check crs' command
# and checks for services which are stopped, and prints
# those service names to STDERR.
#
# Sample output of crsctl, where CRS is not online.
#
# [oracle@oracle-rac-003 ~]$ crsctl check crs
# CRS-4638: Oracle High Availability Services is online
# CRS-4535: Cannot communicate with Cluster Ready Services
# CRS-4529: Cluster Synchronization Services is online
# CRS-4533: Event Manager is online

crsctl_cluster_check() {
  $CRSCTL_CMD check crs | grep -v "online" >&2;
}

#
# Following function runs crsctl status resource command to get the DB instances
# running on the current RAC node and prints the tuple
#
# <DB-Instance-Name> <ORACLE_HOME>
#
# Err handling:
# If crsctl status resource command fails, invoke crsctl_cluster_check()
# to check the cluster status and error out with exit code 1.
#
# Following commands are executed by the below function.
#
# crsctl status resource -w <where> Type == ora.databse.type AND server == current_host
#
# Above command yields DB names which are running in the current host.
#
# Ex:
# [oracle@oracle-rac-001 ~]$ crsctl status resource  -w "((TYPE = ora.database.type) AND (LAST_SERVER = $LOCAL_NODE_NAME))" | grep ^NAME | sed 's/.*=//'
# ora.rac11db.db
# ora.rac12db.db
# ora.sn001.db
#
# For each DB run the following command.
#
# crsctl status resource -w <where> Name == <DB-NAME> AND server = current_host -f
#
# Above command returns complete info about the DB [specified by DB-NAME ]
# GREP the instance name and Oracle_Home from the complete info
#
# Ex:
# [oracle@oracle-rac-001 ~]$ crsctl status resource  -w "((NAME = ora.rac12db.db) AND (LAST_SERVER = $LOCAL_NODE_NAME))" -f | grep ^USR_ORA_INST_NAME= | awk -F= '{ print $2}'
# rac12db1
#
# [oracle@oracle-rac-001 ~]$ crsctl status resource  -w "((NAME = ora.rac12db.db) AND (LAST_SERVER = $LOCAL_NODE_NAME))" -f | grep ^ORACLE_HOME= | awk -F= '{ print $2}'
# /u01/app/oracle/product/12.1.0/db_1
#
# At the end prints the tuple DB-Instance-Name ORACL_HOME
#
# rac12db1 /u01/app/oracle/product/12.1.0/db_1
#
crsctl_status() {
  # Use the 'crsctl get nodename' command to obtain the name of the local node to
  # avoid db discovery failures caused by mismatches between the LAST_SERVER and
  # the local hostname obtained with 'hostname -s'
  LOCAL_NODE_NAME=$($CRSCTL_CMD get nodename)
  if [ $? -ne 0 ]; then
    crsctl_cluster_check;
    exit 1;
  fi

  DB_RESOURCES=$($cmd "((TYPE = ora.database.type) AND (LAST_SERVER = $LOCAL_NODE_NAME)))")
  if [ $? -ne 0 ]; then
    crsctl_cluster_check;
    exit 1;
  fi

  for resource in $(echo "$DB_RESOURCES" | grep ^NAME | sed 's/.*=//'); do
      full_resource=$($cmd "((NAME = $resource) AND (LAST_SERVER = $LOCAL_NODE_NAME)))" -f)
      db_ins_name=$(echo "$full_resource" | grep ^USR_ORA_INST_NAME= | awk -F= '{ print $2 }')
      ora_home=$(echo "$full_resource" | grep ^ORACLE_HOME= | awk -F= '{ print $2 }')
      echo $db_ins_name $ora_home
  done
}

# Main.
while getopts "P:" option; do
   case $option in
     P)
       CRSCTL_CMD=$OPTARG;;
     *) echo "Invalid argument passed"; usage; exit 1 ;;
   esac
done

if [ -z "$CRSCTL_CMD" ]; then
   echo "CRSCTL CMD path missing";
   usage;
   exit 1;
fi

cmd="$CRSCTL_CMD status resource -w"
crsctl_status
