#!/bin/bash
#
# Copyright 2018 Cohesity Inc.
#
# Author: Sai Akhil.
#
# Diagnostics collections in Linux Environment.
# Also collects Oracle related configuration details.

TIMESTAMP=`date '+%s'`
FINAL_TARBALL_LOC="/tmp"
CURRENT_DIR=$(dirname `readlink -f $0`)
SCRIPT_NAME="$(basename "$(test -L "$0" && readlink "$0" || echo "$0")")"

# Installer usage related functions.
f_print_usage() {
  set +x

  local my_exit_status=$1
  echo " "
  echo "Usage: "
  echo "  $SCRIPT_NAME [-O|--output_dir] [Full path directory where tarball is created]"
  echo " "
  echo "To run commands specific to root/sudo user:"
  echo "  $SCRIPT_NAME [-s|--sudo]"
  echo " "
  echo "To get help:"
  echo "  $SCRIPT_NAME [-h|--help]"
  echo " "
  exit $my_exit_status
}

# Prints the command versions of necessary commands.
f_print_command_versions() {
  f_log "COMMAND VERSIONS:"

  # Command version for mount.
  f_log "`mount -V`"

  # Command version for unmount of filesystem.
  f_log "`umount -V`"

  # Command version for blkid
  f_log "`blkid -V`"

  # Command version for sudo.
  f_log "`sudo -V`"

  # Command version for lvcreate.
  f_log "`lvcreate --version | grep "version"`"

  # Command version for wget.
  f_log "`wget -V | grep "GNU Wget"`"

  # Command version for chmod.
  f_log "`chmod --version | grep chmod`"

  # Command version for dmsetup.
  f_log "`dmsetup version | grep "version"`"

  # Command version for ls.
  f_log "ls version: `ls --version | grep "ls"`"

  # Command version for rsync.
  f_log "`rsync --version | grep "version"`"

  # Command version for wget.
  f_log "`wget --version | grep "GNU Wget"`"

  # Command version for dmsetup.
  f_log "dmsetup `dmsetup --version 2> /dev/null | grep Library`"

  # Command version for blkid.
  f_log "`blkid -v`"
}

f_marker() {
  echo "--------------------------------------------------------------------------------"
}

f_log() {
  echo "==> $@"
}

# Parses through the arguments and extracts their arguments into variables.
f_parse_arguments() {

  if ! options=$(getopt -o O:sh -l output_dir:,sudo,help -- "$@")
  then
    f_print_usage
    exit 1
  fi

  set -- $options

  while [ $# -gt 0 ]
  do
    # Consume next (1st) argument.
    case $1 in
    -O|--output_dir)
      eval FINAL_TARBALL_LOC=${2};
      shift;
      ;;
    -s|--sudo)
      ARG_RUN_AS_SUDO=1;
      echo "Running as Sudo User.";
      ;;
    -h|--help)
      f_print_usage
      exit 1
      ;;
    (--)
      shift; break;;
    (-*)
      echo "$0: error - unrecognized option $1" 1>&2; exit 1;;
    (*)
    break;;
    esac
  # Fetch next argument as 1st.
  shift
  done
}

f_separator () {
  echo "################################################################################"
}

# Parses cohesityagent files, returns the variables from the installed service.
f_get_vars_from_installed_service() {
  local agent_service_file=$1

  local service_user=`grep "^COHESITY_USER=" $agent_service_file | awk -F'=' '{ print $2 }' | tr -d '"'`
  if [ $? -ne 0 ]; then
    f_log "Error: Failed to find service user."
  fi
  INSTALLED_AGENT_SERVICE_USER=$service_user

  local service_user_group=`grep "^COHESITY_GROUP=" $agent_service_file | awk -F'=' '{ print $2 }' | tr -d '"'`
  if [ $? -ne 0 ]; then
    f_log "Error: Failed to find service user group."
  fi
  INSTALLED_AGENT_SERVICE_USER_GROUP=$service_user_group

  local install_dir=`grep "^COHESITY_INSTALL_DIR=" $agent_service_file | awk -F'=' '{ print $2 }' | tr -d '"'`
  if [ $? -ne 0 ]; then
    f_log "Error: Failed to find installation directory"
  fi
  INSTALLED_AGENT_INSTALL_DIR=$install_dir

  return 0
}

# Returns the software version of the installed service.
f_find_software_version() {
  local exec_full_path=$1
  if [ ! -f $exec_full_path ]; then
    echo "Not Found"
    return 0
  fi
  local software_version=$($exec_full_path --version | head -1 | awk '{ print $3 }' | awk -F'-' '{ print $2"-"$3}')

  # If this build is from master branch or any other developer branches, then
  # SOFTWARE_VERSION will be set to git branch name by above command. Detect
  # this below with regex and if true, set SOFTWARE_VERSION to
  # 0.0.0-<branchname>.

  echo $software_version | grep "^[0-9]\.[0-9]" &> /dev/null
  if [ $? -ne 0 ]; then
    software_version="0.0.0-$software_version"
  fi
  echo $software_version
  return 0
}

# Run with sudo to execute sudo specific commands.
f_run_as_sudo () {
  if [[ $ARG_RUN_AS_SUDO -eq 1 ]]; then

    # Gets Sudoers file output into location given by user.
    f_log "SUDOERS FILE OUTPUT"
    sudo cat /etc/sudoers | tr -d '\000' | tee $USER_FILES_LOCATION/sudoers.$TIMESTAMP.txt
    echo "EXTRACTING SUDOERS FILE TO USER LOCATION.."
  else
    f_log "CANNOT EXTRACT SUDOERS FILE WITHOUT SUDO PERMISSION."
    f_log "To run as root/sudo user run script as: ./diagnostics_collection_linux.sh [-s|--sudo]"

  fi
}

# Run Diagnostics from Linux environment.
# The files are stored in location provided by user.
f_run_diagnostics () {
  USER_FILES_LOCATION=$FINAL_TARBALL_LOC/cohesityagent
  if [ ! -d "$USER_FILES_LOCATION" ]; then
  mkdir $USER_FILES_LOCATION
  fi

  (
  f_marker
  f_log "DIAGNOSTICS COLLECTION FROM LINUX ENVIRONMENT"
  f_separator

  # The service user, service group and installation directory.
  if [ -f /etc/init.d/cohesity-agent ]
  then
    f_get_vars_from_installed_service "/etc/init.d/cohesity-agent"
  else
    ps -ef | grep linux_agent_exec | awk '{print $18} {print $19}' | tee $USER_FILES_LOCATION/linux_agent_exec.$TIMESTAMP.log
    INSTALLED_AGENT_SERVICE_USER=`sed -n '3p' $USER_FILES_LOCATION/linux_agent_exec.$TIMESTAMP.log | awk -F'=' '{print $2}'`
    INSTALLED_AGENT_SERVICE_USER_GROUP=`sed -n '4p' $USER_FILES_LOCATION/linux_agent_exec.$TIMESTAMP.log | awk -F'=' '{print $2}'`
  fi

  if [ $? -ne 0 ]; then
    f_log "Error: Failed to find installation settings."
  fi

  f_log "SERVICE USER: $INSTALLED_AGENT_SERVICE_USER"
  f_separator
  f_log "SERVICE GROUP: $INSTALLED_AGENT_SERVICE_USER_GROUP"
  f_separator
  f_log "INSTALLATION DIR: $INSTALLED_AGENT_INSTALL_DIR"
  f_separator

  # Linux OS with version.
  OS_VERSION=`grep PRETTY_NAME /etc/os-release | awk -F'"' '{print $2}'`
  f_log "LINUX OS AND VERSION: $OS_VERSION"
  f_separator

  # Linux installer version.
  MY_SOFTWARE_VERSION=$(f_find_software_version $INSTALLED_AGENT_INSTALL_DIR/software/crux/bin/linux_agent_exec)
  f_log "SOFTWARE VERSION: $MY_SOFTWARE_VERSION"
  f_separator

  # To find out if IPv6 is supported.
  test -f /proc/net/if_inet6 && f_log "IPv6 SUPPORTED: Yes" || f_log "IPv6 SUPPORTED: No"
  f_separator

  # Terminal type permission for "Service user".
  TTY_PERMISSION=`w | grep $INSTALLED_AGENT_SERVICE_USER | awk ' { print $1"---->"$2 } '`
  f_log "TTY PERMISSION FOR SERVICE USER: $TTY_PERMISSION"
  f_separator

  # To find the Init System.
  if [[ `/sbin/init --version` =~ upstart ]]; then f_log "INIT SYSTEM: UPSTART";
  elif [[ `systemctl` =~ -\.mount ]]; then f_log "INIT SYSTEM: SYSTEMD";
  elif [[ -f /etc/init.d/cron && ! -h /etc/init.d/cron ]]; then f_log "INIT SYSTEM: SYSV-INIT";
  else f_log "INIT SYSTEM: UNABLE TO DETERMINE";
  fi
  f_separator

  # Determine if Linux Agent process running on system.
  # Get output of linux_agent_exec process commandline.
  LINUX_AGENT_EXEC_RUNNING=`ps -aux | grep linux_agent_exec | grep -v color=auto | awk -F'?' '{print $1}'`
  LINUX_PROCESS_RUNNING=`ps -ef | grep linux_agent_exec | grep -v color=auto | awk -F'?' '{print $1}'`
  if [ $? -ne 0 ]; then
      f_log "PROCESS STATUS: Not Running"
  else
    f_log "PROCESS STATUS: Running"
    echo "$LINUX_PROCESS_RUNNING"
  fi
  f_log "OUTPUT OF LINUX_AGENT_EXEC PROCESS COMMANDLINE:"
  echo "$LINUX_AGENT_EXEC_RUNNING"
  f_separator

  # Telnet Cohesity Cluster to Linux Agent machine with port 50051.
  HOST=127.0.0.1
  PORT_1=50051
  f_log "TELNET TO COHESITY CLUSTER:"
  echo >/dev/tcp/$HOST/$PORT_1 &>/dev/null && echo "Connection to $HOST on port $PORT_1 successful" || echo "Connection to $HOST on port $PORT_1 failed"

  # Telnet Cohesity Cluster to Linux Agent machine with port 59999.
  PORT_2=59999
  echo >/dev/tcp/$HOST/$PORT_2 &>/dev/null && echo "Connection to $HOST on port $PORT_2 successful" || echo "Connection to $HOST on port $PORT_2 failed"
  f_separator

  f_log "STATUS OF NFS SERVICES:"
  echo "`ps aux | grep nfsd | grep -v color=auto`"
  f_separator

  f_log "STATUS OF mountd:"
  echo "`ps aux | grep 'mountd' | grep -v color=auto`"
  f_separator

  # Version number of commands.
  f_print_command_versions
  f_separator

  # Listing mount options for /tmp directory.
  f_log "LISTING MOUNT TEMP DIRECTORY: `timeout 5 ls -lha /tmp`"
  f_separator

  # Get output of mount.
  f_log "OUTPUT OF MOUNT: `mount`"
  echo "`curl -XGET localhost:59999/mountz > $USER_FILES_LOCATION/mountz.html`"
  f_separator

  # gflags of magneto.
  f_log "MAGNETO GFLAGS:"
  echo "`curl -XGET localhost:59999/flagz > $USER_FILES_LOCATION/flagz.html`"
  f_separator

  # umask of service user.
  f_log "UMASK OF SERVICE USER: `umask`"
  f_separator

  # Get Cohesityagent logs in zipped tarball.
  f_log "COHESITYAGENT LOGS ZIPPED AS COHESITYAGENTLOG.TAR.GZ"
  tar --no-recursion -cvf $USER_FILES_LOCATION/cohesityagentlogfiles.$TIMESTAMP.tar.gz $INSTALLED_AGENT_INSTALL_DIR/data/logs/*
  f_separator

  # Get Cohesityagent config files in zipped tarball.
  f_log "COHESITYAGENT CONFIG ZIPPED AS COHESITYAGENTCONFIG.TAR.GZ"
  tar --no-recursion -cvf $USER_FILES_LOCATION/cohesityagentconfig.$TIMESTAMP.tar.gz $INSTALLED_AGENT_INSTALL_DIR/data/config/*
  f_separator

  f_run_as_sudo

  ) | tee $USER_FILES_LOCATION/cohesityagent-linuxenvironment.$TIMESTAMP.log

  f_marker
}

# Run Diagnostics collection for Oracle Adapter.
f_run_oracleadapter_diagnostics () {
  (

  f_log "DIAGNOSTICS COLLECTION FOR ORACLE ADAPTER"
  f_log "ORACLE HOME: $ORACLE_HOME"
  f_log "ORACLE SID: $ORACLE_SID"
  f_separator

  # Output of /etc/hosts.
  f_log "OUTPUT OF /ETC/HOSTS"
  echo "`cat /etc/hosts`"
  f_separator

  # Output of /etc/oratab.
  f_log "OUTPUT OF /ETC/ORATAB"
  echo "`cat /etc/oratab`"
  f_separator

  # Running which findmnt on oraclehost.
  f_log "WHICH FINDMNT"
  echo "`which findmnt`"
  f_separator

  # Running which orabase on oraclehost.
  f_log "WHICH ORABASE"
  echo "`which orabase`"
  f_separator

  # Permissions for Oracle.
  f_log "PERMISSIONS FOR ORACLE"
  echo "`ls -lh $ORACLE_HOME/bin/oracle`"
  f_separator

  # Running which sqlplus on oraclehost.
  f_log "WHICH SQLPLUS"
  echo "`which sqlplus`"

  # Permissions for sqlplus.
  f_log "PERMISSIONS FOR SQLPLUS"
  echo "`ls -lh $ORACLE_HOME/bin/sqlplus`"
  f_separator

  # Running which findmnt on oraclehost.
  f_log "WHICH RMAN"
  echo "`which rman`"

  # Permissions for rman.
  f_log "PERMISSIONS FOR RMAN"
  echo "`ls -lh $ORACLE_HOME/bin/rman`"
  f_separator

  # Copy rman_shell.INFO file into user given location.
  f_log "COPYING rman_shell.INFO INTO USER LOCATION"
  cp /tmp/rman_shell.INFO $USER_FILES_LOCATION

  ) | tee $USER_FILES_LOCATION/cohesityagent-oracleadapter.$TIMESTAMP.log
}

#------------------------------------------------------------------------------
#
# Execution starts here.
#
#------------------------------------------------------------------------------

# All command line arguments.
# Initialize all command line arguments here.

ARG_RUN_AS_SUDO=0

f_parse_arguments "$@"

[ -f $FINAL_TARBALL_LOC/diagnostics_linux.tar ] && rm -rf $FINAL_TARBALL_LOC/diagnostics_linux.tar

f_run_diagnostics

cat /etc/oratab > /dev/null
if [ $? -ne 0 ]; then
  f_log "ORACLE ADAPTER NOT PRESENT ON CURRENT SYSTEM."
else
  f_run_oracleadapter_diagnostics
fi
  f_marker

# A tarball is created with all the processed files.
tar -czvf $FINAL_TARBALL_LOC/diagnostics_linux.tar $USER_FILES_LOCATION
f_marker

# Delete all temporary files generated by the script.
rm -rf $USER_FILES_LOCATION