#!/bin/bash
#
# Copyright 2016 Cohesity Inc.
#
# Author: Sachin Gaikwad (sachin@cohesity.com)
#
# Description: This script mainly provides functionality to mount/unmount
#  VMDK volumes. Apart from this, it also provides several VMDK disk and
#  volume information. Please see usage of script with "--help".
#

# -------------------------- Utility Functions: Start ------------------------

# Arguments: $1 = flat file path
f_extract_sectorsize() {
  local ret_status=0
  local flat_file_path="$1"
  local sectorsize=`$FDISK -l "${flat_file_path}" | grep "Units:" | cut -d= -f2 | awk '{print $1}'`
  ret_status=$?
  echo $sectorsize
  return $ret_status
}

# Arguments: $1 == flat file path. $2 == partition number
f_extract_offset() {
  local ret_status=0

  # First get sectorsize for flat file path
  sector_size=`f_extract_sectorsize "$1"`
  ret_status=$?
  if [ "$ret_status" != 0 ]; then
    echo "ERROR: Failed to get sector size for $1"
    return $ret_status
  fi

  # Get number of blocks(sectors) from fdisk.
  # Replace '*' by ' ', there will be a '*' for bootable device.
  line=`$FDISK -l "${1}" | grep "${1}""${2} "`
  ret_status=$?
  line=`echo "${line}" | tr '*' ' '`
  line=`echo "${line}" | grep -o "\-flat.vmdk""${2}"" *[0-9]\+ *[0-9]\+ *[0-9]\+"`
  num_blocks=`echo "${line}" | awk '{print $2}'`

  echo $((num_blocks * sector_size))
  return $ret_status
}

# Arguments: $1 offset. $2 - flat file path.
f_extract_type() {
  local ret_status=0
  $BLKID -s TYPE -p -n ext2,ext3,ext4,xfs,ntfs,vfat,ReFS -O $1 "${2}" | sed 's/"//g' | sed 's/^.*TYPE=\| *//g'
  ret_status=$?
  return $ret_status
}

# -------------------------- Utility Functions: End -------------------------

# Arguments: $1 == mount point
f_do_unmount() {
  local ret_status=0
  local mountpoint="$1"
  # try -f first.
  for ((xx = 0;xx < 3;xx++)); do
    umount -f "${mountpoint}"
    ret_status=$?
    if [ "$ret_status" == "0" ]; then
      break;
    fi
  done
  if [ "$ret_status" != "0" ]; then
    #Print outout of lsof command to stderr.
    echo "unmount -f failed, printing output of lsof ""${mountpoint}"
    lsof -b -w "${mountpoint}"
    # -f has failed, try -l
    umount -lf "${mountpoint}"
    ret_status=$?
  fi
  return $ret_status
}

# Arguments $1 == path to vmdk
f_do_list() {
  local ret_status=0
  local flat_file_path="$1"
  echo line one
  echo line two
  $FDISK -l "${flat_file_path}" | grep "${flat_file_path}[0-9]\+" | sed 's/^.*-flat\.vmdk/ /g' | tr '*' ' '
  ret_status=$?
  return $ret_status
}

# Arguments $1 == path to vmdk
f_get_disk_uuid() {
  local ret_status=0
  local flat_file_path="$1"
  $FDISK -l "${flat_file_path}" | grep "Disk identifier:" | cut -d: -f2
  ret_status=$?
  return $ret_status
}

# Arguments: $1 == path to vmdk, $2 == partition number
f_get_fstype() {
  local ret_status=0
  local flat_file_path="$1"
  local partition="$2"
  if [ "${partition}" == "0" ]; then
    offset="0"
  else
    offset=`f_extract_offset "${flat_file_path}" "${partition}"`
  fi
  f_extract_type $offset "${flat_file_path}"
  ret_status=$?
  return $ret_status
}

# Arguments: $1 == path to vmdk
f_get_diskpartitiontype() {
  local ret_status=0
  local flat_file_path="$1"
  $FDISK -l "${flat_file_path}" | grep "Disklabel type:" | cut -d: -f2
  ret_status=$?
  return $ret_status
}

# Arguments: $1 == path to vmdk
f_get_sectorsize() {
  local ret_status=0
  local flat_file_path="$1"
  sector_size=`f_extract_sectorsize "$flat_file_path"`
  ret_status=$?
  if [ "$ret_status" != 0 ]; then
    echo "ERROR: Failed to get sector size for $flat_file_path"
    return $ret_status
  fi
  echo "$sector_size"
  return $ret_status
}

# Arguments: $1 == path to vmdk, $2 == fs_type,
#            $3 == mount options, $4 == mount point
f_do_mount() {
  local ret_status=0
  local flat_file_path="$1"
  local fs_type="$2"
  local mount_options="$3"
  local mountpoint="$4"

  if [ "x$mount_options" == "x" ]; then
    echo "ERROR: Mount options are empty. Please specify mount options."
    ret_status=1
    return $ret_status
  fi

  if [[ "$fs_type" == "ext4" || "$fs_type" == "ext3" || "$fs_type" == "ext2" ]]; then
    # RedHat 6.6 observation: If ext3 kernel module is loaded on system,
    # 'norecovery' option is not recognized by kernel module. So, forcing to use
    # ext4 subsystem instead. This works for ext3 filesystems as well.
    fs_type=ext4
  fi

  mount -t $fs_type -o $mount_options "${flat_file_path}" "${mountpoint}"
  ret_status=$?
  return $ret_status
}

f_do_unmount_all() {
  local ret_status=0
  local unmount_status=0
  IFS=$'\n'
  for x in `mount | grep "flat.vmdk"`
  do
    unmount_status=0
    mount_point=`echo "${x}" | grep -o "on .* type" | sed 's/on //g' | sed 's/ type//g'`
    f_do_unmount "${mount_point}"
    unmount_status=$?
    if [ $unmount_status -ne 0 ]; then
      echo "ERROR: Unmount of ${mount_point} failed"
      ret_status=$unmount_status
    fi
  done
  return $ret_status
}

# Arguments: $1 == path to vmdk, $2 == partition number
f_get_partition_offset() {
  local ret_status=0
  local flat_file_path="$1"
  local partition="$2"
  if [ "${partition}" == "0" ]; then
    offset="0"
  else
    offset=`f_extract_offset "${flat_file_path}" "${partition}"`
    ret_status=$?
  fi
  if [ "$ret_status" != "0" ]; then
    echo "ERROR: Failed to find offset for partition number $partition"
    return $ret_status
  fi
  echo $offset
  return $ret_status
}

f_print_usage() {
  local my_exit_status="$1"
  local bold=$(tput bold)
  local normal=$(tput sgr0)
  cat << EndOfMessage
Usage:
  ./$SCRIPT_NAME [-w|--work] [options]

Mandatory Args:
  -w, --work                What work do you want to do?
      ${bold}mount ${normal}
      ${bold}unmount ${normal}
      ${bold}unmount_all ${normal}
      ${bold}list ${normal}
      ${bold}get_disk_uuid ${normal}
      ${bold}get_partition_offset ${normal}
      ${bold}get_diskpartitiontype ${normal}
      ${bold}get_sectorsize ${normal}

Options:
  -f, --vmdk-file           VMDK flat file
  -p, --partition-number    Partition number
  -o, --options             Options to mount filesystem
  -F, --fs-type             Filesystem type
  -m, --mountpoint          Mountpoint for mount/unmount work
  -h, --help                Prints this help

EndOfMessage
  exit $my_exit_status
}

f_parse_arguments() {
  local SHORT_OPTIONS="w:f:p:o:F:m:h"
  local LONG_OPTIONS="work:,vmdk-file:,partition-number:,options:,fs-type:,mountpoint:,help"

  # Read the options.
  TEMP="$(getopt -o $SHORT_OPTIONS --long $LONG_OPTIONS -n "${SCRIPT_NAME}" -- "$@")"
  if [ $? -ne 0 ]; then
    f_print_usage 1
  fi

  eval set -- "$TEMP"
  unset TEMP

  # extract options and their arguments into variables.
  while true ; do
    case "$1" in
      -w|--work)
        ARG_WORK="$2" ; shift 2 ;;
      -f|--vmdk-file)
        ARG_VMDK_FILE="$2" ; shift 2 ;;
      -p|--partition-number)
        ARG_PARTITION_NUMBER="$2" ; shift 2 ;;
      -o|--options)
        ARG_MOUNT_OPTIONS="$2" ; shift 2 ;;
      -F|--fs-type)
        ARG_FS_TYPE="$2" ; shift 2 ;;
      -m|--mountpoint)
        ARG_MOUNTPOINT="$2" ; shift 2 ;;
      -h|--help)
        f_print_usage 0 ; shift ;;
      --) shift ; break ;;
    *) echo "Internal error!" ; f_print_usage 1 ;;
  esac
done
}

current_dir=$(dirname `readlink -f $0`)
. $current_dir/set_env.sh

export FDISK="${COHESITY_TOOLCHAIN_VERSION_DIR}/bin/fdisk"
if [ ! -f ${FDISK} ]; then
  echo "fdisk not found in toolchain path: ${FDISK}"
  exit 1
fi

export BLKID="${COHESITY_TOOLCHAIN_VERSION_DIR}/bin/blkid"
if [ ! -f ${BLKID} ]; then
  echo "blkid not found in toolchain path: ${BLKID}"
  exit 1
fi

# Script's exit status
export SCRIPT_EXIT_STATUS=0

# All command line argument variables are here.
ARG_WORK=""
ARG_VMDK_FILE=""
ARG_PARTITION_NUMBER=""
ARG_MOUNT_OPTIONS=""
ARG_FS_TYPE=""
ARG_MOUNTPOINT=""

SCRIPT_NAME="$(basename "$(test -L "$0" && readlink "$0" || echo "$0")")"
f_parse_arguments "$@"

case "$ARG_WORK" in
  mount)
    f_do_mount "$ARG_VMDK_FILE" "$ARG_FS_TYPE" "$ARG_MOUNT_OPTIONS" "$ARG_MOUNTPOINT"
    SCRIPT_EXIT_STATUS=$?
    ;;
  unmount)
    f_do_unmount "$ARG_MOUNTPOINT"
    SCRIPT_EXIT_STATUS=$?
    ;;
  unmount_all)
    f_do_unmount_all
    SCRIPT_EXIT_STATUS=$?
    ;;
  list)
    f_do_list "$ARG_VMDK_FILE"
    SCRIPT_EXIT_STATUS=$?
    ;;
  get_disk_uuid)
    f_get_disk_uuid "$ARG_VMDK_FILE"
    SCRIPT_EXIT_STATUS=$?
    ;;
  get_diskpartitiontype)
    f_get_diskpartitiontype "$ARG_VMDK_FILE"
    SCRIPT_EXIT_STATUS=$?
    ;;
  get_sectorsize)
    f_get_sectorsize "$ARG_VMDK_FILE"
    SCRIPT_EXIT_STATUS=$?
    ;;
  get_partition_offset)
    f_get_partition_offset "$ARG_VMDK_FILE" "$ARG_PARTITION_NUMBER"
    SCRIPT_EXIT_STATUS=$?
    ;;
  *)
    echo "ERROR: Unknown work $ARG_WORK specified."
    f_print_usage 1
    ;;
esac
exit $SCRIPT_EXIT_STATUS
